#include <openssl/rsa.h>
#include <openssl/err.h>
#include <string.h>
#include <qstring.h>
#include <stdio.h>
#include "hash.h"
#include "crypt_rsa.h"

bool generate_key_pair(QString &pubkey, QString &privkey)
{
    RSA *keys=NULL;
    unsigned char *pubkey_bin=NULL;
    int pubkey_size=0;
    unsigned char *privkey_bin=NULL;
    int privkey_size=0;
    char temp[3];
    int i;

    keys = RSA_generate_key(1024, RSA_F4,NULL, (char *)stdout);
    if (keys==NULL)
      return false;

    pubkey_size = i2d_RSAPublicKey(keys,&pubkey_bin);
    privkey_size = i2d_RSAPrivateKey(keys,&privkey_bin);

    pubkey = "";
    privkey = "";


    for (i=0;i<pubkey_size;i++)
      {
        sprintf(temp,"%02x",(unsigned char)pubkey_bin[i]);
        pubkey.append(temp);
      }
    for (i=0;i<privkey_size;i++)
      {
        sprintf(temp,"%02x",(unsigned char)privkey_bin[i]);
        privkey.append(temp);
      }

    RSA_free(keys);
    return true;

}

bool check_key_pair(const QString &pubkey, const QString &privkey)
{
  int privkey_size=0;
  int pubkey_size=0;
  unsigned char *pubkey_bin;
  unsigned char *privkey_bin;
  unsigned char *pubkey_bin_bak;
  unsigned char *privkey_bin_bak;
  unsigned int t;
  char tmp[3];
  RSA *keys;

  tmp[2]=0;

  privkey_size = privkey.length()/2;
  pubkey_size = pubkey.length()/2;

  pubkey_bin = (unsigned char *)malloc(pubkey_size);
  privkey_bin = (unsigned char *)malloc(privkey_size);
  privkey_bin_bak = privkey_bin;
  pubkey_bin_bak = pubkey_bin;

  for (int i=0;i<pubkey_size;i++)
    {
      strncpy(tmp,pubkey.mid(2*i,2).latin1(),2);
      sscanf(tmp,"%02X",&t);
      pubkey_bin[i]=(unsigned char)t;
    }

  for (int i=0;i<privkey_size;i++)
    {
      strncpy(tmp,privkey.mid(2*i,2).latin1(),2);
      sscanf(tmp,"%02X",&t);
      privkey_bin[i]=(unsigned char)t;
    }

  keys = RSA_new();
  if (d2i_RSAPublicKey(&keys,(const unsigned char **) &pubkey_bin, pubkey_size)==NULL)
    {
      qDebug("[check_key_pair] - Cle publique pas cool...");
      RSA_free(keys);
      free(pubkey_bin_bak);
      free(privkey_bin_bak);
      return false;
    }
  if (d2i_RSAPrivateKey(&keys,(const unsigned char **) &privkey_bin, privkey_size)==NULL)
    {
      qDebug("[check_key_pair] - Cle privee pas cool...");
      RSA_free(keys);
      free(pubkey_bin_bak);
      free(privkey_bin_bak);
      return false;
    }
  if (RSA_check_key(keys)==1)
    {
      RSA_free(keys);
      free(pubkey_bin_bak);
      free(privkey_bin_bak);
      return true;
    }
  else
    {
      RSA_free(keys);
      free(pubkey_bin_bak);
      free(privkey_bin_bak);
      return false;
    }
}

QString generate_nodeid(const QString &privkey, const QString &pubkey)
{
  QString temp = "";
  unsigned char *hash_bin;
  char tmp[3];
  int i;


  hash_bin = (unsigned char *)malloc(MD5_DIGEST_LENGTH);

  temp.append(privkey);
  temp.append(pubkey);

  md5_buffer_hash((const unsigned char *)temp.latin1(),temp.length(),hash_bin);

  temp="";
  for (i=0;i<MD5_DIGEST_LENGTH;i++)
    {
      sprintf(tmp,"%02x",hash_bin[i]);
      temp.append(tmp);
    }

  free(hash_bin);
  return temp.left(25);

}

const QString rsa_encrypt_printable(const unsigned char * data, unsigned int len, const QString &pubkey)
{
  int pubkey_size=0;
  unsigned char *pubkey_bin;
  unsigned char *pubkey_bin_bak;
  unsigned char *crypted_bin;
  unsigned int t;
  char tmp[3];
  RSA *keys;
  QString res = "";

  tmp[2]=0;

  pubkey_size = pubkey.length()/2;

  pubkey_bin = (unsigned char *)malloc(pubkey_size);
  pubkey_bin_bak = pubkey_bin;

  for (int i=0;i<pubkey_size;i++)
    {
      strncpy(tmp,pubkey.mid(2*i,2).latin1(),2);
      sscanf(tmp,"%02X",&t);
      pubkey_bin[i]=(unsigned char)t;
    }

  keys = RSA_new();
  if (d2i_RSAPublicKey(&keys,(const unsigned char **) &pubkey_bin, pubkey_size)==NULL)
    {
      qDebug("[check_key_pair] - Cle publique pas cool...");
      free(pubkey_bin_bak);
      RSA_free(keys);
      return res;
    }

  crypted_bin = (unsigned char *)malloc(RSA_size(keys));
  RSA_public_encrypt(len,data,crypted_bin,keys,RSA_PKCS1_PADDING);

  res = buffer_bin_to_hex(crypted_bin,RSA_size(keys));

  RSA_free(keys);
  free(pubkey_bin_bak);
  free(crypted_bin);

  return res;
}

bool rsa_printable_decrypt(unsigned char * res, const QString &data, const QString &privkey)
{
  int privkey_size=0;
  unsigned char *privkey_bin;
  unsigned char *privkey_bin_bak;
  unsigned char *crypted_bin;
  unsigned int t,data_size;
  char tmp[3];
  RSA *keys;

  tmp[2]=0;

  privkey_size = privkey.length()/2;

  privkey_bin = (unsigned char *)malloc(privkey_size);
  privkey_bin_bak = privkey_bin;

  for (int i=0;i<privkey_size;i++)
    {
      strncpy(tmp,privkey.mid(2*i,2).latin1(),2);
      sscanf(tmp,"%02X",&t);
      privkey_bin[i]=(unsigned char)t;
    }

  keys = RSA_new();
  if (d2i_RSAPrivateKey(&keys,(const unsigned char **) &privkey_bin, privkey_size)==NULL)
    {
      qDebug("[check_key_pair] - Cle privee pas cool...");
      free(privkey_bin_bak);
      RSA_free(keys);
      return false;
    }

  data_size = data.simplifyWhiteSpace().length()/2;
  crypted_bin = (unsigned char *)malloc(data_size);

  for (unsigned int i=0;i<data_size;i++)
    {
      strncpy(tmp,data.mid(2*i,2).latin1(),2);
      sscanf(tmp,"%02X",&t);
      crypted_bin[i]=(unsigned char)t;
    }


  for (unsigned int i=0;i<data_size;i++)
    {
      fprintf(stderr,"%02X",crypted_bin[i]);
    }
  fprintf(stderr,"\n");



  qDebug("Decryptage de {%s}",data.simplifyWhiteSpace().latin1());
  int z = RSA_private_decrypt(data_size,crypted_bin,res,keys,RSA_PKCS1_PADDING);
  qDebug("Taille decryptee : %d",z);
  if (z==-1)
    {
      qDebug("Erreur : %lu",ERR_get_error());
      return false;
    }
  RSA_free(keys);
  free(privkey_bin_bak);
  free(crypted_bin);
  return true;
}

void buffer_hex_to_bin(const char *in, unsigned char *out)
{
	unsigned int t;
	while (*in && sscanf(in,"%2x",&t))
	{
		*(out++)=t;
		in+=2;
	}
}

/* convertit du binaire en hexa ascii
*  in de longueur len
*  out doit etre malloc(len*2+1)
*/
QString buffer_bin_to_hex(const unsigned char *in, unsigned int len)

{
	len *= 2;
	char *out=new char[len+1];
	for(unsigned int i=0;i<len;i+=2)
		sprintf(out+i,"%02x",*(in++));
	QString res=out;
	delete [] out;
	return res;

}
