#include <qsocket.h>
#include <string.h>
#include <qdatetime.h>
#include "gcryptsocket.h"
#include "crypt_des.h"
#include "crypt_rsa.h"
#include "hash.h"

extern QString MyPrivKey;

GCryptSocket::GCryptSocket(DES_key_schedule key_schedule, QObject * parent, const char * name)
  : QSocket( parent, name )
{
  crypted_buffer=NULL;
  crypted_buffer_size=0;
  decrypted_buffer=NULL;
  decrypted_buffer_size=0;
  remaining_data=0;
  key = key_schedule;
  connect(this,SIGNAL(readyRead()),this,SLOT(newFirstCryptedData()));
  connect(this,SIGNAL(connected()),this,SLOT(outgoingSocket()));
}


GCryptSocket::GCryptSocket(DES_key_schedule key_schedule, socket_type t, QObject * parent, const char * name)
  : QSocket( parent, name )
{
  type=t;
  crypted_buffer=NULL;
  crypted_buffer_size=0;
  decrypted_buffer=NULL;
  decrypted_buffer_size=0;

  remaining_data=0;
  key = key_schedule;
  if (type == GSOCKET)
    {
      connect(this,SIGNAL(connected()),this,SLOT(outgoingSocket()));
    }
  else
    {
      connect(this,SIGNAL(readyRead()),this,SLOT(newBinCryptedData()));
    }

}

GCryptSocket::~GCryptSocket()
{
  if (crypted_buffer)
    free(crypted_buffer);
}

void GCryptSocket::outgoingSocket()
{
  disconnect(this, SIGNAL(readyRead()),0,0);
  connect(this, SIGNAL(readyRead()), this, SLOT(newCryptedData()));
}

void GCryptSocket::newFirstCryptedData()
{
  Q_ULONG data_size, real_size;
  QString temp_decrypted_buffer;
  char *temp_block;
  char *temp_pointer;
  bool complete=false;
  unsigned int i,j=0;

  qDebug("NewFirstCryptedData");
  data_size = bytesAvailable();
  crypted_buffer = (char *)realloc(crypted_buffer, crypted_buffer_size + data_size);
  real_size = readBlock(crypted_buffer + crypted_buffer_size, data_size);
  crypted_buffer = (char *)realloc(crypted_buffer, crypted_buffer_size + real_size);
  crypted_buffer_size += real_size;
  temp_block = (char *)malloc(9);
  temp_block[8]=0;
  while (crypted_buffer_size>=8)
    {
      des_decrypt_buff(crypted_buffer, temp_block, 8, key);

      temp_pointer = (char *)malloc(crypted_buffer_size-8);
      memcpy(temp_pointer, crypted_buffer+8, crypted_buffer_size-8);
      free(crypted_buffer);
      crypted_buffer = temp_pointer;
      crypted_buffer_size -= 8;
      decrypted_text_buffer += temp_block;
      if (decrypted_text_buffer.contains('\n'))
        {
          qDebug("Premier paquet complet, reste %s octets dans le buffer crypte",QString::number(crypted_buffer_size).latin1());
          complete=true;
          break;
        }
    }
  free(temp_block);

  if (complete)
    {
      if (decrypted_text_buffer.left(7)=="GSOCKET")
	{
	  qDebug("Je suis un GSOCKET");
	  disconnect(this, SIGNAL(readyRead()),0,0);
	  connect(this, SIGNAL(readyRead()), this, SLOT(newCryptedData()));
	  decrypted_text_buffer="";

	  type = GSOCKET;

          if (crypted_buffer_size>0)
	    emit(readyRead());
	}
      else if (decrypted_text_buffer.left(7) == "GULSOCK")
	{
	  qDebug("Je suis un socket de DL");
          qDebug("Cle cryptee recue : %s",decrypted_text_buffer.section(' ',1,1).simplifyWhiteSpace().latin1());

          DES_cblock sessionkey;
          if (!rsa_printable_decrypt(sessionkey,decrypted_text_buffer.section(' ',1,1).simplifyWhiteSpace().latin1(),gconf->myPrivKey()))
            {
              close();
              return;
            }
          qDebug("Cle de session : %s",buffer_bin_to_hex(sessionkey,8).latin1());
	  decrypted_lines.clear();
	  disconnect(this, SIGNAL(readyRead()),0,0);
	  connect(this, SIGNAL(readyRead()), this, SLOT(newBinCryptedData()));
	  type=GDLSOCK;

          for (i=crypted_buffer_size-1;i!=0;i--)
            {
              ungetch(crypted_buffer[i]);
              j++;
            }
          if (crypted_buffer_size>0)
            ungetch(crypted_buffer[0]);

          qDebug("J'ai remis %d octets dans le buffer du socket",j+1);
          crypted_buffer_size=0;
          free(crypted_buffer);
          crypted_buffer = NULL;

          setCryptKey(&sessionkey);

          emit(readyRead());
	  newBinCryptedData();

	}
      else if (decrypted_text_buffer.left(7) == "GDLSOCK")
	{
	  qDebug("Je suis un socket d'UL");
          qDebug("Cle cryptee recue : %s",decrypted_text_buffer.section(' ',1,1).simplifyWhiteSpace().latin1());

          DES_cblock sessionkey;
          if (!rsa_printable_decrypt(sessionkey,decrypted_text_buffer.section(' ',1,1).simplifyWhiteSpace().latin1(),gconf->myPrivKey()))
            {
              close();
              return;
            }
          qDebug("Cle de session : %s",buffer_bin_to_hex(sessionkey,8).latin1());
	  decrypted_lines.clear();
	  disconnect(this, SIGNAL(readyRead()),0,0);
	  connect(this, SIGNAL(readyRead()), this, SLOT(newBinCryptedData()));
	  type=GULSOCK;

          for (i=crypted_buffer_size-1;i!=0;i--)
            {
              ungetch(crypted_buffer[i]);
              j++;
            }
          if (crypted_buffer_size>0)
            ungetch(crypted_buffer[0]);
          qDebug("J'ai remis %d octets dans le buffer du socket",j+1);

          crypted_buffer_size=0;
          free(crypted_buffer);
          crypted_buffer = NULL;


          setCryptKey(&sessionkey);

          emit(readyRead());
	  newBinCryptedData();

	}
      else
	{
	  decrypted_text_buffer="";
	  disconnect(this, SIGNAL(readyRead()),0,0);
	  //Socket pourri
	  qWarning("Tentative d'entree sur le reseau sans cle! socket rejette");
	  close();
	}
    }
}



void GCryptSocket::newCryptedData()
{
  Q_ULONG data_size, real_size;
  QString temp_decrypted_buffer;
  QStringList new_lines;
  char *temp_block;
  char *temp_pointer;

  data_size = bytesAvailable();
  crypted_buffer = (char *)realloc(crypted_buffer, crypted_buffer_size + data_size);
  real_size = readBlock(crypted_buffer + crypted_buffer_size, data_size);
  crypted_buffer = (char *)realloc(crypted_buffer, crypted_buffer_size + real_size);
  crypted_buffer_size += real_size;
  temp_block = (char *)malloc(9);
  temp_block[8]=0;
  while (crypted_buffer_size>=8)
    {
      des_decrypt_buff(crypted_buffer, temp_block, 8, key);

      temp_pointer = (char *)malloc(crypted_buffer_size-8);
      memcpy(temp_pointer, crypted_buffer+8, crypted_buffer_size-8);
      free(crypted_buffer);
      crypted_buffer = temp_pointer;
      crypted_buffer_size -= 8;

      decrypted_text_buffer += temp_block;
    }
  free(temp_block);

  if (decrypted_text_buffer.contains('\n'))
    new_lines = QStringList::split( '\n', decrypted_text_buffer );
  decrypted_lines += new_lines;
  decrypted_text_buffer = decrypted_text_buffer.section( '\n', -1 );
  if (!(decrypted_lines.isEmpty()))
    emit readyDecryptedRead();
}

void GCryptSocket::newBinCryptedData()
{
  unsigned int readable_data;
  unsigned int size_to_read;
  char *temp_block;

  readable_data = bytesAvailable();
  if (remaining_data==0)
    {
      if (readable_data>=sizeof(Q_ULONG))
	{
	  readBlock((char *)&remaining_data,sizeof(Q_ULONG));
	  valid_size = remaining_data;
	  if (remaining_data % 8 != 0)
	    remaining_data = remaining_data + (8 - (remaining_data % 8));
	}
    }

  while (bytesAvailable()>0)
    {
      readable_data = bytesAvailable();
      while ((remaining_data>0) && (readable_data>0))
	{
	  if (remaining_data<=readable_data)
	    size_to_read=remaining_data;
	  else
	    size_to_read = readable_data;
	  crypted_buffer = (char *)realloc(crypted_buffer, crypted_buffer_size + size_to_read);
	  readBlock(crypted_buffer+crypted_buffer_size, size_to_read);
	  crypted_buffer_size += size_to_read;
	  remaining_data -= size_to_read;

	  readable_data = bytesAvailable();

	}

      if (remaining_data==0)
	{

	  temp_block = (char *)malloc(crypted_buffer_size);
	  des_decrypt_buff(crypted_buffer, temp_block, crypted_buffer_size, key);
	  decrypted_buffer = (char *)realloc(decrypted_buffer, decrypted_buffer_size + valid_size);
	  memcpy(decrypted_buffer+decrypted_buffer_size,temp_block, valid_size);
	  decrypted_buffer_size += valid_size;
	  free(crypted_buffer);
	  crypted_buffer_size = 0;
	  crypted_buffer = NULL;
	  free(temp_block);
	  emit(readyDecryptedRead());
	  break;
	}
    }
  if (bytesAvailable()>0)
    {
      emit(readyRead());
    }
}

bool GCryptSocket::canReadCryptedLine()
{
  return !(decrypted_lines.isEmpty());
}

QString GCryptSocket::readCryptedLine()
{
  QString res="";
  if (decrypted_lines.isEmpty())
    {
      return res;
    }
  else
    {
      res = decrypted_lines.first();
      decrypted_lines.pop_front();
      return res;
    }
}

Q_ULONG GCryptSocket::decryptedBytesAvailable()
{
  return decrypted_buffer_size;
}

void GCryptSocket::writeCryptedLine(QString data)
{
  char *buffer;
  char *normalized_buffer;
  char *crypted_buffer;
  unsigned int crypted_buffer_size;

  data += '\n';
  buffer = (char *)malloc(data.length()+1);
  memcpy(buffer,(char *)data.latin1(),data.length());
  buffer[data.length()]=0;
  crypted_buffer_size = normalize_buffer_size(buffer,&normalized_buffer,strlen(buffer));
  crypted_buffer = (char *)malloc(crypted_buffer_size);
  des_crypt_buff(normalized_buffer, crypted_buffer, crypted_buffer_size, key);
  writeBlock(crypted_buffer, crypted_buffer_size);
  free(buffer);
  free(normalized_buffer);
  free(crypted_buffer);
}

Q_LONG GCryptSocket::readCryptedBlock(char *data, Q_ULONG maxlen)
{

  Q_LONG real_size;
  if (decrypted_buffer_size<= maxlen)
    {
      real_size = decrypted_buffer_size;
    }
  else
    {
      real_size = maxlen;
    }

  memcpy(data, decrypted_buffer, real_size);


  memmove(decrypted_buffer, decrypted_buffer + real_size, decrypted_buffer_size-real_size);
  decrypted_buffer = (char *)realloc(decrypted_buffer, decrypted_buffer_size-real_size);
  decrypted_buffer_size -= real_size;
  return real_size;
}

int GCryptSocket::decryptedGetch()
{
  unsigned char res;
  if (decrypted_buffer_size>=1)
    {
      res = *decrypted_buffer;
      memmove(decrypted_buffer, decrypted_buffer+1, --decrypted_buffer_size);
      decrypted_buffer = (char *)realloc(decrypted_buffer,decrypted_buffer_size);
      return res;
    }
  else
    return -1;
}

int GCryptSocket::decryptedUngetch(int ch)
{
  decrypted_buffer = (char *)realloc(decrypted_buffer,decrypted_buffer_size+1);
  memmove(decrypted_buffer+1,decrypted_buffer,decrypted_buffer_size++);
  (*decrypted_buffer) = (char) ch;
  return ch;
}

int GCryptSocket::putCryptedCh(int ch)
{
  writeCryptedBlock((char*)&ch,1);
  return ch;
}

Q_LONG GCryptSocket::writeCryptedBlock(const char *data, Q_ULONG len)
{
  char *temp;
  char *normalized_buffer;
  unsigned int crypted_buffer_size;

  temp = (char *)malloc(sizeof(Q_ULONG));
  memcpy(temp,(char *)&len,sizeof(Q_ULONG));
  crypted_buffer_size = normalize_buffer_size(data,&normalized_buffer,len);
  temp = (char *)realloc(temp,crypted_buffer_size + sizeof(Q_ULONG));
  des_crypt_buff(normalized_buffer, temp+sizeof(Q_ULONG), crypted_buffer_size, key);
  writeBlock(temp,crypted_buffer_size+sizeof(Q_ULONG));
  free(normalized_buffer);
  free(temp);
  return len;

}

void GCryptSocket::setCryptKey(const_DES_cblock *key_data)
{
  DES_set_key(key_data,&key);
}
