#ifndef _GCRYPTSOCKET_H_
#define _GCRYPTSOCKET_H_

#include <qsocket.h>
#include <qstring.h>
#include <qstringlist.h>
#include <openssl/des.h>
#include "../gconfig.h"

extern GConfig * gconf;

typedef enum socket_type {GSOCKET, GULSOCK, GDLSOCK} socket_type;

class GCryptSocket : public QSocket
{
  Q_OBJECT
 public:
	GCryptSocket( DES_key_schedule key_schedule, QObject * parent = 0, const char * name = 0 );
  GCryptSocket( DES_key_schedule key_schedule, socket_type t, QObject * parent = 0, const char * name = 0 );
  ~GCryptSocket();
  QString readCryptedLine();
  void writeCryptedLine(QString data);
  bool canReadCryptedLine();
  Q_LONG readCryptedBlock(char *, Q_ULONG);
  Q_LONG writeCryptedBlock(const char *, Q_ULONG);
  int decryptedGetch();
  int putCryptedCh(int ch);
  int decryptedUngetch(int ch);
  Q_ULONG decryptedBytesAvailable();
  void setCryptKey(const_DES_cblock *);

  socket_type type;


 private:
  char *crypted_buffer;
  char *decrypted_buffer;
  Q_ULONG crypted_buffer_size;
  Q_ULONG decrypted_buffer_size;
  Q_ULONG remaining_data;
  Q_ULONG valid_size;
  DES_key_schedule key;
  QString decrypted_text_buffer;
  QStringList decrypted_lines;

 private slots:
  void newFirstCryptedData();
  void newCryptedData();
  void newBinCryptedData();
  void outgoingSocket();

 signals:
  void readyDecryptedRead();

};

#endif
