/***************************************************************************
 *            gdownloader.cpp
 *
 *  Sun May  8 11:54:43 2005
 *  Copyright  2005  CrYpToNyM
 *  crypt0nym@sf.net
 ****************************************************************************/

#include <qdir.h>
#include "gdownloader.h"
#include "gtempfile.h"
#include "gfileconst.h"
#include "ghashfile.h"

GDownloader::GDownloader(const QString &tempdir,const QString &incoming,QObject * parent,const char * name)
	:QObject(parent,name)
{
	tempDir=tempdir;
	incomingDir=incoming;
}

bool GDownloader::isDownloading(Q_UINT64 filesize,const QString &hash)
{
	QValueVector <GTempFile*>::iterator it;
	for( it = tempfileVector.begin();it != tempfileVector.end(); ++it)
	{
		if ((*it)->fileSize()==filesize && (*it)->fileHash()==hash)
			return true;
	}
	return false;
}

void GDownloader::getFileList(Q_UINT16 uid,const QString &nodeid,Q_UINT64 filesize,const QString &hash)
{
	unsigned int i;
	QString fname,filename;
	QStringList files;
	QBitArray part;
	GHashFile hasher;
	QDir dir(tempDir);

	if (isDownloading(filesize,hash))
		return;

	filename="filelist_gloster."+nodeid+".xml";
	
	if (hasher.readFromFile(incomingDir+filename) && hasher.getFileHash()==hash)
		emit finishedList(incomingDir+filename);
	else
	{
		
		files=dir.entryList("[0-9][0-9][0-9].part",QDir::All,QDir::Name | QDir::IgnoreCase);
		i=1;
		fname="001.part";
		for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it )
		{
			if ((*it) != fname)
				break;
			else
			{
				fname=QString::number(++i);
				while (fname.length()<3)
					fname='0'+fname;
				fname+=".part";
			}
		}

		dlingFiles.resize(dlingFiles.size()+1);
		dlingFiles[dlingFiles.size()-1].tickCount=0;

		qDebug("new GTempFile("+tempDir+fname+","+filename+",0,"+QString::number(filesize)+","+hash+",this,"+nodeid+")");
		tempfileVector.append(new GTempFile(tempDir+fname,filename,0,filesize,hash,this,nodeid));
		tempfileVector[(int)(tempfileVector.count()-1)]->newSource(nodeid,uid);
		connectGTempFile(tempfileVector[(int)(tempfileVector.count()-1)]);
		part.resize((filesize+PART_SIZE-1)/PART_SIZE);
		part.fill(false);
		emit tempLoaded(filename,filesize,hash,part);
	}
}

void GDownloader::loadTempFiles()
{
	char p;
	int k;
	unsigned int i;
	Q_UINT32 chunk,c;
	GTempFile *g;
	s_dlingFile dlfile;

	QBitArray part_array;
	QStringList files;
	QDir dir(tempDir,"[0-9][0-9][0-9].part",QDir::Name | QDir::IgnoreCase,QDir::Files|QDir::NoSymLinks|QDir::Readable|QDir::Writable);


	timer = new QTimer(this);
	connect(timer,SIGNAL(timeout()),this,SLOT(onTick()));
	timer->start(1000*15); // 15secondes avant la recherche des sources




	
	files=dir.entryList();
	
		/* init dlingFiles */
	dlfile.tickCount=254;
	dlingFiles.resize(files.count(),dlfile);

	for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it )
	{
		if (!QFile::exists(tempDir+(*it)+".xml"))
			continue;
		tempfileVector.append(g = new GTempFile(tempDir+(*it),this));

		chunk=g->nbChunk();
		i=g->fileSize() - (chunk-1)*CHUNK_SIZE;
		i=(i+PART_SIZE-1)/PART_SIZE;
		part_array.resize(chunk*PART_BY_CHUNK+i);

		for (c=0,k=0;c<chunk-1;++c)
		{
			for (p=0;p<PART_BY_CHUNK;++p,++k)
			{
				part_array[k]=g->getPartState(c,p);
			}

		}

		for (p=0;p<(char)i;++p,++k)
			part_array[k]=g->getPartState(c,p);

		connectGTempFile(tempfileVector[(int)(tempfileVector.count()-1)]);
		emit tempLoaded(tempfileVector[(int)(tempfileVector.count()-1)]->fileName(),tempfileVector[(int)(tempfileVector.count()-1)]->fileSize(),tempfileVector[tempfileVector.count()-1]->fileHash(),part_array);
	}
}
	




void GDownloader::dlSpeed(Q_UINT64 filesize,const QString& hash, Q_UINT64 received, const QValueList<GSrcSpeed> &srclist)
{
	emit downloadSpeed(filesize,hash,received,srclist);
}


void GDownloader::download(const QString &filename,Q_UINT64 filesize,const QString &hash)
{
	unsigned int i;
	QString fname;
	QStringList files;
	QBitArray part;
	QDir dir(tempDir);

	if (isDownloading(filesize,hash))
		return;

	files=dir.entryList("[0-9][0-9][0-9].part",QDir::All,QDir::Name | QDir::IgnoreCase);
	i=1;
	fname="001.part";
	for ( QStringList::Iterator it = files.begin(); it != files.end(); ++it )
	{
		if ((*it) != fname)
			break;
		else
		{
			fname=QString::number(++i);
			while (fname.length()<3)
				fname='0'+fname;
			fname+=".part";
		}
	}

	dlingFiles.resize(dlingFiles.size()+1);
	dlingFiles[dlingFiles.size()-1].tickCount=0;

	tempfileVector.append(new GTempFile(tempDir+fname,filename,0,filesize,hash,this));
	connectGTempFile(tempfileVector[(int)(tempfileVector.count()-1)]);
	part.resize((filesize+PART_SIZE-1)/PART_SIZE);
	part.fill(false);
	emit tempLoaded(filename,filesize,hash,part);
	emit askSources(filesize,hash); //On en lance quand meme un pour les impatients
}


void GDownloader::cancel(Q_UINT64 filesize,const QString &hash)
{
	QString tmpfile;
	QValueVector <GTempFile*>::iterator i;
	QDir d(tempDir);
	
	for (i = tempfileVector.begin(); i != tempfileVector.end(); ++i)
	{
		if ((*i)->fileSize()==filesize && (*i)->fileHash()==hash)
			break ;
	}
	if (i==tempfileVector.end())
		return ;
	tmpfile = (*i)->tempFileName();
	delete (*i);

	qDebug("GDownloader::cancel(%llu,%s)",filesize,hash.latin1());

	if (d.remove(tmpfile + ".xml"))
	{
		QValueVector <GTempFile*>::iterator it;
		QValueVector <s_dlingFile>::iterator it2;
		for( it = tempfileVector.begin(),it2 = dlingFiles.begin(); it != tempfileVector.end(); ++it, ++it2 )
		{
			if (*it==sender())
			{
				dlingFiles.erase(it2);
				tempfileVector.erase(it);
				break;
			}
		}
		d.remove( tmpfile );
	}
	else
		qWarning("impossible de supprimer le fichier!!! " + tmpfile + ".xml");
}

void GDownloader::pause(Q_UINT64 filesize,const QString &hash)
{
	qDebug("GDownloader::pause(%llu,%s) - aucune action pour le moment",filesize,hash.latin1());
}

void GDownloader::resume(Q_UINT64 filesize,const QString &hash)
{
	qDebug("GDownloader::resume(%llu,%s) - aucune action pour le moment",filesize,hash.latin1());
}
void GDownloader::setPriority(Q_UINT64 filesize,const QString &hash,unsigned char p)
{
	unsigned int i;

	qDebug("GDownloader::setPriority(%llu,%s,%i)",filesize,hash.latin1(),p);
	for (i=0;i<tempfileVector.count();++i)
	{
		if ((tempfileVector[i]->fileSize()==filesize) && (tempfileVector[i]->fileHash()==hash))
		{
			tempfileVector[i]->setPriority(p);
			break;
		}
	}
}

void GDownloader::onTick()
{
	unsigned int i;
	
	timer->changeInterval ( 1000*60*5 ); //10min
	//On vire tout ce qui etait en attente
	waitingSrc.clear();

	for (i=0;i<dlingFiles.size();++i)
	{
		if (++dlingFiles[i].tickCount>=tempfileVector[i]->priority())
		{
			dlingFiles[i].tickCount=0;
			if (tempfileVector[i]->nodeId().isEmpty())
				emit askSources(tempfileVector[i]->fileSize(),tempfileVector[i]->fileHash());
			else
				tempfileVector[i]->newSource(tempfileVector[i]->nodeId());
		}
	}
}


void GDownloader::addSource(Q_UINT16 uid,const QString &nodeid, const QString &,Q_UINT64 filesize,const QString &hash)
{
	unsigned int i;
	for (i=0;i<tempfileVector.count();++i)
	{
		if ((tempfileVector[i]->fileSize()==filesize) && (tempfileVector[i]->fileHash()==hash))
		{
			qDebug("GDownloader::addSource @ tempfileVector[%i]->newSource(%s)",i,nodeid.latin1());
			tempfileVector[i]->newSource(nodeid,uid);
			break;
		}
	}
}

void GDownloader::incomingSocket(const QString &nodeid, GCryptSocket *s)
{
	/* envoi au bon GTempFile */
	t_waitingList::iterator it;
	for (it=waitingSrc.begin(); it!=waitingSrc.end(); ++it)
	{
		if ((*it).nodeid == nodeid)
		{
				(*it).gtempf->newDlSocket(s);
				waitingSrc.remove(it);
				return;
		}
	}
	delete s; // si on est arrive au bout de la liste alors on l'attendait pas!
}

void GDownloader::unableToConnect(const QString &nodeid)
{
	s_waiting_src t;

	//On l'ajoute a la fin de la liste
	t.nodeid=nodeid;
	t.gtempf=(GTempFile *)sender();
	waitingSrc.append(t);

	emit askConnect(nodeid);
}


bool copyFile(const QString &src, const QString &dst)
{
	QFile fsrc(src);
	QFile fdst(dst);
	if( ! fsrc.open( IO_ReadOnly | IO_Raw ) ) return false;
	if( ! fdst.open( IO_WriteOnly | IO_Raw ) ) return false;
	int  r = 0;
	char buffer[4096];
	while( ( r = fsrc.readBlock(buffer, 4096) ) > 0 )
		if( r != fdst.writeBlock(buffer, r) )
			return false;
	fsrc.close();
	fsrc.remove();
	return true;
}


void GDownloader::transferFinished()
{
	QDir d(tempDir);
	QFile f(((GTempFile *)sender())->tempFileName()+".xml");
	if (!((GTempFile *)sender())->nodeId().isEmpty())
		d.remove(incomingDir + ((GTempFile *)sender())->fileName());
	if ( (d.rename( ((GTempFile *)sender())->tempFileName() , incomingDir + ((GTempFile *)sender())->fileName() ))
		|| (copyFile(((GTempFile *)sender())->tempFileName() , incomingDir +((GTempFile *)sender())->fileName())) )
	{
		QValueVector <GTempFile*>::iterator it;
		QValueVector <s_dlingFile>::iterator it2;
		for( it = tempfileVector.begin(),it2 = dlingFiles.begin(); it != tempfileVector.end(); ++it, ++it2 )
		{
			if (*it==sender())
			{
				dlingFiles.erase(it2);
				tempfileVector.erase(it);
				break;
			}
		}
		if(((GTempFile *)sender())->nodeId().isEmpty())
				shareList->addNewFileToXml(((GTempFile *)sender())->fileName(),((GTempFile *)sender())->fileHash(),((GTempFile *)sender())->chunksVector());
		emit finished(((GTempFile *)sender())->fileSize(),((GTempFile *)sender())->fileHash());
		if (!((GTempFile *)sender())->nodeId().isEmpty())
			emit finishedList(incomingDir +((GTempFile *)sender())->fileName());
		((GTempFile *)sender())->deleteLater();
	}
	else
		qWarning("impossible de renommer le fichier!!! " + ((GTempFile *)sender())->tempFileName()+" --> "+incomingDir +((GTempFile *)sender())->fileName());
	f.remove();
}
void GDownloader::newPart(Q_UINT16 chunk,unsigned char part)
{
	emit newPart(((GTempFile *)sender())->fileSize(),((GTempFile *)sender())->fileHash(),chunk,part);
}
void GDownloader::newChunk(Q_UINT16 chunk)
{
	qDebug("new chunk %i : %s",chunk,((GTempFile *)sender())->fileName().latin1());
}
void GDownloader::chunkLost(Q_UINT16 chunk)
{
	emit chunkLost(((GTempFile *)sender())->fileSize(),((GTempFile *)sender())->fileHash(),chunk);
	qDebug("chunk lost! %i : %s",chunk,((GTempFile *)sender())->fileName().latin1());
}

void GDownloader::connectGTempFile(GTempFile *f)
{
	connect(f,SIGNAL(unableToConnect(const QString&)),SLOT(unableToConnect(const QString&)));
	connect(f,SIGNAL(transferFinished()),SLOT(transferFinished()));
	connect(f,SIGNAL(newPart(Q_UINT16,unsigned char)),SLOT(newPart(Q_UINT16,unsigned char)));
	connect(f,SIGNAL(newChunk(Q_UINT16)),SLOT(newChunk(Q_UINT16)));
	connect(f,SIGNAL(chunkLost(Q_UINT16)),SLOT(chunkLost(Q_UINT16)));
	connect(f,SIGNAL(dlSpeed(Q_UINT64,const QString &,Q_UINT64,const QValueList<GSrcSpeed> &)),SLOT(dlSpeed(Q_UINT64,const QString&,Q_UINT64,const QValueList<GSrcSpeed> &)));
}
