/***************************************************************************
 *            gdownloader.h
 *
 *  Sat May  7 18:11:12 2005
 *  Copyright  2005  CrYpToNyM
 *  crypt0nym
 ****************************************************************************/

#ifndef _GDOWNLOADER_H
#define _GDOWNLOADER_H

#include <qobject.h>
#include <qstring.h>
#include <qbitarray.h>
#include <qcstring.h>	//QByteArray
#include <qvaluevector.h>
#include <qvaluelist.h>
#include <qptrvector.h>
#include <qtimer.h>

#include "../gtypes.h"
#include "../network/gfiledlsocket.h"
#include "../crypt/gcryptsocket.h"
#include "gtempfile.h"

struct s_waiting_src
{
	GTempFile *gtempf;
	QString nodeid;
};

typedef QValueList <s_waiting_src> t_waitingList;

struct s_dlingFile
{
	unsigned char tickCount;
};
	
class GDownloader : public QObject
{
	Q_OBJECT
	public:
		GDownloader(const QString &tempdir,const QString &incoming,QObject * parent=0,const char * name=0);
		QValueVector <GTempFile*> *gettempfileVector() {return &tempfileVector;}
	
	public slots:
		//Pour l'interface
		void loadTempFiles();
		void download(const QString &filename,Q_UINT64 filesize,const QString &hash);
		void cancel(Q_UINT64 filesize,const QString &hash);
		void pause(Q_UINT64 filesize,const QString &hash);
		void resume(Q_UINT64 filesize,const QString &hash);
		void setPriority(Q_UINT64 filesize,const QString &hash,unsigned char p);
	
		//Pour le GNetwork
		void getFileList(Q_UINT16 uid,const QString &nodeid,Q_UINT64 filesize,const QString &hash);
		void addSource(Q_UINT16 uid,const QString &nodeid,const QString &filename,Q_UINT64 filesize,const QString &hash);
		void incomingSocket(const QString &nodeid, GCryptSocket *s);
	
	
	signals:
	//Pour l'interface
		//Debut ou resume d'un dl
		void tempLoaded(const QString &filename,Q_UINT64 filesize,const QString &hash,QBitArray &part);
		//Nouvelle source pour un dl en cours
		void newSource(const QString &nodeid,const QString &filename,Q_UINT64 filesize,const QString &hash,const QBitArray &chunks, Q_UINT16 queuepos);
		//Perte d'une source
		void delSource(const QString &nodeid,Q_UINT64 filesize,const QString &hash);
		//Toutes les 2sec, update de la vitesse et de l'avancement
		void downloadSpeed(Q_UINT64 filesize,const QString& hash,Q_UINT64 received,const QValueList<GSrcSpeed> &srclist);
		//Un nouveau part (pour la barre de progression)
		void newPart(Q_UINT64 filesize,const QString &hash,Q_UINT32 chunk,unsigned char part);
		//Perte d'un chunk (pour la barre de progression)
		void chunkLost(Q_UINT64 filesize,const QString &hash,Q_UINT32 chunk);
		//Fin du DL
		void finished(Q_UINT64 filesize,const QString &hash);
		//Fin de dl de liste de fichier
		void finishedList(const QString &file);
		
	//Pour le GNetwork
		void askConnect(const QString &nodeid);
		void askSources(Q_UINT64 filesize, const QString &hash);
	
	private slots:
		//Pour les GTempFile
		void unableToConnect(const QString &nodeid);
		void transferFinished();
		void newPart(Q_UINT16 chunk,unsigned char part);
		void newChunk(Q_UINT16 chunk);
		void chunkLost(Q_UINT16 chunk);
		void dlSpeed(Q_UINT64 filesize,const QString& hash,Q_UINT64 received,const QValueList<GSrcSpeed> &srclist);

		void onTick(); //recherche de sources
	
	private:
		void connectGTempFile(GTempFile *f);
		bool isDownloading(Q_UINT64 filesize,const QString &hash);
	
		QString tempDir,incomingDir;
		QValueVector <GTempFile*> tempfileVector;
		QValueVector <s_dlingFile> dlingFiles;
		t_waitingList waitingSrc;
		QTimer *timer;
	
};

#endif /* _GDOWNLOADER_H */
