/***************************************************************************
 *            gtempfile.cpp
 *
 *  Wed May  4 17:02:10 2005
 *  Copyright  2005  CrYpToNyM
 *  Email
 ****************************************************************************/
#include "gtempfile.h"

#ifndef min
#define min(x,y) (((x)<(y))?(x):(y))
#endif

#define MIN_PART_ASKED 3
#define SPEEDTIMER_TIME 2000

GTempFile::GTempFile(const QString &tempfile, QObject * parent, const char * name) : QObject(parent,name)
{
	isok=false;
	chunkLeft = 0;
	nodeid = QString::null;
	nbchunkDL = 0;

	thread = new gtfThread(this);

	//Chargement de tempfile.xml dans xml
	//Un pointeur vers chaque node chunk dans le vecteur chunks
	qDebug(tempfile + ".xml");
	if(!loadExistingXml(tempfile + ".xml"))
	{
		qWarning("loadExistingXml : fichiers xml temporaire corrompu");
		return;
	}
	updateXml();
	filename =xml->documentElement().attribute("name");
	filesize = xml->documentElement().attribute("size").toULongLong();
	filehash = xml->documentElement().attribute("hash");
	priorityDL = (unsigned char) xml->documentElement().attribute("priority").toInt();

	qDebug("Reprise du download : " + filename + " - taille : " + QString::number(filesize)+ " - hash : " +filehash);

	//Ouverture de l'acces en ecriture
	file= new QFile(tempfile);
	if (!file->open(IO_WriteOnly | IO_Raw))
	{
		qWarning("GTempFile : Erreur d'ecriture sur " + tempfile);
		return;
	}
	qDebug("Reprise du telechargement ok");
	isok=true;

	timer=new QTimer(this);
	connect(timer,SIGNAL(timeout ()),this,SLOT(calcSpeed()));
	timer->start(SPEEDTIMER_TIME);
}



GTempFile::GTempFile(const QString& pathToPartFile, const QString &filen,unsigned char type,Q_UINT64 size,const QString &hash,
		QObject * parent,const QString& nid, const char * name) : QObject(parent,name)
{
	isok=false;

	nodeid = nid;
	filename = filen;
	filehash = hash;
	filesize = size;
	filetype = type;
	priorityDL = 1;
	isdownloading = FALSE;
	nbchunkDL = 0;
	thread = new gtfThread(this);

	unsigned int nbchunk = nbChunk();
	dlingChunks.resize(nbchunk);
	emptyChunks.resize(nbchunk);
	chunkSourceList.resize(nbchunk);
	chunkLeft = nbchunk;

	//Creation de xml et ecriture de tempfile.xml
	//Un pointeur vers chaque node chunk dans le vecteur chunks
	qDebug("Creation xml");
	createEmptyXmlTree(filename,hash,size);
	qDebug("Creation xml ok");
	xmlFile = new QFile(pathToPartFile + ".xml");
	xmlFile->open(IO_Truncate | IO_ReadWrite | IO_Translate);

	//Allouer le fichier puis Ouverture de l'acces en ecriture
	file= new QFile(pathToPartFile);
	if ( !(file) || (!(file->open(IO_WriteOnly | IO_Raw))) )
	{
		qWarning("GTempFile : Erreur d'ecriture sur " + pathToPartFile);
		return;
	}
	updateXml();
	isok=true;

	timer=new QTimer(this);
	connect(timer,SIGNAL(timeout ()),this,SLOT(calcSpeed()));
	timer->start(SPEEDTIMER_TIME);
}



GTempFile::~GTempFile()
{
  QValueList<GFileDlSocket *>::iterator it;

  for ( it = socketList.begin(); it != socketList.end(); ++it )
		delete(*it);

	delete timer;
  
	thread->wait();

  	//XML -> ecriture de xml
	updateXml();
  
	xmlFile->close();
	file->close();
	/*
	if (deltmp) //si annulation
	{
		//destruction du fichier temporaire
		QDir::remove(file->name());
		QDir::remove(file->name()+".xml");
	}*/
	delete(xml);
	delete(file);
	delete(xmlFile);
	delete(thread);

}



void GTempFile::calcSpeed()
{
	QValueList<GSrcSpeed> srclist;
	GSrcSpeed src;
	Q_UINT64 downloaded=0;
	QValueList<GFileDlSocket *>::iterator it;

	for (it=socketList.begin();it!=socketList.end();++it)
	{
		src.nodeId=(*it)->nodeId();
		src.speed=(*it)->written/(SPEEDTIMER_TIME/1000);
		downloaded+=(*it)->written;
		(*it)->written=0;
		srclist.append(src);
	}

	emit dlSpeed(filesize,filehash,downloaded,srclist);
}



bool GTempFile::ok()
{
	return isok;
}

/* retourne :
	0 : tout est ok
	1 : tout est ok et on a un nouveau chunk
	2 : tout est ok, le fichier est complet
	3 : ca s'est pas bien passe, il etait 12h53, le soleil brillait tres fort et malgre nos efforts, nous avons perdu le chunk
*/
void GTempFile::newPart(Q_UINT16 chunk, unsigned char part, QByteArray &dat)
{
	thread->addQueue(chunk,part,dat);
	if(thread->dataRecovered && !thread->running())
	{
		thread->dataRecovered = FALSE;
		thread->start(QThread::LowPriority);
	}
	emit newPart(chunk,part);
	askPart((GFileDlSocket *)sender());
}

bool GTempFile::checkChunkIntegrity(Q_UINT16 chunk)
{
	unsigned char h[MD5_DIGEST_LENGTH];
	char c[MD5_DIGEST_LENGTH*2 +1];

	qDebug("%ud,%llu",chunk,filesize);
	if ( md5_filepart_hash(file->name(),chunk*CHUNK_SIZE,min(filesize,(chunk+1)*(Q_UINT64)CHUNK_SIZE),h) )
	{
		isok=false;
		return false;
	}
	else
	{
		bin2hex_hash(h,c);
		qDebug(QString(c)+" == "+chunkHash(chunk));
		return (QString(c) == chunkHash(chunk));
	}
}

bool GTempFile::loadExistingXml(const QString& tempfile)
{
	xml = new QDomDocument("Gloster-Temp-File");
	xmlTempFile handler(this);
  xmlFile = new QFile( tempfile );
	xmlFile->open(IO_ReadWrite | IO_Translate);
  QXmlInputSource source( xmlFile );
  QXmlSimpleReader reader;
  reader.setContentHandler( &handler );

  if (!reader.parse( source ))
		return FALSE;

	return TRUE;
}


Q_UINT64 GTempFile::fileSize()
{
	return filesize;
}


const QString& GTempFile::fileName()
{
	return filename;
}


const QString& GTempFile::fileHash()
{
	return filehash;
}

QString GTempFile::chunkHash(unsigned long int c)
{
	return chunks[c].attribute("hash");
}

bool GTempFile::isDownloading()
{
	return isdownloading;
}

void GTempFile::startDownloading(QStringList& hash)
{
	qDebug("start downloading");
	/* ajout des hash dans le xml + modif de l'attribut isDownloading*/
	unsigned int i=0;
	gtfmutex.lock();
	for ( QStringList::Iterator it = hash.begin(); it != hash.end(); ++it )
	{
			chunks[i].setAttribute("hash",*it);
			i++;
	}
	xml->documentElement().setAttribute("isDownloading",QString::number(1));
	isdownloading = TRUE;
	gtfmutex.unlock();
	updateXml();
}

void GTempFile::createEmptyXmlTree(const QString& name, const QString& hash, const Q_UINT64 size)
{
	unsigned int n,i;
	xml = new QDomDocument("Gloster-Temp-File");
	QDomElement chunk,part,dFile;
	unsigned int nbchunk = nbChunk();

	chunks.resize(nbchunk);
	chunkLeft = nbchunk;

	dFile = xml->createElement("file");
	dFile.setAttribute("name",name);
	dFile.setAttribute("hash",hash);
	dFile.setAttribute("size",QString::number(size));
	dFile.setAttribute("isDownloading",QString::number(0));
	dFile.setAttribute("priority",QString::number(2));
	if(nodeid != QString::null)
		dFile.setAttribute("nodeid",nodeid);
	priorityDL = (unsigned char) 2; //affectation de la variable private
	xml->appendChild(dFile);

	for(i = 0;i<nbchunk-1;++i)
	{
		chunks[i] = xml->createElement("chunk");
		chunks[i].setAttribute("n",QString::number(i));
		chunks[i].setAttribute("hash",QString::null);
		chunks[i].setAttribute("state",QString::number(s_empty));
		chunks[i].setAttribute("count",QString::number(PART_BY_CHUNK));
		dFile.appendChild(chunks[i]);

		part = xml->createElement("part");

		for(unsigned int j=0;j<PART_BY_CHUNK;++j)
		{
			emptyChunks[i] << j;
			part.setAttribute("p"+QString::number(j),QString::number(s_empty));
		}

		chunks[i].appendChild(part);
	}
	i = nbchunk-1;
	chunks[i] = xml->createElement("chunk");
	chunks[i].setAttribute("n",QString::number(i));
	chunks[i].setAttribute("hash",QString::null);
	chunks[i].setAttribute("state",QString::number(s_empty));
	dFile.appendChild(chunks[i]);
	part = xml->createElement("part");
	if((n = filesize%CHUNK_SIZE))
		n = n/PART_SIZE + (n%PART_SIZE?1:0);
	else
		n = PART_BY_CHUNK;
	chunks[i].setAttribute("count",QString::number(n));
	for(unsigned int j=0;j<n;++j)
	{
		emptyChunks[i] << j;
		part.setAttribute("p"+QString::number(j),QString::number(s_empty));
	}

	chunks[i].appendChild(part);
}


Q_UINT64 GTempFile::nbChunk()
{
	return (filesize+CHUNK_SIZE-1)/CHUNK_SIZE;
}

QString GTempFile::tempFileName()
{
	return (file->name());
}

void GTempFile::setPriority(unsigned char p)
{
	gtfmutex.lock();
	xml->documentElement().setAttribute("priority",QString::number(p));
	gtfmutex.unlock();
	priorityDL = (unsigned char) p;
	updateXml();
}

void GTempFile::updateXml()
{
	gtfmutex.lock();
	xmlFile->at(0);
	QTextStream stream( xmlFile );
	stream << xml->toString();
	xmlFile->flush();
	gtfmutex.unlock();
}

void GTempFile::setCompletedPart(Q_UINT16 chunk, unsigned char part)
{
	gtfmutex.lock();
	chunks[chunk].firstChild().toElement().setAttribute("p"+QString::number(part),QString::number(2));
	dlingChunks[chunk].erase(dlingChunks[chunk].find(part));
	gtfmutex.unlock();
	updateXml();
}



void GTempFile::resetChunk(Q_UINT16 chunk)
{
	gtfmutex.lock();
	emptyChunks[chunk].clear();
	for(unsigned int i = 0;i<(chunks[chunk].attribute("count").toUInt());i++)
	{
		emptyChunks[chunk] << i;
		chunks[chunk].firstChild().toElement().setAttribute("p"+QString::number(i) ,QString::number(0) );
	}
	dlingChunks[chunk].clear();
	chunks[chunk].setAttribute("state",QString::number(0) );
	chunkLeft += 1;
	gtfmutex.unlock();
	updateXml();
	gtfmutex.lock();
	nbchunkDL = chunk;
	gtfmutex.unlock();

	lostChunkEvent * event;
	event = new lostChunkEvent(chunk);
	QApplication::postEvent(this, event);
	if(chunkLeft == 1)
		for(unsigned int k=0;k<MIN_PART_ASKED;k++)
			askPart((GFileDlSocket *)sender());
}


void GTempFile::chunkComplete(Q_UINT16 chunk)
{
	chunkLeft -= 1;
	chunks[chunk].setAttribute("state",QString::number(2));
	updateXml();

	chunkEvent * event;
	event = new chunkEvent(chunk);
	QApplication::postEvent(this, event);
}

void GTempFile::newSource(const QString& nid,Q_UINT16 uid)
{
	GFileDlSocket * temp;
	QString vhash;
	QValueList<GFileDlSocket *>::iterator it;
	qDebug("GTempFile::newSource : creation d'un GFileDlSocket, nodeid : "+nid);

  for ( it = socketList.begin(); it != socketList.end(); ++it )
		if ((*it)->nodeId() == nid)
			return;
	if(nodeid.isEmpty())
		temp = (new GFileDlSocket(uid,nid,filehash,filesize,this));
	else
	{
		vhash.fill('0',MD5_DIGEST_LENGTH*2);
		temp = (new GFileDlSocket(uid,nodeid,vhash,0,this,filesize));
	}
	if(temp->ok)
	{
		socketList << temp;
		connectSocket(temp);
	}
	else
		delete(temp);
}

void GTempFile::newDlSocket(GCryptSocket * socket)
{
	QString vhash;
	GFileDlSocket * temp;

	if(nodeid.isEmpty())
		temp = (new GFileDlSocket(filehash,filesize,this));
	else
	{
		vhash.fill('0',MD5_DIGEST_LENGTH*2);
		temp = (new GFileDlSocket(vhash,0,this,filesize));
	}
	if(temp->ok)
	{
		socketList << temp;
		connectSocket(temp);
		temp->setSocket(socket);
	}
	else
		delete(temp);
}


void GTempFile::connectSocket(GFileDlSocket * socket)
{
		connect(socket,SIGNAL(haventFile()),this,SLOT(haventFile()));
		connect(socket,SIGNAL(newPart(Q_UINT16,unsigned char,QByteArray&)),this,SLOT(newPart(Q_UINT16 , unsigned char , QByteArray &)));
		connect(socket,SIGNAL(recvFileInf(QStringList, QBitArray &)),this,SLOT(recvFileInf(QStringList, QBitArray &)));
		connect(socket,SIGNAL(newChunkSource(Q_UINT16 )),this,SLOT(newChunkSource(Q_UINT16 )));
		connect(socket,SIGNAL(unableToConnect()),this,SLOT(unabletoconnect()));
		connect(socket,SIGNAL(sockClose()),this,SLOT(sockClose()));
}


void GTempFile::haventFile()
{
	QValueList<GFileDlSocket *>::iterator it;
	it = socketList.find((GFileDlSocket *)sender());
	if(it != socketList.end())
	{
		disconnect((*it),0,0,0);
		delete((*it));
		socketList.remove(it);
	}
}

void GTempFile::unabletoconnect()
{
	emit unableToConnect(((GFileDlSocket *)sender())->nodeId());
	QValueList<GFileDlSocket *>::iterator it;
	it = socketList.find((GFileDlSocket *)sender());
	if(it != socketList.end())
	{
		disconnect((*it),0,0,0);
		delete((*it));
		socketList.remove(it);
	}
}


void GTempFile::sockClose()
{
	QString nodeid = ((GFileDlSocket *)sender())->nodeId();

	PartPosQueue::Iterator itQueue;
	for ( itQueue = ((GFileDlSocket *)sender())->part_queue.begin(); itQueue != ((GFileDlSocket *)sender())->part_queue.end(); ++itQueue )
	{
		emptyChunks[(*itQueue).chunk] << ((*itQueue).part);
		dlingChunks[(*itQueue).chunk].erase(dlingChunks[(*itQueue).chunk].find((*itQueue).part));
	}

	for(unsigned int i=0;i<chunkSourceList.size();++i)
	{
		QStringList::Iterator itl = chunkSourceList[i].begin();
		while(itl != chunkSourceList[i].end())
		{
				if(*itl == nodeid)
				{
					itl = chunkSourceList[i].erase(itl);
					continue;
				}
				++itl;
		}
	}

	QValueList<GFileDlSocket *>::iterator it;
	it = socketList.find((GFileDlSocket *)sender());
	if(it != socketList.end())
	{
		disconnect((*it),0,0,0);
		delete((*it));
		socketList.remove(it);
	}
}


bool GTempFile::isChunkComplete(Q_UINT16 chunk)
{
	gtfmutex.lock();
	bool result = (dlingChunks[chunk].isEmpty() && emptyChunks[chunk].isEmpty());
	gtfmutex.unlock();
	return result;
}


void GTempFile::recvFileInf(QStringList chunkhashes, QBitArray &chunksState)
{
	qDebug("recvfileinf");
	//test des hashs avec ceux qu'on a, ajout des hash si isdownloading=FALSE
	if(!isdownloading)
		startDownloading(chunkhashes);
	else
	{
		unsigned int j=0;
    for ( QStringList::Iterator it = chunkhashes.begin(); it != chunkhashes.end(); ++it )
		{
      	if(chunks[j].attribute("hash") != *it)
				{
					((GFileDlSocket *)sender())->close();
					return;
				}
				j++;
		}
	}

	//mise a jour des sources des hashs
	gtfmutex.lock();
	for(int i=0;i<(int) chunkSourceList.size();++i)
	{
		if(chunksState[i])
			chunkSourceList[i] << ((GFileDlSocket *)sender())->nodeId();
	}
	gtfmutex.unlock();
	for(unsigned int k=0;k<MIN_PART_ASKED;k++)
		askPart((GFileDlSocket *)sender());
}


void GTempFile::newChunkSource(Q_UINT16 chunk)
{
	chunkSourceList[chunk] << ((GFileDlSocket *)sender())->nodeId();
}


void GTempFile::askPart(GFileDlSocket * socket)
{
	qDebug("askpart");
	Q_INT32 chunk;
	chunk = findChunkToDownload(socket->nodeId());
	qDebug(chunk==-1?"Plus de chunk a demander":"demande du chunk : %d",chunk);
	if(chunk > -1)
	{
		gtfmutex.lock();
		chunks[chunk].setAttribute("state",QString::number(1));
		unsigned char part = emptyChunks[chunk].first();
		chunks[chunk].firstChild().toElement().setAttribute("p"+QString::number(part),QString::number(1));
		emptyChunks[chunk].pop_front();
		dlingChunks[chunk] << part;
		gtfmutex.unlock();
		socket->getPart(chunk,part);
	}
}


Q_INT32 GTempFile::findChunkToDownload(const QString& nodeid)
{
	gtfmutex.lock();
	unsigned int i=nbchunkDL;
	while((i < emptyChunks.size()) && ((emptyChunks[i].empty() || (chunkSourceList[i].find(nodeid) == chunkSourceList[i].end()))))
		i++;
	nbchunkDL = i;
	gtfmutex.unlock();
	if(i == emptyChunks.size())
		return -1;
	else
		return i;
}


t_state GTempFile::getPartState(Q_UINT16 chunk, unsigned char part)
{
	gtfmutex.lock();
	t_state result = (t_state)chunks[chunk].firstChild().toElement().attribute("p"+QString::number(part)).toInt(); ;
	gtfmutex.unlock();
	return result;
}

unsigned char GTempFile::priority()
{
	return priorityDL;
}

bool GTempFile::event(QEvent * e)
{
	switch(e->type())
	{
		case (QEvent::User+1):
			emit transferFinished();
			break;

		case (QEvent::User+2):
			gtfmutex.lock();
			emit newChunk(((chunkEvent *)e)->chunk);
			gtfmutex.unlock();
			break;

		case (QEvent::User+3):
			gtfmutex.lock();
			emit chunkLost(((lostChunkEvent *)e)->chunk);
			gtfmutex.unlock();
			break;

		case (QEvent::User+10):
			gtfmutex.lock();
			thread->dataRecovered = TRUE;
			gtfmutex.unlock();
			break;

		default :
				return QObject::event(e);
	}
	return TRUE;
}

/*****************************************************************************
 *
 * Class gtfThread
 *
 *****************************************************************************/


void gtfThread::addQueue(Q_UINT16 chunk, unsigned char part, QByteArray &dat)
{
	t_gtfqueue temp;
	temp.chunk = chunk;
	temp.part = part;
	temp.dat = dat;

	parent->gtfmutex.lock();
	queue << temp;
	parent->gtfmutex.unlock();
}


void gtfThread::run()
{
	while(!queue.isEmpty())
	{
		writePart();
		parent->gtfmutex.lock();
		queue.pop_front();
		parent->gtfmutex.unlock();
	}

	QEvent * event;
	event = new QEvent((QEvent::Type)(QEvent::User+10));
	QApplication::postEvent(parent, event);
}


void gtfThread::writePart()
{
	unsigned int nbchunk;

	parent->gtfmutex.lock();
	unsigned char part = queue.first().part;
	Q_UINT16 chunk = queue.first().chunk;
	parent->gtfmutex.unlock();

	nbchunk = parent->nbChunk();

	//ecriture dans le fichier
	parent->gtfmutex.lock();
	parent->file->at(chunk*CHUNK_SIZE + part*PART_SIZE);
	parent->file->writeBlock(qUncompress(queue.first().dat));
	parent->gtfmutex.unlock();

	//XML -> mise a jour dans le xml pour le part uniquement
	parent->setCompletedPart(chunk,part);
	//si tous les parts du chunk sont telecharges
	if(parent->isChunkComplete(chunk))
	{
		qDebug("chunk Complete");
		if (!parent->checkChunkIntegrity(chunk))
		{
			qDebug("prob d'integrite");
			if (!parent->ok())
			{
				qWarning("GTempFile : Erreur de lecture sur "+parent->file->name());
				return;
			}
			else
			{
				//XML -> mettre ce qu'il faut dans le xml (tout a 0 pr ce chunk)
				qDebug("reset chunk");
				parent->resetChunk(chunk);
			}
		}
		else
		{
			//XML -> mettre le chunk a complet dans le xml
			qDebug("validation chunk Complete");
			parent->chunkComplete(chunk);

			/*si le fichier est complet*/
			if(!parent->chunkLeft)
			{
				qDebug("fichier complet");
				//Pourquoi pas un petit test d'integrite
				parent->file->close();
				QEvent * event = new QEvent((QEvent::Type)(QEvent::User+1));
				QApplication::postEvent(parent, event);
				return;
			}
		}
	}
}
