/***************************************************************************
 *            gtempfile.h
 *
 *  Tue May  3 17:46:29 2005
 *  Copyright  2005  CrYpToNyM
 *  Email
 ****************************************************************************/

#ifndef _GTEMPFILE_H
#define _GTEMPFILE_H

#include <qdom.h>
#include <qxml.h>
#include <qstring.h>
#include <qvaluevector.h>
#include <qtextstream.h>
#include <qstringlist.h>
#include <qobject.h>
#include <qbitarray.h>
#include <qmutex.h>
#include <qthread.h>
#include <qcstring.h>
#include <qdeepcopy.h>
#include <qapplication.h>
#include <qevent.h>
#include <qtimer.h>
#include "../gtypes.h"
#include "gfileconst.h"
#include "../network/gfilesocketcommon.h"
#include "ghashfile.h"
#include "xmltempfile.h"
#include "../network/gfiledlsocket.h"
#include "../crypt/gcryptsocket.h"
#include "qsharelist.h"

enum t_state {s_empty,s_dling,s_complete};

extern QSharelist * shareList;

typedef struct
{
	Q_UINT16 chunk;
	unsigned char part;
	QDeepCopy<QByteArray> dat;
}t_gtfqueue; 

typedef struct
{
	Q_UINT16 chunk;
	unsigned char part;
}t_written; 


class partEvent : public QEvent
{
	public :
	partEvent(Q_UINT16 c,unsigned char p)
	:QEvent(QEvent::User),part(p),chunk(c) {}
	
	unsigned char part;
	Q_UINT16 chunk;
};


class chunkEvent : public QEvent
{
	public :
	chunkEvent(Q_UINT16 c)
	:QEvent((QEvent::Type)(QEvent::User+2)),chunk(c) {}
	
	Q_UINT16 chunk;
};

class lostChunkEvent : public QEvent
{
	public :
	lostChunkEvent(Q_UINT16 c)
	:QEvent((QEvent::Type)(QEvent::User+3)),chunk(c) {}
	
	Q_UINT16 chunk;
};

class gtfThread : public QThread 
{
	public:
		gtfThread(GTempFile * p):QThread(),parent(p),dataRecovered(TRUE) {}
		void addQueue(Q_UINT16 chunk, unsigned char part, QByteArray &dat);
		void run();
		void writePart();
	
		GTempFile * parent;
		bool dataRecovered;
		QValueList<t_gtfqueue> queue;
};


class GTempFile : public QObject
{
	Q_OBJECT
	
	public:
		/*tempfile correspond au nom du fichier temporaire dans lequel les donnees
			sont ecrites. par exemple 01.part
			Le fichier xml associe est 01.part.xml (soit tempfile + ".xml")
		*/
		GTempFile(const QString &tempfile, QObject * parent = 0, const char * name = 0); //constructor
		GTempFile(const QString&,const QString &filename,unsigned char type,
				Q_UINT64 size,const QString &hash, QObject * parent = 0, const QString& nid = QString::null, const char * name = 0);
		/* Mettre le booleen a vrai si annulation de download */
		~GTempFile();                        //destructor

		Q_UINT64 fileSize();
		const QString& fileName();
		const QString& fileHash();
		QValueVector<QDomElement> chunksVector() {return chunks;}
		QString chunkHash(unsigned long int c);
		QString tempFileName();
		unsigned char priority();
		void setPriority(unsigned char);		
	
		bool ok();
		bool isDownloading();	
		void startDownloading(QStringList&);
		bool loadExistingXml(const QString &);
		bool isChunkComplete(Q_UINT16 chunk);
		Q_UINT64 nbChunk();
		t_state getPartState(Q_UINT16 chunk, unsigned char part);
		const QString& nodeId() {return nodeid;}

	signals :
		void unableToConnect(const QString& nodeid);
		void transferFinished();
		void newPart(Q_UINT16 chunk,unsigned char part);
		void newChunk(Q_UINT16 chunk);
		void chunkLost(Q_UINT16 chunk);
		void finalizing();
		void dlSpeed(Q_UINT64 filesize,const QString& hash,Q_UINT64 bytesincoming,const QValueList<GSrcSpeed> &srclist);
		
		
	public slots :
		void newSource(const QString& nodeid, Q_UINT16 uid=0); /* cree un nouveau socket et le connecte */
		void newDlSocket(GCryptSocket * socket);
		
	private slots :
		/*L'integrite de la partie doit avoir ete verifiee avant
			Ecrit dat a la position voulue et change les valeurs de chunk et part
			pour les mettres a une position vide
			Si le part retourne est >=PART_BY_CHUNK alors il n'y a plus de part a dl
		*/
		void unabletoconnect();
		void haventFile();
		void newPart(Q_UINT16 chunk, unsigned char part, QByteArray &dat); /* mettre a jour les listes d'etats des parts */
		void sockClose();
		void recvFileInf(QStringList chunkhashes, QBitArray &chunks);
		void newChunkSource(Q_UINT16 chunk);
		void calcSpeed();

	private:
		void createEmptyXmlTree(const QString& name, const QString& hash, const Q_UINT64 size);
		bool checkChunkIntegrity(Q_UINT16 chunk);	
		bool checkFileIntegrity();
		void updateXml();
		void setCompletedPart(Q_UINT16 chunk, unsigned char part);
		void resetChunk(Q_UINT16 chunk);
		void chunkComplete(Q_UINT16 chunk);
		void connectSocket(GFileDlSocket * socket);
		void askPart(GFileDlSocket * socket);
		Q_INT32 findChunkToDownload(const QString& nodeid);
		QTimer *timer;
	
	protected:
		bool GTempFile::event(QEvent * e);
		
		unsigned char filetype;
		QString filename;
		Q_UINT64 filesize;
		QString filehash;
		QDomDocument * xml;
		QFile *file;   
		QFile * xmlFile;
		Q_UINT16 chunkLeft;
		QValueVector<QDomElement> chunks;		/* pointeur vers les elements XML des chunks */
		QString nodeid;
		Q_UINT16 nbchunkDL;
			
		QValueVector<QValueList<unsigned char> > dlingChunks; /* liste des chunks en cours de download */
		QValueVector<QValueList<unsigned char> > emptyChunks; 	/* Liste des chunks vides */
		QValueVector<QStringList> chunkSourceList; 	/* liste des sources pour chaque part */
		QValueList<GFileDlSocket *> socketList;	/* Liste de tous les socket ouverts pour la reception du fichier */
	
		bool isok;				
		bool isdownloading;
		unsigned char priorityDL;
		gtfThread * thread;
		QMutex gtfmutex;
		
	friend class xmlTempFile;
	friend class gtfThread;
};

#endif /* _GTEMPFILE_H */
