/***************************************************************************
 *            guploader.cpp
 *
 *  Tue May 10 11:14:47 2005
 *  Copyright  2005  CrYpToNyM
 *  Email
 ****************************************************************************/

#include "guploader.h"
#include "../crypt/hash.h"
#include "qsharelist.h"

#ifndef min
#define min(a,b) (((a)<(b))?(a):(b))
#endif

#define SPEEDTIMER_TIME 2000


extern QSharelist * shareList;

GUploader::GUploader(Q_UINT16 uspeed, Q_UINT16 nbslot, QValueVector <GTempFile*> *tmpfilevect,QObject * parent,const char * name)
	:QObject(parent,name)
{
	temp_files=tmpfilevect;
	speed=uspeed;
	socket.setAutoDelete(TRUE);
	nbslots=0;
	setNbSlots(nbslot);
	timer=new QTimer(this);
	connect(timer,SIGNAL(timeout ()),this,SLOT(calcSpeed()));
	timer->start(SPEEDTIMER_TIME);
}

GUploader::~GUploader()
{
	delete timer;
}

void GUploader::calcSpeed()
{
	int i,j;
	long int s;

	for (i=0; i<(int)socket.size(); ++i)
	{
		if (socket[i] && (j = socket[i]->file_index) >= 0)
		{
			s = socket[i]->written / (SPEEDTIMER_TIME / 1000);
			emit Transferring(open_files[j].size, open_files[j].hash, socket[i]->nodeName(), s);
			socket[i]->written = 0;
		}
	}
}

/*
** met un socket dans un slot libre et demarre ce dernier (il est ferme si aucun slot n'est libre)
*/
void GUploader::startSocket(GFileUlSocket *s)
{
		int i;

		if ((i = socket.findRef(NULL)) == -1)
			delete s;
		else
		{
			socket.insert(i, s);
			s->index=i;
			
			connect(s,SIGNAL(askingFile(const char *,Q_UINT64)),SLOT(askingFile(const char *,Q_UINT64)));
			connect(s,SIGNAL(newPart(unsigned int, unsigned char)),SLOT(getPart(unsigned int, unsigned char)));
			
			emit connection(s->nodeId());
			
			setUlSpeed(speed); // repartir la vitesse
			s->start();
		}
}

/*
** Reparti equitablement la vitesse entre les sockets
** Cette maniere est loin d'etre optimale mais pour le moment ca ira
*/
void GUploader::setUlSpeed(Q_UINT16 uspeed)
{
	unsigned int i;
	Q_UINT32 speed_by_sock;

	if (!socket.count())
		return ;
	speed=uspeed;
	speed_by_sock = speed * 1024 / socket.count();
	for (i=0; i<socket.size(); ++i)
		if (socket[i])
			socket[i]->setMaxSpeed(speed_by_sock);
}


void GUploader::setNbSlots(Q_UINT16 n)
{
	int i;
	GFileUlSocket *s;

	if (n < nbslots) //Si on reduit la taille alors on detruit les sockets en trop
	{
		for (i = n; i < nbslots; ++i)
			delete socket.take(i);
	}
	nbslots = n;
	socket.resize(n);
	while (queue.count() && socket.count() < n) //Remplir les slots vides
	{
		startSocket(queue.take(0));
		for (s = queue.first(); s; s = queue.next())
			s->index++;
	}
	setUlSpeed(speed); //Penser a repartir a nouveau la vitesse
}


/*
** Quand on a un socket entrant qui demande un upload
** Si on a de la place, on le demarre
** Sinon on le met en attente
*/
void GUploader::incomingUl(const QString &nodeid, GCryptSocket *so)
{
	GFileUlSocket *s = new GFileUlSocket(nodeid, so,this);

	connect(s,SIGNAL(sockClose()),SLOT(sockClose()));
	if (socket.count() < nbslots )
		startSocket(s);
	else
	{
		queue.append(s);
		s->index = -(int)queue.count();
	}
}

void GUploader::outgoingUl(const QString &nodeid, const QString &ip, Q_UINT16 port)
{
	GFileUlSocket *s = new GFileUlSocket(nodeid, ip, port,this);

	connect(s,SIGNAL(sockClose()),SLOT(sockClose()));
	if (socket.count() < nbslots )
		startSocket(s);
	else
	{
		queue.append(s);
		s->index = -(int)queue.count();
	}
}

void GUploader::askingFile(const char *h,Q_UINT64 size)
{
	unsigned int i,j,index;
	int k;
	bool filelist=false;
	char c[MD5_DIGEST_LENGTH*2+1], *chunkh;
	QString hash,tmp;

	bin2hex_hash((const unsigned char *)h,c);
	hash = c;

	index=((GFileUlSocket *) sender())->index;

	if (hash==tmp.fill('0',MD5_DIGEST_LENGTH*2) && size==0)
	{
		filelist=true;
		hash=shareList->fileListHash();
		size=shareList->fileListSize();
	}



	for (i=0;i<open_files.count();++i)
	{
		if ((open_files[i].f) && (open_files[i].size==size) &&(open_files[i].hash==hash) )
			break;
	}

	if (i<open_files.count())
	{
		for (k=0;k<(int)socket.size();++k)
		{
			if ( (socket[k]->nodeId() == socket[index]->nodeId())
				&& (socket[k]->file_index == (int)i) )
			{
				qWarning("La meme personne demande 2 fois le meme fichier -> CLOSE");
				socket[index]->close();
				return;
			}
		}
		socket[index]->file_index=i;
		open_files[i].nbopen++;
		chunkh=(char *)malloc(open_files[i].chunk_hash.count()*MD5_DIGEST_LENGTH);
		for (j=0;j<open_files[i].chunk_hash.count();++j)
			hex2bin_hash(open_files[i].chunk_hash[j].latin1(),chunkh+j*MD5_DIGEST_LENGTH);
		socket[index]->sendFileInf(chunkh,open_files[i].chunks);
		free(chunkh);
		emit askingFile(open_files[i].size, open_files[i].hash, open_files[i].f->name().section(QDir::separator(),-1), socket[index]->nodeName());
		return;
	}




	s_ofile file;
/* Telechargement de fichier temp a faire
	for (i=0;i<temp_files->count();++i)
	{
		if ( (temp_files->at(i)->fileSize()==size) &&(temp_files->at(i)->fileHash()==hash) )
		{

				QFile *f;
	Q_UINT64 size;
	QString hash;
	QValueVector <QString> chunk_hash;
	QBitArray chunks;
	Q_UINT16 nbopen;
			file.f= new QFile(temp_files->at(i)->)


			socket[index]->file_index=i;
			emit askingFile(open_files[i].size, open_files[i].hash, open_files[i].f->name(), socket[index]->nodeName());
			return;
		}
	}
*/
	QStringList lst;
	QString fname;

	if ( (filelist && !(fname=shareList->fileListName()).isEmpty()) || (!(fname=shareList->haveFile(hash,size)).isEmpty()) )
	{
		if (filelist)
		{
			lst=shareList->getHashPartFileList();
			if (lst.empty())
			{
				((GFileUlSocket *) sender())->close();
				return;
			}
		}
		else
			lst=shareList->getHashPart(hash,size);
		file.size=size;
		file.hash=hash;
		file.f=new QFile(fname);
		if (!(file.f->open(IO_Raw | IO_ReadOnly)))
		{
			qWarning("GUploader::askingFile - Impossible de lire "+fname);
			socket[index]->close();
			return;
		}
		for ( QStringList::Iterator it = lst.begin(); it != lst.end(); ++it )
		{
			file.chunk_hash.append(*it);
		}
		file.chunks.fill(true,file.chunk_hash.count());
		file.nbopen=1;
		i=addToOpenedFiles(file);
		socket[index]->file_index=i;
		chunkh=(char *)malloc(open_files[i].chunk_hash.count()*MD5_DIGEST_LENGTH);
		for (j=0;j<open_files[i].chunk_hash.count();++j)
			hex2bin_hash(open_files[i].chunk_hash[j].latin1(),chunkh+j*MD5_DIGEST_LENGTH);
		socket[index]->sendFileInf(chunkh,open_files[i].chunks);
		free(chunkh);
		emit askingFile(open_files[i].size, open_files[i].hash, open_files[i].f->name().section(QDir::separator(),-1), socket[index]->nodeName());
	}
	else
	{
		qDebug("GUploader::askingFile - Demande d'un fichier inconnu, close du socket");
		socket[index]->close();
	}
}

int GUploader::addToOpenedFiles(s_ofile &ofile)
{
	int i;
	for (i=0;i<(int)open_files.count();++i)
		if (!open_files[i].f)
		{
			open_files[i]=ofile;
			return i;
		}
	open_files.append(ofile);
	return i;
}

void GUploader::getPart(unsigned int chunk, unsigned char part)
{
	unsigned int findex;
	Q_UINT64 start,end;
	QByteArray b;

	findex=((GFileUlSocket *) sender())->file_index;
	start = chunk*CHUNK_SIZE+part*PART_SIZE;
	if ( (start<open_files[(int)findex].size) && (open_files[findex].chunks[(int)chunk]) && (open_files[(int)findex].f->at(start)) )
	{
		end = min(start+PART_SIZE,open_files[(int)findex].size);
		b.resize(end-start);
		if (open_files[(int)findex].f->readBlock(b.data(),b.size())==(Q_LONG)b.size())
			((GFileUlSocket *) sender())->sendPart(qCompress(b));
		else
			((GFileUlSocket *) sender())->close();
	}
	else
		((GFileUlSocket *) sender())->close();
}

void GUploader::sockClose()
{
	int i;
	Q_UINT64 fsize = 0;
	QString hash;
	GFileUlSocket *s;

	i=((GFileUlSocket *) sender())->file_index;
	if (i>=0)
	{
		fsize=open_files[i].size;
		hash=open_files[i].hash;
		if (!(--open_files[i].nbopen))	//On regarde si du monde a encore besoin du fichier
		{
			delete open_files[i].f;
			open_files[i].f=NULL;
		}
	}
	emit connectionLost(hash, fsize, ((GFileUlSocket *) sender())->nodeName());
	if (((GFileUlSocket *) sender())->index < 0) //Dans la file d'attente
		queue.removeRef((GFileUlSocket *) sender());
	else //Dans un slot, il faut donc faire avancer la file
	{
		socket.take(((GFileUlSocket *) sender())->index);
		if (queue.count())
		{
			startSocket(queue.take(0));
			for (s = queue.first(); s; s = queue.next())
				s->index++;
		}
	}
	delete (GFileUlSocket *) sender();
}