/****************************************************************************
 *            qsharelist.cpp
 *
 *  Tue Mar 29 19:59:20 2005
 *  Copyright  2005  z
 *  z@zzteam.net
 ****************************************************************************/
#include <iostream>
#include <qstring.h>
#include <qstringlist.h>
#include <qapplication.h>
#include <qdeepcopy.h>
#include "xmlparser.h"
#include "qsharelist.h"
#include "ghashfile.h"
#include "../gui/filelist/sharefilebrowser.h"


/* Constructor */
QSharelist::QSharelist(QObject * parent):QObject(parent),refreshTimer(0)
{
	filelistname = gconf->confDir()+"filedb.xml";
	initFileListInfos();
	nb = 0;
	dataRecovered = TRUE;
	thread = new searchThread(this);
	updateConfRefreshShareList();
}

QSharelist::~QSharelist()
{
	thread->wait();
}

void QSharelist::initFileListInfos()
{
	QFileInfo f(filelistname);
	GHashFile hash;
	hash.readFromFile(filelistname);
	filelisthash = hash.getFileHash();
	filelistsize = f.size();
}

void QSharelist::loadxmlfile()
{
	Xmlparser handler(this);
	FileListMutex.lock();
  QFile xmlFile(gconf->confDir()+"filelist.xml" );
  QXmlInputSource source( &xmlFile );
  QXmlSimpleReader reader;
  reader.setContentHandler( &handler );

  if (!reader.parse( source ))
	{
		qDebug("QSharelist : liste de fichier partages corrompue! Creation d'un nouveau xml en cours....");
		FileListMutex.unlock();
		makeEmptyFileList();
		return;
	}

	FileListMutex.unlock();
}


QString QSharelist::haveFile(const QString& hash, Q_UINT64, unsigned int type)
{
	qDebug("[qsharelist.cpp] havefile - hash = " + hash);
	GfileMap::Iterator it;
	if(type == 8)
	{
		for(unsigned int i=0;i<8;++i)
		{
			  if((it = filelist[i].find(hash)) != filelist[i].end())
					return it.data();
		}
	}
	else
		if((it = filelist[type].find(hash)) == filelist[type].end())
			return QString::null;
		else
			return it.data();

	return QString::null;
}

void QSharelist::searchFile(const QString& name, unsigned int type)
{
	t_queue tmp;

	if(name.simplifyWhiteSpace() != QString::null && name.length()>=3)
	{
		tmp.name = name;
		tmp.type = type;
		queue << tmp;
		if(dataRecovered && !thread->running())
		{
			thread->init(queue.front());
			queue.pop_front();
			dataRecovered = FALSE;
			thread->start(QThread::LowPriority);
		}
	}
}

bool QSharelist::event ( QEvent * e )
{
	if(e->type() == (QEvent::User))
		emit resultToReceive();

	return TRUE;
}

QString QSharelist::getResult(t_searchResult& res)
{
	res = QDeepCopy<t_searchResult>(result);
	dataRecovered = TRUE;
	QDeepCopy<QString> result = lastSearch;
	if(!queue.isEmpty())
	{
		thread->init(queue.front());
		queue.pop_front();
		dataRecovered = FALSE;
		thread->start(QThread::LowPriority);
	}
	return result;
}

QStringList QSharelist::getHashPart(const QString& hash,Q_UINT64 size)
{
	QStringList result;
	Xmlparser handler(hash,size,&result);
	FileListMutex.lock();
  QFile xmlFile(gconf->confDir()+"filelist.xml" );
  QXmlInputSource source( &xmlFile );
  QXmlSimpleReader reader;
  reader.setContentHandler( &handler );

  if (!reader.parse( source ))
	{
		qDebug("QSharelist : liste de fichier partages corrompue! Creation d'un nouveau xml en cours....");
		FileListMutex.unlock();
		makeEmptyFileList();
		return result;
	}
	FileListMutex.unlock();
	return result;
}

QStringList QSharelist::getHashPartFileList()
{
	QStringList result;

	Xmlparser handler(&result);
	FileListMutex.lock();
	QFile xmlFile(gconf->confDir()+"filelist.xml" );
	QXmlInputSource source( &xmlFile );
	QXmlSimpleReader reader;
	reader.setContentHandler( &handler );

  if (!reader.parse( source ))
	{
		qDebug("QSharelist : liste de fichier partages corrompue! Creation d'un nouveau xml en cours....");
		FileListMutex.unlock();
		makeEmptyFileList();
		return result;
	}
	FileListMutex.unlock();
	return result;
}

/* SEARCHTHREAD */

void searchThread::init(t_queue q)
{
	head = q;
}

void searchThread::run()
{
	launchSearchFile();
}


void searchThread::launchSearchFile()
{
	QStringList list;
	QString name;
	t_result temp;
	QFileInfo f;
	t_searchResult::iterator it2;
	GfileMap::Iterator it;

	parent->result.clear();
	parent->lastSearch = head.name;
	list = QStringList::split(' ',head.name);

	if(head.type == 8)
	{
		mutex->lock();
		name = QDeepCopy<QString>(list.first());
		qDebug("recherche : "+ name);
		list.pop_front();
		for(unsigned int i=0;i<8;++i)
		{
        for ( it = parent->filelist[i].begin(); it != parent->filelist[i].end(); ++it )
					if((it.data().section("/",-1)).contains(name,FALSE) > 0)
					{
						qDebug("resultat : %s",f.fileName().latin1());
						f.setFile(it.data());
						temp.hash = it.key();
						temp.name = f.fileName();
						temp.size = f.size();
						temp.type = gconf->defineFileType(f.extension(FALSE));
						parent->result << temp;
					}
		}
		mutex->unlock();
	}
	else
	{
		mutex->lock();
		name = list.first();
		list.pop_front();
		for ( it = parent->filelist[head.type].begin(); it != parent->filelist[head.type].end(); ++it )
			if((it.data().section("/",-1)).contains(name,FALSE) > 0)
			{
				f.setFile(it.data());
				temp.hash = it.key();
				temp.name = f.fileName();
				temp.size = f.size();
				temp.type = gconf->defineFileType(f.extension(FALSE));
				parent->result << temp;
			}
		mutex->unlock();
	}
	while(!list.isEmpty())
	{
		name = list.first();

		it2 = parent->result.begin();
		while(it2!=parent->result.end())
		{
			if(!((*it2).name.contains(name,FALSE)))
			{
				it2 = parent->result.remove(it2);
				continue;
			}
			++it2;
		}
		list.pop_front();
	}

	QEvent * event;
	event = new QEvent(QEvent::User);
	QApplication::postEvent(parent, event);
}

void QSharelist::clearQMaps()
{
	mutex->lock();
	FileListMutex.lock();
	for(int i=0;i<8;++i)
		filelist[i].clear();
	mutex->unlock();
	FileListMutex.unlock();
}

//DEBUG
void QSharelist::QMapsInfos()
{
	mutex->lock();
	FileListMutex.lock();
	qDebug("Nombre de fichiers charges");
	for(int i=0;i<8;++i)
		qDebug("QMap %d : %d",i,filelist[i].count());
	mutex->unlock();
	FileListMutex.unlock();
}



void QSharelist::addNewFileToXml(const QString& filename, const QString& filehash, const QValueVector<QDomElement> & chunks)
{
	QString infos;
	QFileInfo f;
	unsigned int type = gconf->defineFileType(filename.section('.',-1));

	f.setFile(gconf->downloadDir()+filename);
	this->filelist[type][filehash] = f.absFilePath();
	QMapsInfos();

	if(!QFile::exists(gconf->confDir()+"filelist.xml" )
			|| !QFile::exists(gconf->confDir()+"filedb.xml" ) )
	{
		makeEmptyFileList();
	}
	updateFileDb(filename,filehash,type,f);
	updateFileList(filename,filehash,chunks,type,f);
}


void QSharelist::makeEmptyFileList()
{
	unsigned int i;
	GHashFile h;
	QDomElement xml,xml2;
	QDomDocument flistdb( "Gloster-Shared-Files" );
	QDomDocument flist("Gloster-File-List");
	QFile filedb,filelist;

	FileListMutex.lock();
	filedb.setName(gconf->confDir()+"filedb.xml" );
	filelist.setName(gconf->confDir()+"filelist.xml");
	filedb.open(IO_Truncate | IO_WriteOnly | IO_Translate);
	filelist.open(IO_Truncate | IO_WriteOnly | IO_Translate);
	QTextStream dbstream( &filedb );
	QTextStream liststream( &filelist);

	xml2 = flistdb.createElement("infos");
	xml2.setAttribute("nodeid",graphe->nodes()[0].nodeid);
	flistdb.appendChild(xml2);
	xml = flistdb.createElement("dir");
	xml.setAttribute("name","/");
	xml2.appendChild(xml);

	dbstream << flistdb.toString();
	filedb.close();

	xml2 = flist.createElement("hashes");

	h.readFromFile(gconf->confDir() +"filedb.xml");
	xml2.setAttribute("hash",h.getFileHash());
	for(i=0;i<h.getChunkNum();++i)
		xml2.setAttribute("p"+QString::number(i),h.getChunkHash(i));

	flist.appendChild(xml2);
	xml = flist.createElement("dir");
	xml.setAttribute("name","/");
	xml2.appendChild(xml);

	liststream << flist.toString();
	filelist.close();

	FileListMutex.unlock();
}

void QSharelist::updateFileList(const QString& filename, const QString& filehash, const QValueVector<QDomElement> & chunks, unsigned int type,QFileInfo & f)
{
	GHashFile h;
	QString infos,ht,enddir;
	unsigned int i=1;
	unsigned int j=0;
	long unsigned int n;
	QFile filelist(gconf->confDir()+"filelist.xml" );
	QTextStream stream(&filelist);
	char * endf=new char[filelist.size()];

	FileListMutex.lock();

	filelist.open(IO_ReadWrite | IO_Translate);
	while(i<=filelist.size() && (infos.left(6)!="</dir>" && infos.left(16)!="<dir name=\"/\" />"))
	{
		filelist.at(filelist.size()-i);
		infos.insert(0,filelist.getch());
		++i;
	}
	filelist.at(filelist.size()-i);

	if(infos.left(16)=="<dir name=\"/\" />")
	{
		filelist.readLine(endf,filelist.size());
		infos	 = " <dir name=\"/\" >\n";
		infos += " <file size=\"" + QString::number(f.size()) + "\"";
		enddir = "</dir>\n";
	}
	else
		infos = " <file size=\"" + QString::number(f.size()) + "\"";

	n = filelist.readBlock(endf,filelist.size());
	if(n<f.size())
		endf[n] = 0;

	for(j=0;j<chunks.count();++j)
		infos += " p"+QString::number(j)+"=\""+chunks[j].attribute("hash")+"\"";

	infos +=  " lastmodified=\"" + f.lastModified().toString()+"\" path=\"" + f.absFilePath() + "\" type=\"" + QString::number(type) +
							"\" name=\"" + filename + "\" hash=\"" + filehash +"\" />\n" + enddir + QString(endf);

	filelist.at(filelist.size()-i+1);
	filelist.writeBlock(infos,infos.length());

	stream.setEncoding(QTextStream::Latin1);
	stream.readLine();
	stream.readLine();
	ht = stream.read();

	h.readFromFile(gconf->confDir() +"filedb.xml");
	filelisthash = h.getFileHash();
	filelistsize = h.getFileSize();

	infos = "<hashes hash=\"" + filelisthash + "\" ";
	for(i=0;i<h.getChunkNum();++i)
		infos += "p"+QString::number(i) + "=\""+h.getChunkHash(i) + "\" ";
	infos += ">\n";
	filelist.at(0);
	stream.reset();
	stream.readLine();
	stream << infos + ht;
	delete endf;
	filelist.close();

	FileListMutex.unlock();
}

void QSharelist::updateFileDb(const QString& filename, const QString& filehash,unsigned int type,QFileInfo & f)
{
	QString infos;
	char endf[30];
	QString enddir;
	unsigned int i=1;
	long int n;

	FileListMutex.lock();
	QFile filedb(gconf->confDir()+"filedb.xml" );

	filedb.open(IO_ReadWrite | IO_Translate);
	infos = QString::null;
	while(i<=filedb.size() && (infos.left(6)!="</dir>" && infos.left(16)!="<dir name=\"/\" />"))
	{
		filedb.at(filedb.size()-i);
		infos.insert(0,filedb.getch());
		++i;
	}
	filedb.at(filedb.size()-i);

	if(infos.left(16)=="<dir name=\"/\" />")
	{
		filedb.readLine(endf,30);
		filedb.readLine(endf,30);
		infos	 = " <dir name=\"/\" >\n";
		enddir = "</dir>\n";
	}
	else
	{
		infos = QString::null;
		enddir = QString::null;
	}

	n = filedb.readBlock(endf,30);
	if(n<29)
		endf[n] = 0;

	infos += " <file size=\"" + QString::number(f.size()) + "\" type=\"" + QString::number(type) +
							"\" name=\"" + filename + "\" hash=\"" + filehash +"\""+" />\n" + enddir + QString(endf);

	filedb.at(filedb.size()-i+1);
	filedb.writeBlock(infos,infos.length());
	filedb.close();
	FileListMutex.unlock();
}


void QSharelist::updateConfRefreshShareList()
{
	delete refreshTimer;
	if(gconf->autoRefreshShares())
	{
		refreshTimer = new QTimer(this,"refreshTimer");
		switch(gconf->autoRefreshUnit())
		{
			case 0:
				refreshTimer->start(gconf->autoRefreshDelay()*60*1000);
				break;
			case 1:
				refreshTimer->start(gconf->autoRefreshDelay()*3600*1000);
				break;
			case 2:
				refreshTimer->start(gconf->autoRefreshDelay()*24*3600*1000);
		}
		connect(refreshTimer,SIGNAL(timeout()),gconf->mainWindow(),SLOT(refreshSharedFiles()));
	}
	else
		refreshTimer = NULL;
}

