/****************************************************************************
 *            Qsharelist.h
 *
 *  Tue Mar 29 20:10:35 2005
 *  Copyright  2005  z
 *  z@zzteam.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _QSHARELIST_H
#define _QSHARELIST_H

#include <qtimer.h>
#include <qobject.h>
#include <qdom.h>
#include <qstring.h>
#include <qmutex.h>
#include <qmap.h>
#include <qevent.h>
#include <qthread.h>
#include <qstringlist.h>
#include <qdir.h>
#include "../graphes.h"
#include "../gconfig.h"

extern c_graphes * graphe;
extern QMutex * mutex;
extern GConfig * gconf;
extern QMutex FileListMutex;

typedef QMap<QString, QString> GfileMap;
typedef struct
{
	 QString hash;
	 QString name;
	 unsigned int type;
	 Q_UINT64 size;
}t_result;
typedef struct
{
	 QString name;
	 unsigned int type;
}t_queue;

typedef QValueList<t_result> t_searchResult;
class QSharelist;


class searchThread : public QThread
{
	public:
		searchThread(QSharelist * p):QThread(),parent(p) {}
		void init(t_queue);
		virtual void run();
		void launchSearchFile();

		t_queue head;
		QSharelist * parent;
};



class QSharelist : public QObject
{
 Q_OBJECT

 public:
	 /* functions */
	QSharelist(QObject * parent=0);
 	~QSharelist();
 	void loadxmlfile();
  void initFileListInfos();

 	QString haveFile(const QString&, Q_UINT64, unsigned int = 8);
 	QStringList getHashPart(const QString&,Q_UINT64); // A FAIRE - retourne les hashs des parties
 	bool event ( QEvent * e );
 	QString getResult(t_searchResult& res);
 	const QString& fileListHash() {return filelisthash;}
	Q_UINT64 fileListSize() {return filelistsize;}
	const QString& fileListName() {return filelistname;}
	void addNewFileToXml(const QString& filename, const QString& filehash, const QValueVector<QDomElement> & chunks);
	QStringList getHashPartFileList();
	void setFileListSize(Q_UINT64 i) {filelistsize = i;}
	void setFileListHash(QString h) {filelisthash = h;}

 public slots:
	void updateConfRefreshShareList();
 	void searchFile(const QString& name, unsigned int type = 8);
 	void clearQMaps();
 	void QMapsInfos();

 signals :
 	void resultToReceive();

 private:
	/* fonctions */
 	void makeEmptyFileList();
 	void updateFileDb(const QString& filename, const QString& filehash, unsigned int type,QFileInfo & f);
 	void updateFileList(const QString& filename, const QString& filehash, const QValueVector<QDomElement> & chunks,unsigned int type,QFileInfo & f);

	/* variables */
 	unsigned int nb;
 	GfileMap filelist[8];
 	t_searchResult result;
 	QValueList<t_queue> queue;
 	bool dataRecovered;
 	searchThread * thread;
 	QString lastSearch;
 	QString filelisthash;
 	Q_UINT64 filelistsize;
 	QString filelistname;
	QTimer * refreshTimer;

	friend class Xmlparser;
 	friend class searchThread;
	friend class SFBThread;
	friend class shareFileBrowser;
};


#endif
