#ifndef GCONFIG_H
#define GCONFIG_H

#include <qapplication.h>
#include <qvaluelist.h>
#include <qmainwindow.h>
#include <qstring.h>
#include <qstringlist.h>
#include <qdict.h>
#include <qdir.h>
#include <qobject.h>
#include <qdict.h>
#include <qtranslator.h>

#define G_DOMAIN "Goatsoft"
#define G_PRODUCT "Gloster"

class QApplication;

union u_fcatResult{
	unsigned int *p;
	unsigned int c;
};

class GConfig: public QObject
{
	Q_OBJECT

	public :
		GConfig();
		~GConfig();

		inline QString applicationDirPath() { return applicationdirpath; }
		inline QMainWindow * mainWindow() { return MainWindow; }
		inline QTranslator * translator() { return Translator; }
		QString confDir() { return ConfDir; }
		bool isConfigured();

		int xPos();
		int yPos();
		int wSize();
		int hSize();
		bool isMaximized();

		inline QString nickname() { return Nick; }
		inline QString language() { return Language; }
		inline QString netKey() { return NetKey; }
		inline bool splashScreen() { return SplashScreen; }
		inline bool textUnderIcons() { return TextUnderIcons; }

		inline QString publicIp() { return PublicIp; }
		inline int conSpeed() { return ConSpeed; }
		inline int listenPort() { return ListenPort; }
		inline int slotCount() { return SlotCount; }
		inline int conType() { return ConType; }
		inline bool autoCon() { return AutoCon; }
		inline int keepCon() { return KeepCon; }
		QStringList ipList();
		QStringList portList();

		inline bool autoRefreshShares() { return AutoRefreshShares; }
		inline int autoRefreshUnit() { return AutoRefreshUnit; }
		inline int autoRefreshDelay() { return AutoRefreshDelay; }
		inline QString downloadDir() { return DownloadDir; }
		inline QString tempDir() { return TempDir; }

		inline bool autoAcceptKey() { return AutoAcceptKey; }
		QString pubKey(const QString& nodeid);
		inline QStringList nodeIds() { return Keys.keys(); }
		inline QStringList pubKeys() { return Keys.values(); }
		inline QString myPrivKey() { return MyPrivKey; }
		inline QString myPubKey() { return MyPubKey; }

		inline bool webActivated() { return WebActivated; }
		inline QString webPassword() { return WebPassword; }
		inline int webListenPort() { return WebListenPort; }

		inline QColor chatTextColor() { return ChatTextColor; }
		inline bool isChatTextBold() { return ChatTextBold; }
		inline bool isChatTextItalic() { return ChatTextItalic; }
		inline bool isChatTextUnderlined() { return ChatTextUnderlined; }

		inline bool isChatTimeStamp() { return ChatTimeStamp; }
		inline bool isChanTimeStamp() { return ChanTimeStamp; }
		inline bool isCustomTimeStamp() { return CustomTimeStamp; }
		inline QString timeStampString() { return TimeStampString; }

		unsigned int defineFileType(const QString& ext);
		static bool validateDir(const QString& dir);

	public slots :
		void setIsConfigured(bool b);
		void setApplicationDirPath(const QString & dir) { applicationdirpath = dir + '/'; }
		void setMainWindowPointer(QMainWindow * m) { MainWindow = m; }
		void setTranslator(QTranslator * t) { Translator = t; }

		void setXPos(int x);
		void setYPos(int y);
		void setWSize(int w);
		void setHSize(int h);
		void setIsMaximized(bool b);

		void setNickname(const QString& nickname);
		void setLanguage(const QString& lang);
		void setNetKey(const QString& key);
		void setSplashScreen(bool b);
		void setTextUnderIcons(bool b);

		void setPublicIp(const QString& ip);
		void setConSpeed(int s);
		void setListenPort(int port);
		void setSlotCount(int n);
		void setConType(int n);
		void setAutoCon(bool b);
		void setKeepCon(int n);
		void setIpList(const QStringList& l);
		void setPortList(const QStringList& l);

		void setAutoRefreshShares(bool b);
		void setAutoRefreshUnit(int unit);
		void setAutoRefreshDelay(int n);
		bool setDownloadDir(const QString& d);
		bool setTempDir(const QString& d);

		void setAutoAcceptKey(bool b);
		void setMyPrivKey(const QString& key);
		void setMyPubKey(const QString& key);
		void addPubKey(const QString& nodeid, const QString& key, bool save = 1);

		void setWebActivated(bool b);
		void setWebPassword(const QString& pass);
		void setWebListenPort(int n);

		void setChatTextColor(const QColor& color);
		void setIsChatTextBold(bool b);
		void setIsChatTextItalic(bool b);
		void setIsChatTextUnderlined(bool b);

		void setIsChatTimeStamp(bool b);
		void setIsChanTimeStamp(bool b);
		void setIsCustomTimeStamp(bool b);
		void setTimeStampString(const QString &ts);

		void writeTypes();
		void saveKeys();

		void save();
		void saveConf();
		void reload();
		void loadConf();

	private slots :
		void initfcat();
		void loadPublicKeys();

	signals :
		void updateConfTextUnderIcons(bool);

		void updateConfAutoCon();
		void updateConfKeepCon();

		void updateConfWebserver();
		void updateConfRefreshShareList();

	private :
	/* functions */
		QString colorToString(const QColor& color);
		void stringToColor(const QString& str, QColor& color);

	/* variables */
		QString applicationdirpath;
		QMainWindow * MainWindow;
		QTranslator * Translator;
		QString ConfDir;
		QString Nick;
		QString Language;
		QString NetKey;
		bool SplashScreen;
		bool TextUnderIcons;
		QString PublicIp;
		int ConSpeed;
		int ListenPort;
		int SlotCount;
		int ConType;
		bool AutoCon;
		int KeepCon;
		bool AutoRefreshShares;
		int AutoRefreshUnit;
		int AutoRefreshDelay;
		QString DownloadDir;
		QString TempDir;
		bool AutoAcceptKey;
		QString MyPrivKey;
		QString MyPubKey;
		bool WebActivated;
		QString WebPassword;
		int WebListenPort;
		QColor ChatTextColor;
		bool ChatTextBold;
		bool ChatTextItalic;
		bool ChatTextUnderlined;
		bool ChatTimeStamp;
		bool ChanTimeStamp;
		bool CustomTimeStamp;
		QString TimeStampString;
		QDict<unsigned int> cat;
		QMap<QString,QString> Keys;
};

#endif
