/***************************************************************************
 *            graphes.cpp
 *
 *  Tue Dec 21 13:20:44 2004
 *  Copyright  2004  z
 *  z@zzteam.net
 *
 * NOTE : ATTENTION, pour le cas de la suppression d'un sommet, il faudra
 *				gerer a part le fait que certains sommets n'aient plus d'arrete
 *				dans le graphe. Il faudra recreer une arete avec un autre sommet,
 *				et donc une connexion avec un autre client.
 *
 ****************************************************************************/

 /*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */



#include <stdio.h>
#include <string.h>
#include <qstring.h>
#include <qstringlist.h>
#include "graphes.h"


/***************************************************/
/**************** Public functions *****************/
/***************************************************/

/* Description :
   Createur de l'objet graph
*/
c_graphes::c_graphes(const QString& name,const char * nodeid,const QString& host, Q_UINT16 port, t_connexion ct, char cs,QObject * parent):QObject(parent)
{
	s_sommet sommet;
			sommet.name = name;
			memcpy(sommet.nodeid,nodeid,NODE_ID_SIZE*sizeof(char));
			sommet.host = host;
			sommet.port = port;
      sommet.ConnexionType = ct;
			sommet.ConnexionSpeed = cs;
			sommet.starcount = 0;
			sommet.arete = NULL;
			sommet.nbaretes = 0;
			sommet.blocked = FALSE;
			sommets_list.append(sommet);
      nbSommets = 1;

			Is_Modified = 1;
			pcc = NULL;
			equiv_table = (Q_UINT16 *) malloc(PAS_DU_REALLOC*sizeof(Q_UINT16));
			nodeidtoid.insert(nodeid,0);
}


/* Description :
	 Destructeur de l'objet graphe
*/
c_graphes::~c_graphes()
{
	p_arete T,U;
	Q_UINT16 i;

	for(i=0;i<sommets_list.size();i++)
	{
		if(sommets_list[i].arete!=NULL)
		{
			T = sommets_list[i].arete;
			while(T!=NULL)
			{
				U = T;
				T = T->suivant;
				free(U);
			}
		}
	}
	free(pcc);
	free(equiv_table);
}


/* Description :
	 Fonction de debug. Affiche le graph dans la console
*/
void c_graphes::show_graph()
{
	unsigned short int i;
	p_arete T;
	QString temp;

	qDebug("[graph.h] - Debug : Etat des connexions du graph\n");
	i=0;
	while(i<sommets_list.size())
	{
		temp = sommets_list[i].name + " : ";
		if(sommets_list[i].arete!=NULL)
		{
			T = sommets_list[i].arete;
			while(T!=NULL)
			{
				temp += sommets_list[T->sommet].name + ", ";
				T = T->suivant;
			}
		}
		qDebug(temp);
		i++;
	}
}

/* Description :
	Defini si un utilisateur est bloque ou pas.
	Si l'utilisateur est bloque, le client ne recoit plus les messages de chat 
	envoyes par cet utilisateur.
*/
void c_graphes::setBlocked(Q_UINT16 uid,bool b)
{
	sommets_list[uid].blocked = b;
	if(b)
		blockedNodeId << sommets_list[uid].nodeid;
	else
		blockedNodeId.remove(blockedNodeId.find(sommets_list[uid].nodeid));
}

bool c_graphes::isBlocked(Q_UINT16 uid)
{
	return sommets_list[uid].blocked;
}

/* Description :
	 Supprime un sommet dans le graphe

	 s : numero du sommet
*/
void c_graphes::Supprimer(Q_UINT16 s)
{
	SupprimerNode(s);
	nodeidtoid.remove(sommets_list[s].nodeid);
	emit suppnode(s);
	Check_composantesConnexes();
}

/* Description :
	 Supprime une arete entre 2 nodes passes en parametres
*/
void c_graphes::supp_connection(Q_UINT16 u, Q_UINT16 v)
{
	sommets_list[u].nbaretes -= 1;
	sommets_list[v].nbaretes -= 1;
	Supp_arete(&(sommets_list[v].arete), u);
	if((v>0) && (sommets_list[v].arete == NULL))
	{
			nbSommets -= 1;
			nodeidtoid.remove(sommets_list[v].nodeid);
			emit suppnode(v);
	}
	Supp_arete(&(sommets_list[u].arete), v);
	if((u>0) && (sommets_list[u].arete == NULL))
	{
			nbSommets -= 1;
			nodeidtoid.remove(sommets_list[u].nodeid);
			emit suppnode(u);
	}
	Check_composantesConnexes();
	Is_Modified = 1;
	emit graphe_modified();
}


/* Description :
	 Ajoute une connexion entre 2 sommets existants

	 s1,s2 : sommets entre lesquels on veut etablir une connexion
*/
bool c_graphes::Ajouter_Arete(Q_UINT16 s1, Q_UINT16 s2)
{
	if((s1!=s2) && (Ajouter_Arc(s1,s2)))
	{
		sommets_list[s1].nbaretes += 1;
		sommets_list[s2].nbaretes += 1;
		Ajouter_Arc(s2,s1);
		Is_Modified = 1;
		emit graphe_modified();
		return true;
	}
	return false;
}



/* Description :
   Ajoute un sommet dans le graphe

	s : Sommet existant avec lequel on cree un lien vers notre nouveau sommet
	cs : vitesse de connexion du sommet qu'on ajoute
	ct : type de connexion du sommet qu'on ajoute
*/
Q_UINT16 c_graphes::Ajouter_Sommet(Q_UINT16 s, const QString& name,const char * nodeid, const QString& host, Q_UINT16 port, t_connexion ct, char cs, bool starrecv, unsigned char starcount, class GSocket * socket)
{
	Q_UINT16 i;
	p_arete T;
	i=Get_SommetVide();
	qDebug(QString::number(i));
	if(!i)
	{
		s_sommet newSommet;
		sommets_list.append(newSommet);

		i = sommets_list.size()-1;
	}
	sommets_list[s].nbaretes += 1;
	sommets_list[i].name = name;

	memcpy(sommets_list[i].nodeid,nodeid,NODE_ID_SIZE*sizeof(char));
	sommets_list[i].host = host;
	sommets_list[i].port = port;
	sommets_list[i].ConnexionType = ct;
	sommets_list[i].ConnexionSpeed = cs;
	sommets_list[i].starcount = starcount;
	sommets_list[i].starrecv = starrecv;
	sommets_list[i].socket = socket;
	sommets_list[i].arete = NULL;
	sommets_list[i].nbaretes = 1;
	sommets_list[i].blocked = blockedNodeId.find(nodeid) != blockedNodeId.end();
	nodeidtoid.insert(nodeid,i);
	Create_arete(&(sommets_list[i].arete),s,sommets_list[s].ConnexionSpeed,sommets_list[s].ConnexionType);

	if(sommets_list[s].arete!=NULL)
	{
		T = sommets_list[s].arete;
		while(T->suivant!=NULL)
			T = T->suivant;
		Create_arete(&(T->suivant),i,cs,ct);
	}
	else
		Create_arete(&(sommets_list[s].arete),i,cs,ct);

	nbSommets += 1;
	Is_Modified = 1;
	emit graphe_modified();
	return i;
}


/* Description :
	 Dit si un sommet possede au moins une arete

	 s : numero du sommet
*/
char c_graphes::AUneArete(Q_UINT16 s)
{
	return(sommets_list[s].arete != NULL);
}


/* Description :
	 Give the first next node to go to a specific node

	 v : node you want to contact
*/
Q_UINT16 c_graphes::GoToNode(Q_UINT16 v,e_ParcoursMode mode)
{
	Update_pcc();
	switch(mode)
	  {
	  default:
	  case Delay:
	    while(pcc[v].pred_delay>0)
					v = pcc[v].pred_delay;
	    return(v);
	    break;

	  case Bandwith:
			if(pcc[v].pred_bw == INFINITY)
				return USHRT_MAX;
	    while(pcc[v].pred_bw>0)
					v = pcc[v].pred_bw;
	    return(v);
	    break;
	  }

}



/* Description :
   Construit les listes de broadcast
*/
s_broadcast_dc_list * c_graphes::Get_BroadcastLists()
{
  Q_UINT16 i,j;
  s_broadcast_dc_list * broadcast_list;
  Q_UINT16 * nbmaxlist;

  broadcast_list = (s_broadcast_dc_list *) malloc(sizeof(s_broadcast_dc_list));

  init_broadcast_list(broadcast_list);
  init_nbmaxlist(nbmaxlist,(*broadcast_list).nb_dc);

  for(i=1;i<sommets_list.size();i++)
		if(sommets_list[i].arete != NULL)
    	if(i != (j=GoToNode(i)))
      	update_bcastlist(broadcast_list,nbmaxlist,(equiv_table[j]),i);

  free(nbmaxlist);
  return broadcast_list;
}


/* Description :
	 Donne le nombre de nodes dans le graphe
*/
Q_UINT16 c_graphes::Get_nbnodes()
{
  return(nbSommets);
}


/* Description :
	 Fonction de multicast. traite une liste de nodes a qui envoyer des donnees.
	 Retourne une liste de nodes necessaire au broadcast

	 list : liste de nodes a qui renvoyer les donnes. Elle ne comprend que des nodes
					differentes du client.
*/
s_broadcast_dc_list * c_graphes::Get_MulticastLists(Q_UINT16 *& list, Q_UINT16 nb)
{
  Q_UINT16 i,j;
  Q_UINT16 * nbmaxlist;
  s_broadcast_dc_list * multicast_list;

  multicast_list = (s_broadcast_dc_list *) malloc(sizeof(s_broadcast_dc_list));

  init_multicast_list(multicast_list);
  init_nbmaxlist(nbmaxlist,(*multicast_list).nb_dc);

  for(i=0;i<nb;i++)
		if((j=GoToNode(list[i],Delay)))
    	update_bcastlist(multicast_list,nbmaxlist,equiv_table[j],list[i]);

  free(nbmaxlist);
  return multicast_list;
}

/* Description :
	 retourne un pointeur sur le tableau de sommet
*/
p_sommet& c_graphes::nodes()
{
	return(sommets_list);
}

s_sommet& c_graphes::nodes(Q_UINT16 n)
{
	return sommets_list[n];
}


bool c_graphes::uidExists(Q_UINT16 uid)
{
	if(uid<sommets_list.size())
		return sommets_list[uid].nbaretes;
	else
		return FALSE;
}

/* Description :
	 Renvoi l'id en fonction de la nodeid fournie en parametre
*/
Q_UINT16 c_graphes::NodeidToId(const char nodeid[NODE_ID_SIZE])
{
	return(node_exist(nodeid));
}


/* Description :
	 Cree l'etoile avec la liste de connection directe
*/
QString c_graphes::Get_Star()
{
	QString star;
	QString n;
	p_arete dc;
	s_sommet s;

	s = sommets_list[0];
	star += s.name;
	star += " ";
	star += s.nodeid;
	star += " ";
	star += s.host;
	star += " ";
	star += n.setNum(s.port);
	star += " ";
	star += n.setNum(s.ConnexionType);
	star += " ";
	star += n.setNum(s.ConnexionSpeed);
	star += " ";
	star += QString::number(++ (sommets_list[0].starcount));
	dc = sommets_list[0].arete;
	while(dc)
	{
		s = sommets_list[dc->sommet];
		star +=",";
		star += s.name;
		star += " ";
		star += s.nodeid;
		star += " ";
		star += s.host;
		star += " ";
		star += n.setNum(s.port);
		star += " ";
		star += n.setNum(s.ConnexionType);
		star += " ";
		star += n.setNum(s.ConnexionSpeed);
		dc = dc->suivant;
	}
	return star;
}


/* Description :
	 Recupere une etoile et met a jour le graphe en consequence
	 Renvoi Vrai si l'etoile comportait au moins une node connue, donc qu'elle etait valide.
	 Renvoi Faux sinon.
*/
bool c_graphes::Put_Star(QStringList & star)
{
	Q_UINT16 j,k;
	QStringList::Iterator it;

	it = star.begin();
	qDebug("Put_star : " + (*it).section(' ',1,1));
	if((j=node_exist(((*it).section(' ',1,1)).ascii())) == USHRT_MAX)
	{
		qDebug("Put_Star : %d",j);
		if((k=found_dc(star))<USHRT_MAX)
		{
			qDebug("Put_Star : %d",k);
			 emit addnode((j=Ajouter_Sommet(k
																		,((*it).section(' ',0,0))
																		,((*it).section(' ',1,1))
																		,((*it).section(' ',2,2))
																		,((*it).section(' ',3,3)).toUShort()
																		,(t_connexion)((*it).section(' ',4,4)).toInt()
																		,((*it).section(' ',5,5)).toInt()
																		,true,(*it).section(' ',6,6).toUShort())));
			Put_Star_new(j,star);
		}
		else
			return 0;
	}
	else
	{
		qDebug("Put_Star : %d",j);
		k=(*it).section(' ',6,6).toUShort();
		if ( (!sommets_list[j].starrecv)||(k>sommets_list[j].starcount)||((k<50)&&(200<sommets_list[j].starcount)) )
		{
			sommets_list[j].starrecv=true;
			sommets_list[j].starcount=k;
			Put_Star_old(j,star);
		}
	}

	return 1;
}


/* Description :
	 A partir du nodeID, definit si une node appartient au graphe
	 retoune le numero de la node dans le graphe si elle existe, USHRT_MAX sinon
*/
Q_UINT16 c_graphes::node_exist(const char nodeid[NODE_ID_SIZE])
{
			t_nodeidDict::Iterator it;
			if(( it= nodeidtoid.find(nodeid)) != nodeidtoid.end())
				return *it;
	return USHRT_MAX;
}


/*------------------ Fonctions de liberation ---------------*/


/* Description :
	 Libere proprement une liste necessaire au broadcast ou a l'unicast
*/
void c_graphes::Free_Castlist(s_broadcast_dc_list * L)
{
	Q_UINT16 i;

	if((L->dc)!=NULL)
	{
		for(i=0;i<L->nb_dc;i++)
			free((L->dc)[i].list);
		free(L->dc);
	}
	free(L);
}


/* Description :
	 Libere proprement une structure s_etoile
*/
/*
void c_graphes::Free_Star(s_etoile * star)
{
	delete(star);
}
*/

/****************************************************/
/**************** Private functions *****************/
/****************************************************/

/* Description :
   Retourne le numero de la premiere case vide rencontree
   dans le tableau de sommet.
 */
Q_UINT16 c_graphes::Get_SommetVide()
{
  Q_UINT16 i;

  if(nbSommets<sommets_list.size())
    {
			i = sommets_list.size()-1;
      while((i>0) && (sommets_list[i].arete!=NULL))
				i--;
			return i;
    }

  return 0;
}


/* Description :
	 ajoute un arete a la liste d'arete d'un sommet

	 P : arete que l'on veut creer
	 s : sommet vers lequel on veut creer un arete
	 cs : Vitesse de la connexion de s
	 ct : type de connexion de s
*/
void c_graphes::Create_arete(p_arete * P, Q_UINT16 s, char cs, t_connexion ct)
{
	(*P) = (p_arete) malloc(sizeof(s_arete));
	(*P)->sommet = s;
	(*P)->ConnexionSpeed = cs;
	(*P)->ConnexionType	= ct;
	(*P)->suivant = NULL;

}


/* Description :
	 Supprime un element de de la liste d'arete d'un sommet passee en parametre

	 P : Pointeur sur la liste d'arete d'un sommet
*/
void c_graphes::Supp_arete(p_arete * P, Q_UINT16 s)
{
	p_arete F,G=NULL;

	if(P!=NULL)
	{
		if((*P)->sommet==s)
		{
			F = (*P)->suivant;
			free((*P));
			(*P) = F;
		}
		else
		{
			F = (*P);
			while(F->sommet!=s)
			{
				G = F;
				F = F->suivant;
			}

			F = G->suivant->suivant;
			free(G->suivant);
			G->suivant = F;
		}
	}
}


/* Description :
	Initialize the table useful for the Dijkstra algorithm when we favor bandwith.
*/
void c_graphes::Parcours_init()
{
	Q_UINT16 i;
	p_arete T;

	pcc = (p_parcours) realloc(pcc, sommets_list.size()*sizeof(s_parcours));
	qDebug(QString::number(sommets_list.size()));
	for(i=0;i<sommets_list.size();i++)
	{
		pcc[i].Is_calculated = (sommets_list[i].arete==NULL);
		pcc[i].d_bw = INFINITY;
		pcc[i].pred_bw = -1;
		pcc[i].d_delay = INFINITY;
		pcc[i].pred_delay = -1;
	}
	pcc[0].d_bw = 0;
	pcc[0].d_delay = 0;
	pcc[0].pred_delay = 0;
	pcc[0].pred_bw = 0;

	T = sommets_list[0].arete;
	while(T)
	{
		pcc[T->sommet].pred_bw = 0;
		pcc[T->sommet].pred_delay = 0;
		pcc[T->sommet].d_bw = sommets_list[T->sommet].ConnexionSpeed;
		pcc[T->sommet].d_delay = sommets_list[T->sommet].ConnexionType;
		T = T->suivant;
	}
}


/* Description :
	 Ajoute un arc entre 2 sommets existants

	 s1,s2 : sommets entre lesquels on veut creer un arc
*/
char c_graphes::Ajouter_Arc(Q_UINT16 s1, Q_UINT16 s2)
{
	p_arete T;

	T = sommets_list[s1].arete;
	if(T)
	{
		while((T->suivant!=NULL) && (T->sommet!=s2))
			T = T->suivant;
		if(T->sommet==s2) return 0;
		T->suivant = (p_arete) malloc(sizeof(s_arete));
		T = T->suivant;
	}
	else
		T = sommets_list[s1].arete = (p_arete) malloc(sizeof(s_arete));
	T->sommet = s2;
	T->ConnexionType = sommets_list[s2].ConnexionType;
	T->ConnexionSpeed = sommets_list[s2].ConnexionSpeed;
	T->suivant = NULL;
	return 1;
}


/* Description :
	 Dijkstra algorithm. Calculate the shortest way to send information from the client itself to another
	 node in the graph.
*/
void c_graphes::Dijkstra(e_ParcoursMode mode)
{
	p_ParcoursList head;
	Q_UINT16 i;
	p_arete T;

	init_ParcoursList(&head);
	while(head)
	{
		i = Get_BestNodeBandwith(&head,mode);
		if(i == USHRT_MAX)
		{
			clearParcoursList(head);
			return;
		}
		T = sommets_list[i].arete;
		while(T)
		{
			relachement(i,T->sommet,mode);
			T = T->suivant;
		}
	}

	free_ParcoursList(head);
}


void c_graphes::clearParcoursList(p_ParcoursList head)
{
	p_ParcoursList temp;
	while(head)
	{
		temp = head->suivant;
		free(head);
		head = temp;
	}
}


/* Description :
	 update the table used to calculate the shortest way.
*/
void c_graphes::Update_pcc()
{
	if(Is_Modified)
	{
		Parcours_init();
		Dijkstra(Bandwith);
		Dijkstra(Delay);
		Is_Modified = 0;
	}
}


/* Description :
	 Function used to calculte the shortest way. Update the distance value
*/
void c_graphes::relachement(Q_UINT16 u, Q_UINT16 v, e_ParcoursMode mode)
{
	switch(mode)
	{
		case Bandwith:
			if(u>0 && sommets_list[u].ConnexionSpeed>0)
			{
				if(pcc[v].d_bw > (pcc[u].d_bw+sommets_list[v].ConnexionSpeed))
				{
					pcc[v].d_bw = pcc[u].d_bw+sommets_list[v].ConnexionSpeed;
					pcc[v].pred_bw = u;
				}
			}
			break;

		default:
		case Delay:
			if(pcc[v].d_delay > (pcc[u].d_delay+sommets_list[v].ConnexionType))
			{
				pcc[v].d_delay = pcc[u].d_delay+sommets_list[v].ConnexionType;
				pcc[v].pred_delay = u;
			}
			break;
	}
}

/* Description :
	 Create a list necessary for the shortest way calculation
*/
void c_graphes::init_ParcoursList(p_ParcoursList * head)
{
	Q_UINT16 i=1;
	p_ParcoursList T;

	while((i<sommets_list.size()) && (sommets_list[i].arete==NULL))
		i++;

	if(i==sommets_list.size())
	{
		(*head) = NULL;
		return;
	}
	(*head) = (p_ParcoursList) malloc(sizeof(s_ParcoursList));
	(*head)->i = i;
	(*head)->pred = NULL;
	(*head)->suivant = NULL;
	T = (*head);
	i++;
	while(i<sommets_list.size())
	{
		if(sommets_list[i].arete!=NULL)
		{
			T->suivant = (p_ParcoursList) malloc(sizeof(s_ParcoursList));
			(T->suivant)->i = i;
			(T->suivant)->pred = T;
			T = T->suivant;
		}
		i++;
	}
	T->suivant = NULL;
}


/* Description :
	 Function to clean the list created to calculate the shortest way
*/
void c_graphes::free_ParcoursList(p_ParcoursList head)
{
	p_ParcoursList T,temp;
	T = head;
	while(T!=NULL)
	{
		temp = T->suivant;
	 	free(T);
		T = temp;
	}
}


/* Description :
	 Get the # of the node which has the best bandwith

	 head : list's head
*/
Q_UINT16 c_graphes::Get_BestNodeBandwith(p_ParcoursList * head, e_ParcoursMode mode)
{
	float min=INFINITY;
	p_ParcoursList min_p=NULL;
	p_ParcoursList T;
	Q_UINT16 i;

	T = (*head);
	while(T)
	{
		Check_BestNodeBandwith(T,&min,&min_p,mode);
		T = T->suivant;
	}
	if(!min_p)
		return USHRT_MAX;

	if(min_p->pred==NULL)
	{
		(*head) = min_p->suivant;
		if(min_p->suivant!=NULL)
			(min_p->suivant)->pred = NULL;
	}
	else
	{
		(min_p->pred)->suivant = min_p->suivant;
		if(min_p->suivant!=NULL)
			(min_p->suivant)->pred = min_p->pred;
	}
	i = min_p->i;
	free(min_p);
	return(i);
}


void c_graphes::Check_BestNodeBandwith(p_ParcoursList T,float * min, p_ParcoursList * min_p, e_ParcoursMode mode)
{
	if(mode == Bandwith)
	{
		if((pcc[T->i].d_bw)<(*min))
		{
			(*min) = pcc[T->i].d_bw;
			(*min_p) = T;
		}
	}
	else
	{
		if((pcc[T->i].d_delay)<(*min))
		{
			(*min) = pcc[T->i].d_delay;
			(*min_p) = T;
		}
	}
}

/* Description :
	 Check if a node is still in the list

	 head : list's head
	 i : # node we search
*/
char c_graphes::is_in_ParcoursList(p_ParcoursList head,Q_UINT16 i)
{
	while((head!=NULL) && (head->i!=i))
		head = head->suivant;

	return(head!=NULL);
}



/* Description :
	 initialise le tableau temporaire necessaire a l'envoi de broadcast

	 tab : liste contenant l'id de chaque node
	 Is_InList : tableau a initialiser
*/
void c_graphes::init_broadcast_list(s_broadcast_dc_list *& tab)
{
	p_arete S;

	S = sommets_list[0].arete;
	(*tab).dc = (p_broadcast_dc) malloc(MIN_BC_LIST*sizeof(s_broadcast_dc));
	equiv_table = (Q_UINT16 *) realloc(equiv_table,sommets_list.size()*sizeof(Q_UINT16));
	(*tab).nb_dc = 0;

	while(S)
	{
		if((*tab).nb_dc%MIN_BC_LIST == 0)
			(*tab).dc = (p_broadcast_dc) realloc((*tab).dc,((*tab).nb_dc+MIN_BC_LIST)*sizeof(s_broadcast_dc));

		(*tab).dc[(*tab).nb_dc].i = S->sommet;
		(*tab).dc[(*tab).nb_dc].list = (Q_UINT16 *) malloc(MIN_DC_LIST*sizeof(Q_UINT16));
		(*tab).dc[(*tab).nb_dc].list[0] = S->sommet;
		(*tab).dc[(*tab).nb_dc].nbnodes = 1;
		equiv_table[S->sommet] = (*tab).nb_dc;
		(*tab).nb_dc += 1;
		S = S->suivant;
	}
	(*tab).dc = (p_broadcast_dc) realloc((*tab).dc,((*tab).nb_dc)*sizeof(s_broadcast_dc));
}


/* Description :
	 initialise le tableau temporaire necessaire a l'envoi de multicast

	 tab : liste contenant l'id de chaque node
	 Is_InList : tableau a initialiser
*/
void c_graphes::init_multicast_list(s_broadcast_dc_list *& tab)
{
	p_arete S;

	S = sommets_list[0].arete;
	(*tab).dc = (p_broadcast_dc) malloc(MIN_BC_LIST*sizeof(s_broadcast_dc));
	equiv_table = (Q_UINT16 *) realloc(equiv_table,sommets_list.size()*sizeof(Q_UINT16));
	(*tab).nb_dc = 0;

	while(S)
	{
		if((*tab).nb_dc%MIN_BC_LIST == 0)
			(*tab).dc = (p_broadcast_dc) realloc((*tab).dc,((*tab).nb_dc+MIN_BC_LIST)*sizeof(s_broadcast_dc));

		(*tab).dc[(*tab).nb_dc].i = S->sommet;
		(*tab).dc[(*tab).nb_dc].list = (Q_UINT16 *) malloc(MIN_DC_LIST*sizeof(Q_UINT16));
		(*tab).dc[(*tab).nb_dc].nbnodes = 0;
		equiv_table[S->sommet] = (*tab).nb_dc;
		(*tab).nb_dc += 1;
		S = S->suivant;
	}
	(*tab).dc = (p_broadcast_dc) realloc((*tab).dc,((*tab).nb_dc)*sizeof(s_broadcast_dc));
}


/* Description :
	 retourne une liste de node pour aller de la node 0 a la node v

	 v : Node dont on veut le plus court chemin
	 Is_InList : tableau de booleen indiquant si une node fait deja partie d'une liste
*/
p_broadcast_list c_graphes::GoToNode_list(Q_UINT16 v,char * Is_InList)
{
	s_broadcast_list * L;

	L = (s_broadcast_list *) malloc(sizeof(s_broadcast_list));

	Update_pcc();
	(*L).next = (Q_UINT16 *) malloc(MIN_BC_LIST*sizeof(Q_UINT16));
	(*L).nb = 0;

	if(pcc[v].pred_delay>-1)
	{
		printf("%d : ",v);
		while(v>0)
		{
			if(Is_InList[v]<2)
			{
				if((*L).nb%MIN_BC_LIST == 0)
					(*L).next = (Q_UINT16 *) realloc((*L).next,(MIN_BC_LIST+(*L).nb)*sizeof(Q_UINT16));
				(*L).next[(*L).nb] = v;
				printf("%d",v);
				(*L).nb += 1;
				if(Is_InList[v] == 0) Is_InList[v] = 2;
			}
			v = pcc[v].pred_delay;
		}
	}
	printf("\n");
	(*L).next = (Q_UINT16 *) realloc((*L).next,(*L).nb*sizeof(Q_UINT16));
	return(L);
}


/* Description :
   Initialise le tableau qui indique la taille maximale des vecteurs de la liste de broadcast

   nbmaxlist : liste d'entier correspondant a la taille max des listes de broadcast
   nb_dc : nombre de nodes vers lesquels on va envoyer les listes
*/

void c_graphes::init_nbmaxlist(Q_UINT16 *& nbmaxlist, Q_UINT16 nb_dc)
{
  Q_UINT16 i;

  nbmaxlist = (Q_UINT16 *) malloc(nb_dc*sizeof(Q_UINT16));
  for(i=0;i<nb_dc;i++)
    nbmaxlist[i] = MIN_DC_LIST;
}



/* Description :
	 rajoute une node a la liste de nodes a broadcaster

	 L : Liste des des nodes a broadcaster
 nbmaxlist : contient la taille maximale des listes de broadcast
	 i : connection directe sur laquelle on veut ajouter une node a qui envoyer un paquet
	 j : numero de la node qu'on veut rajouter dans la liste de broadcast de la node i
*/
void c_graphes::update_bcastlist(s_broadcast_dc_list *& L, Q_UINT16 * nbmaxlist, Q_UINT16 i, Q_UINT16 j)
{
  (*L).dc[i].nbnodes += 1;

  if(nbmaxlist[i] < (*L).dc[i].nbnodes)
    {
      (*L).dc[i].list = (Q_UINT16 *) realloc((*L).dc[i].list,((*L).dc[i].nbnodes-1)*sizeof(Q_UINT16));
      nbmaxlist[i] = (*L).dc[i].nbnodes-1+MIN_DC_LIST;
    }
  (*L).dc[i].list[(*L).dc[i].nbnodes-1] = j;
}


/* Description :
	 Verifie si la node i possede deja une connexion vers la node passee en parametre
*/
char c_graphes::arete_exist(Q_UINT16 k,Q_UINT16 i)
{
	p_arete L;

	L = sommets_list[i].arete;
	while(L)
	{
		if((L->sommet) == k)
			return 1;
		L = L->suivant;
	}
	return 0;
}


/* Description :
	 Verifie si une node se trouve dans la liste d'une etoile
*/
Q_UINT16 c_graphes::is_instar(Q_UINT16 som, QStringList star)
{
	QStringList::Iterator it=star.begin();
	Q_UINT16 i=1;

	for(++it;it!=(star.end());++it)
	{
		if(((*it).section(' ',1,1)) == sommets_list[som].nodeid)
			return i;
		i++;
	}
	return USHRT_MAX;
}



/* Description :
	 Met a jour le graphe en utilisant une etoile. Sert dans le cas ou la node emettrice n'existait pas au depart
*/
void c_graphes::Put_Star_new(Q_UINT16 j, QStringList star)
{
	QStringList::Iterator it=star.begin();
	Q_UINT16 k;

	for(++it;it!=star.end();++it)
	{
		if((k=node_exist(((*it).section(' ',1,1)))) < USHRT_MAX)
		{
			if(!arete_exist(k,j))
				Ajouter_Arete(k,j);
		}
		else
		 emit addnode((k=Ajouter_Sommet(j
														,((*it).section(' ',0,0))
														,((*it).section(' ',1,1))
														,((*it).section(' ',2,2))
														,((*it).section(' ',3,3)).toUShort()
														,(t_connexion)((*it).section(' ',4,4)).toInt()
														,((*it).section(' ',5,5)).toInt())));
	}
}



/* Description :
	 Met a jour le graphe en utilisant une etoile. Sert dans le cas ou la node emettrice n'existait pas au depart
*/
void c_graphes::Put_Star_old(Q_UINT16 j, QStringList star)
{
	QStringList::Iterator it=star.begin();
	Q_UINT16 i,l,k;
	Q_UINT16 * is_added;
	bool nonconnexe = true;
	p_arete L;

	is_added = (Q_UINT16 *) calloc(star.count(),sizeof(Q_UINT16));
	L = sommets_list[j].arete;
	while(L)
	{
		if((i=is_instar((l=L->sommet),star)) == USHRT_MAX)
		{
			L = L->suivant;
			supp_connection(l,j);
		}
		else
		{
				nonconnexe = false;
				is_added[i] = 1;
				L = L->suivant;
		}
	}
	if(nonconnexe)
	{
		free(is_added);
		return;
	}
	i = 1;
	for(++it;it!=star.end();++it)
	{
		if(!is_added[i])
		{
			if((k=node_exist((*it).section(' ',1,1))) == USHRT_MAX)
			 emit addnode((k=Ajouter_Sommet(j
											,((*it).section(' ',0,0))
											,((*it).section(' ',1,1))
											,((*it).section(' ',2,2))
											,((*it).section(' ',3,3)).toUShort()
											,(t_connexion)((*it).section(' ',4,4)).toInt()
											,((*it).section(' ',5,5)).toInt())));
			Ajouter_Arete(k,j);
		}
		i++;
	}
	free(is_added);
}

/* Description :
	 Elimine toutes les nodes qui ne sont pas dans la meme composante connexe.
*/
void c_graphes::Check_composantesConnexes()
{
	Q_UINT16 i;
	Q_UINT16 * 	is_incc;

	is_incc = (Q_UINT16 *) malloc(sommets_list.size()*sizeof(Q_UINT16));
		for(i=0;i<sommets_list.size();i++)
			is_incc[i] = 0;

	parcoursprof(0,is_incc);
	for(i=1;i<sommets_list.size();i++)
		if((!is_incc[i]) && (sommets_list[i].arete!=NULL))
		{
			SupprimerNode(i);
			emit suppnode(i);
		}
	free(is_incc);
}


/* Description :
	 Effectue un parcours en profondeur du graphe et marque les sommets dans le tableau passe en parametre
*/
void c_graphes::parcoursprof(Q_UINT16 k, Q_UINT16 * is_incc)
{
	p_arete T;

	is_incc[k] = 1;
	T = sommets_list[k].arete;
	while(T)
	{
		if(!is_incc[T->sommet])
			parcoursprof(T->sommet,is_incc);
		T = T->suivant;
	}
}


/* Description :
	 Supprime une node du graphe sans emettre de signals (QT)
*/
void c_graphes::SupprimerNode(Q_UINT16 s)
{
	p_arete T,P;

	if(s>0)
	{
		T = sommets_list[s].arete;
		while(T!=NULL)
		{
			sommets_list[s].nbaretes -= 1;
			P = T->suivant;
			Supp_arete(&(sommets_list[s].arete),T->sommet);
			T = P;
		}
		sommets_list[s].arete = NULL;
		nbSommets -= 1;
		Is_Modified = 1;
		emit graphe_modified();
		nodeidtoid.remove(sommets_list[s].nodeid);
	}
}


/* Description :
	 Cherche une node preexistante dans le graphe a partir d'une etoile
	 retourne le numero de la node dans le graphe si elle existe, USHRT_MAX sinon
*/
Q_UINT16 c_graphes::found_dc(QStringList list)
{
	QStringList::Iterator it=list.begin();
	Q_UINT16 i;

	for(++it;it!=(list.end());++it)
		if(((i = node_exist((*it).section(" ",1,1))) < USHRT_MAX))
			return i;

  return USHRT_MAX;
}
