/***************************************************************************
 *            graphes.h
 *
 *  Tue Dec 21 13:19:01 2004
 *  Copyright  2004  z
 *  z@zzteam.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GRAPHES_H
#define _GRAPHES_H

#define PAS_DU_REALLOC 5
#define MIN_CONNECT_REALLOC 25
#define MIN_BC_LIST 15
#define MIN_DC_LIST 5

#include <stdio.h>
#include <qobject.h>
#include <qstring.h>
#include <qdeepcopy.h>
#include <qmap.h>
#include <qvaluevector.h>

#ifdef Q_WS_WIN
	#include <float.h>
	#ifndef INFINITY
	#define INFINITY _FPCLASS_PINF
	#endif
#endif

#ifndef INFINITY
#include <math.h>
#endif

#include "gtypes.h"
#include "network/gsocket.h"

enum e_ParcoursMode {Delay, Bandwith};
enum e_couleur {Blanc, Gris, Noir};

typedef QValueVector<struct s_sommet> p_sommet;
typedef struct s_arete * p_arete;
typedef struct s_parcours * p_parcours;
typedef struct s_ParcoursList * p_ParcoursList;
typedef struct s_broadcast_dc * p_broadcast_dc;
typedef struct s_broadcast_list * p_broadcast_list;
typedef QMap<QString,Q_UINT16> t_nodeidDict;


/* Description des types s_sommet et s_arete

	ConnexionType : Correspond au type de connexion de l'utilisateur. La valeur correspond a un type enum.
									Cette information permet de valuer le graphe. Cette valuation permet de favoriser le ping
									en fonction de la connexion.

	ConnexionSpeed : Cette variable contient une information sur la limite d'upload de l'utilisateur. En fonction
									 de la limite qu'il a defini, cette variable prend une valeur plus ou moins eleve. Dans le cas
									 ou elle vaut 0, l'utilisateur n'autorise pas le routage des paquets par sa connection.

	Voici le tableau de correspondance pour les valeurs de ConnexionSpeed :
		0 : pas de routage
		1 : illimite
		2 : >1000 Ko/s
		3 : 500<x<=1000 Ko/s
		4 : 100<x<=500 Ko/s
		5 : 50<x<=100 Ko/s
		6 : 25<x<=50 Ko/s
		7 : 10<x<=25 Ko/s
		8 : 5<x<=10 Ko/s
		9 : 1<x<=5 Ko/s

*/


typedef struct s_sommet
{
	QDeepCopy<QString> name;
	char nodeid[NODE_ID_SIZE];
	QDeepCopy<QString> host;
	Q_UINT16 port;
	t_connexion ConnexionType;
	char ConnexionSpeed;
	unsigned char starcount;
	bool starrecv;
	p_arete arete;
	Q_UINT16 nbaretes;
	class GSocket * socket;
	bool blocked;
} s_sommet;


typedef struct s_arete
{
	t_connexion ConnexionType;
	char ConnexionSpeed;
	Q_UINT16 sommet;
	p_arete suivant;
}s_arete;


typedef struct s_parcours
{
	char Is_calculated;
	float d_delay;
	Q_INT32 pred_delay;
	float d_bw;
	Q_INT32 pred_bw;
}s_parcours;

typedef struct s_ParcoursList
{
	Q_UINT16 i;
	p_ParcoursList pred;
	p_ParcoursList suivant;
}s_ParcoursList;

typedef struct s_broadcast_dc
{
	Q_UINT16 i;
	Q_UINT16 nbnodes;
	Q_UINT16 * list;
}s_broadcast_dc;

typedef struct s_broadcast_dc_list
{
	Q_UINT16 nb_dc;
	p_broadcast_dc dc;
}s_broadcast_dc_list;

typedef struct s_broadcast_list
{
	Q_UINT16 nb;
	Q_UINT16 * next;
}s_broadcast_list;


class c_graphes : public QObject
{
	Q_OBJECT

	public:
	/*functions*/
		c_graphes(const QString&,const char *, const QString&, Q_UINT16, t_connexion, char,QObject * parent=0);
		~c_graphes();
 		void show_graph();
		void setBlocked(Q_UINT16 uid,bool b);

 	/* Modification du graphe */
 		Q_UINT16 Ajouter_Sommet(Q_UINT16, const QString&, const char *, const QString&, Q_UINT16, t_connexion, char, bool = false, unsigned char=0, class GSocket * =NULL);
  		bool Ajouter_Arete(Q_UINT16, Q_UINT16);
  		void Supprimer(Q_UINT16);
  		char AUneArete(Q_UINT16);
  		void supp_connection(Q_UINT16 u, Q_UINT16 v);

 	/* Fonctions de recuperation de donnees */
 		Q_UINT16 NodeidToId(const char *);
	 	Q_UINT16 Get_nbnodes();
	 	p_sommet& nodes();
		s_sommet& nodes(Q_UINT16);
	 	s_broadcast_dc_list * Get_MulticastLists(Q_UINT16 *& list, Q_UINT16 nb); /* Multicast */
		s_broadcast_dc_list * Get_BroadcastLists(); /* Broadcast */
		Q_UINT16 GoToNode(Q_UINT16, e_ParcoursMode mode=Delay); /* unicast */
		QString Get_Star();
		bool Put_Star(QStringList & star);
		Q_UINT16 node_exist(const char *);
		bool uidExists(Q_UINT16);
		bool isBlocked(Q_UINT16);

	/* Fonctions de liberation */
		void Free_Castlist(s_broadcast_dc_list *);

	/*Variables*/
		QValueVector<s_sommet> sommets_list;

	private:
	/*functions*/
		Q_UINT16 Get_SommetVide();
		void Create_arete(p_arete *, Q_UINT16, char, t_connexion);
		void Supp_arete(p_arete *, Q_UINT16);
		void Parcours_init();
		char Ajouter_Arc(Q_UINT16, Q_UINT16);
		void Dijkstra(e_ParcoursMode);
		void init_ParcoursList(p_ParcoursList *);
		void free_ParcoursList(p_ParcoursList);
		Q_UINT16 Get_BestNodeBandwith(p_ParcoursList *, e_ParcoursMode);
		void Check_BestNodeBandwith(p_ParcoursList,float *, p_ParcoursList *, e_ParcoursMode);
		char is_in_ParcoursList(p_ParcoursList, Q_UINT16);
		void relachement(Q_UINT16, Q_UINT16, e_ParcoursMode);
		void Update_pcc();
		void init_broadcast_list(s_broadcast_dc_list *&);
		void init_multicast_list(s_broadcast_dc_list *& tab);
		p_broadcast_list GoToNode_list(Q_UINT16,char *);
		void update_bcastlist(s_broadcast_dc_list *& L, Q_UINT16 * nbmaxlist, Q_UINT16 i, Q_UINT16 j);
		void init_nbmaxlist(Q_UINT16 *& nbmaxlist,Q_UINT16 nb);
		char arete_exist(Q_UINT16, Q_UINT16);
		Q_UINT16 is_instar(Q_UINT16, QStringList);
		void Put_Star_new(Q_UINT16, QStringList);
		void Put_Star_old(Q_UINT16, QStringList);
		void Check_composantesConnexes();
		void parcoursprof(Q_UINT16 k, Q_UINT16 * is_incc);
		void SupprimerNode(Q_UINT16 s);
		Q_UINT16 found_dc(QStringList list);
		void clearParcoursList(p_ParcoursList head);
	
	/*Variables*/
		char Is_Modified;
		p_parcours pcc;
		Q_UINT16 nbSommets;/* Nombre de sommet dans le graph */
		Q_UINT16 * equiv_table;
		t_nodeidDict nodeidtoid;
		QStringList blockedNodeId;

	/* signals pour QT */
	signals:
  		void suppnode(Q_UINT16);
		void addnode(Q_UINT16);
		void checkcompconnexes();
		void graphe_modified();
};

#endif
