#ifndef DIRVIEW_H
#define DIRVIEW_H

#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtimer.h>
#include <qpixmap.h>

class QWidget;
class QDragEnterEvent;
class QDragMoveEvent;
class QDragLeaveEvent;
class QDropEvent;

extern QPixmap * folderLocked;
extern QPixmap * folderClosed;
extern QPixmap * folderOpen;
extern QPixmap * fileNormal;

enum item_type {item_special=0,item_dir,item_file};

class ListItem : public QListViewItem
{
	public :
		ListItem(QListView *parent, const QString &s1, const QString& size = QString::null, QPixmap * px = 0, item_type t = item_file);
		ListItem(QListViewItem *parent, const QString &s1, const QString& size = QString::null, QPixmap * px = 0, item_type t= item_file);

		void setPixmap( QPixmap * p );			
		const QPixmap * pixmap(int i) const;

		inline void setType(item_type t) { setText(2,QString::number((int)t)); }
		inline void setSize(Q_UINT64 s) { setText(1,QString::number(s)); }
		inline item_type type() { return (item_type) text(2).toInt(); }
		inline QString name() { return text(0); }
		inline Q_UINT64 size() { return text(1).toULongLong(); }
		
	private :
		QPixmap * pix;
};

class FileItem : public ListItem
{
	public :
		FileItem(QListView *parent, const QString &s1, const QString& apath, const QString& size = QString::null, QPixmap * px = 0);
		FileItem(QListViewItem *parent, const QString &s1, const QString& apath, const QString& size = QString::null, QPixmap * px = 0);

		QString absFilePath() { return AbsPath; }

	protected:
		QString AbsPath;
};

class Directory : public FileItem
{
	public:
		Directory( QListViewItem * parent, const QString& filename, const QString& apath);
		Directory( QListView * parent, const QString& filename, const QString& apath, bool special = FALSE);

		void setOpen( bool );
		void setup();
};


class DirectoryView : public QListView
{
	Q_OBJECT

	public:
			DirectoryView( QWidget *parent = 0, const char *name = 0);

	public slots:
			void openDir(const QString&);
			void closeDir();

	protected:
			void contentsMouseMoveEvent( QMouseEvent *e );
			void contentsMousePressEvent( QMouseEvent *e );
			void contentsMouseReleaseEvent( QMouseEvent *e );
			void addSelectedItem(QStringList& list, QListViewItem * item);

			QListViewItem *oldCurrent;
			QListViewItem *currentdir;
			bool mousePressed;
};


class dirView : public DirectoryView
{
	Q_OBJECT

	public :
		dirView( QWidget *parent = 0, const char *name = 0)
			:DirectoryView(parent,name) {setSortColumn(2);}

	public slots :
		void displayFolder(QListViewItem *);

	signals :
		void newdir(const QString &);
		void closedir();

	protected :
		void contentsMouseMoveEvent( QMouseEvent* e );
		void contentsMouseDoubleClickEvent ( QMouseEvent * e );
		void qdisplayFolder(QString path);
};

#endif
