#include <qapplication.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qdir.h>
#include "qsharefileview.h"


/*****************************************************************************
 *
 * Class vFileBrowser
 *
 *****************************************************************************/
/*------------------ VFileBrowser - public  ------------------- */
vFileBrowser::vFileBrowser(QWidget * parent, QString xmlpath, bool activeDrop, const char * name, WFlags f)
	:QWidget (parent,name,f),activedrop(activeDrop)
{
	if(!folderLocked)
	{
		folderLocked = new QPixmap(QPixmap::fromMimeSource("file_locked.png"));
		folderClosed = new QPixmap(QPixmap::fromMimeSource("folder.png"));
		folderOpen = new QPixmap(QPixmap::fromMimeSource("folder_open.png"));
		fileNormal = new QPixmap(QPixmap::fromMimeSource("documents.png"));
	}

	QBoxLayout * vBrowserGrid= new QBoxLayout(this,QBoxLayout::LeftToRight,6);

	setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
	setBaseSize(452,250);

  	nameSpace = new  vDirectoryView( this, "virtual_browser",activeDrop);
	nameSpace->setBaseSize(160,250);
	nameSpace->setMinimumSize(160,250);
	nameSpace->setResizeMode(QListView::AllColumns);
	vBrowserGrid->addWidget(nameSpace);
	nameSpace->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
	hideVbutton = new QPushButton("<",this);

	hideVbutton->setBaseSize(15,250);
	hideVbutton->setFixedWidth(15);
	vBrowserGrid->addWidget(hideVbutton);
	hideVbutton->setSizePolicy(QSizePolicy::Minimum,QSizePolicy::Expanding);

	if(!loadXmlFile(xmlpath))
		initNameSpace();

 	vfileViewer = new vDirView(this,"virtual_file_browser",activeDrop);
	vfileViewer->setBaseSize(225,250);
	vfileViewer->setMinimumSize(225,250);
	vfileViewer->setResizeMode(QListView::AllColumns);
	vBrowserGrid->addWidget(vfileViewer);
	vfileViewer->setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);

	connect(nameSpace,SIGNAL(selectionChanged(QListViewItem *)),nameSpace,SLOT(validateSelectionChanged(QListViewItem *)));
	connect(nameSpace,SIGNAL(transmitSelectionChanged(QListViewItem *)),this,SLOT(updateFileview(QListViewItem *)));
	connect(hideVbutton,SIGNAL(pressed()),this,SLOT(hideVfv()));
	connect(nameSpace,SIGNAL(disableVSpace()),this,SLOT(disableVSpace()));
	connect(nameSpace,SIGNAL(enableVSpace()),this,SLOT(enableVSpace()));
	connect(vfileViewer,SIGNAL(disableVSpace()),this,SLOT(disableVSpace()));
	connect(vfileViewer,SIGNAL(enableVSpace()),this,SLOT(enableVSpace()));
	connect(vfileViewer,SIGNAL(transmitSelectionChanged(QListViewItem *)),this,SLOT(updateFileview(QListViewItem *)));
	connect(vfileViewer,SIGNAL(forceUpdatevDirView(QListViewItem*)),this,SLOT(updateFileview(QListViewItem *)));
	connect(vfileViewer,SIGNAL(decNbFiles(double)),nameSpace,SLOT(decNbFiles(double)));

	updateFileview(nameSpace->firstChild());
}


/* ---------------- vFileBrowser - private ----------------- */
bool vFileBrowser::loadXmlFile(QString & xmlpath)
{
	 xmlFile.setName(xmlpath);
	if(xmlFile.exists())
	{
		QXmlInputSource source( &xmlFile );
		QXmlSimpleReader reader;
		handler = new StructureParser(nameSpace,TRUE);
		reader.setContentHandler( handler );
		return reader.parse( source );
	}
	return false;
}

void vFileBrowser::initNameSpace()
{
	(void) new vListItem(nameSpace,"/",QString::null,folderOpen,item_special);
	nameSpace->firstChild()->setText(0, "/");
	nameSpace->firstChild()->setRenameEnabled(0,false);
	nameSpace->firstChild()->setOpen(true);
}

/**********************************/
/* ---------------- SLOTS ----------------- */
/*********************************/
void vFileBrowser::updateFileview(QListViewItem * newselect)
{
	QListViewItem * myChild;
	ListItem * vdir;

	vfileViewer->clear();
	vfileViewer->setSortColumn(0);
	if(newselect->parent())
	{
		vdir = new ListItem((QListView *)vfileViewer,"..",QString::null,folderClosed,item_special);
		vdir->setRenameEnabled (0,FALSE);
	}

	myChild = newselect->firstChild();
	while( myChild )
	{

		if(((vListItem*) myChild)->type() == item_dir)
		{
			vdir = new ListItem((QListView*)vfileViewer,myChild->text(0),QString::null,folderClosed,item_dir);
			vdir->setRenameEnabled (0,activedrop);
		}
		else
			(void) new ListItem((QListView*)vfileViewer,myChild->text(0),myChild->text(1),fileNormal);

		myChild = myChild->nextSibling();
	}
	if(newselect->parent())
		nameSpace->setOpen(newselect->parent(),TRUE);
	nameSpace->setSelected(newselect, TRUE);

	vfileViewer->currentdir = newselect;
	vfileViewer->setSorting(2);
	vfileViewer->sort();
}



void vFileBrowser::hideVfv()
{
		if(nameSpace->isVisible())
		{
			hideVbutton->setText(">");
			nameSpace->hide();
		}
		else
		{
			hideVbutton->setText("<");
			nameSpace->show();
		}
}


void vFileBrowser::disableVSpace()
{
	vfileViewer->setDisabled(TRUE);
	hideVbutton->setDisabled(TRUE);
	nameSpace->setDisabled(TRUE);
	QString msg(tr("Chargement du rpertoire en cours..."));
	emit sendLoadLabel(msg);
	emit enableEnv(TRUE);
}


void vFileBrowser::enableVSpace()
{
	vfileViewer->setDisabled(FALSE);
	hideVbutton->setDisabled(FALSE);
	nameSpace->setDisabled(FALSE);
	nameSpace->update();
	QString msg(tr("Choisissez les rpertoires  partager"));
	emit sendDefaultLabel(msg);
	emit enableEnv(FALSE);
}

vListItem * vFileBrowser::getCurrentDir()
{
	return ((vListItem*)nameSpace->newCurrent);
}


vListItem * vFileBrowser::getParent()
{
	return ((vListItem*)nameSpace->selectedItem()->parent());
}


vListItem * vFileBrowser::getElement(QString& name)
{
	QListViewItem * myChild = nameSpace->selectedItem()->firstChild();
  while( myChild )
	{
		if(myChild->text(0) == name)
			break;
    myChild = myChild->nextSibling();
  }
	return ((vListItem*)myChild);
}
