#include <qapplication.h>
#include <qfile.h>
#include <qxml.h>
#include <qlayout.h>
#include <qmainwindow.h>
#include <qlabel.h>
#include <qobject.h>
#include <qdir.h>
#include <qwidget.h>
#include <qsize.h>
#include <qevent.h>
#include <qpopupmenu.h>
#include <qdom.h>
#include <qsplitter.h>
#include "sharefilebrowser.h"
#include "structureparser.h"
#include "qsharefileview.h"
#include "dirview.h"
#include "vdirview.h"

shareFileBrowser::shareFileBrowser(QWidget * parent, const char * name, WFlags f):QWidget (parent,name,f),pauseHashing(0)
{
	QGridLayout * container;
	QGridLayout * layout;
	QSplitter * splitter;
	QWidget * right;
	container = new QGridLayout ( this,3,2 );

	if(!folderLocked)
	{
		folderLocked = new QPixmap(QPixmap::fromMimeSource("file_locked.png"));
		folderClosed = new QPixmap(QPixmap::fromMimeSource("folder.png"));
		folderOpen = new QPixmap(QPixmap::fromMimeSource("folder_open.png"));
		fileNormal = new QPixmap(QPixmap::fromMimeSource("documents.png"));
	}

	layout = new QGridLayout((QWidget *)0,1,2,5);
	container->addMultiCellLayout(layout,0,0,0,1);
	label = new QLabel(tr("Fichiers partags"), this);
	label->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Minimum);
	layout->addWidget(label,0,0);
	label->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
	label = new QLabel(tr("Fichiers locaux"), this);
	label->setSizePolicy(QSizePolicy::Ignored,QSizePolicy::Minimum);
	label->setAlignment(Qt::AlignHCenter|Qt::AlignVCenter);
	layout->addWidget(label,0,1);

	splitter = new QSplitter(this);
	splitter->setChildrenCollapsible(FALSE);
	splitter->setOpaqueResize(FALSE);
	splitter->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
	container->addMultiCellWidget(splitter,1,1,0,1);
	vbrowser =  new vFileBrowser(splitter,gconf->confDir()+"filelist.xml",TRUE,"vfile_browser");


	right = new QWidget(splitter);
	layout = new QGridLayout(right,1,1,6);
	splitter = new QSplitter(right);
	splitter->setChildrenCollapsible(FALSE);
	splitter->setOpaqueResize(FALSE);
	layout->addWidget(splitter,0,0);
	splitter->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
	mw = new DirectoryView(splitter,"file_browser");
	mw->setMinimumSize(200,450);
	mw->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
	mw->addColumn(tr( "Nom"));

	const QFileInfoList * roots = QDir::drives();
	QPtrListIterator<QFileInfo> i(*roots);
	QFileInfo* fi;
	while ( (fi = *i) )
	{
			++i;
			Directory * root = new Directory( mw, fi->absFilePath(),fi->absFilePath() );
			if ( roots->count() <= 1 )
				root->setOpen( TRUE );
	}

	fileviewer = new dirView(splitter,"fileAndDir");
	fileviewer->setSizePolicy(QSizePolicy::MinimumExpanding,QSizePolicy::MinimumExpanding);
	fileviewer->setMinimumSize(200,450);
	fileviewer->addColumn( tr("Nom") );
	fileviewer->addColumn( tr("Taille" ));

	QBoxLayout * buttons = new QHBoxLayout((QWidget *)NULL,10);
	container->addMultiCellLayout(buttons,2,2,0,1);

	label = new QLabel(tr("Choisissez les rpertoires  partager"), this,"vdirectory_infoLabel");
	buttons->addWidget(label);

	cancelButton = new QPushButton(tr("Annuler"),this);
	buttons->addWidget(cancelButton);
	cancelButton->setMaximumSize(150,25);
	validButton = new QPushButton("Ok",this);
	buttons->addWidget(validButton);
	validButton->setMaximumSize(150,25);

	connect(vbrowser,SIGNAL(sendLoadLabel(const QString &)),label,SLOT(setText(const QString &)));
	connect(vbrowser,SIGNAL(sendDefaultLabel(const QString &)),label,SLOT(setText(const QString &)));
	connect(vbrowser,SIGNAL(enableEnv(bool)),this,SLOT(enableEnv(bool)));
	connect(mw,SIGNAL(selectionChanged ( QListViewItem * )),fileviewer,SLOT(displayFolder(QListViewItem *)));
	connect(fileviewer,SIGNAL(newdir(const QString&)),mw,SLOT(openDir(const QString&)));
	connect(fileviewer,SIGNAL(closedir()),mw,SLOT(closeDir()));
	connect(validButton,SIGNAL(clicked()),this,SLOT(launchXmlGen()));
	connect(cancelButton,SIGNAL(clicked()),parent,SLOT(close()));

	thread = new SFBThread(this,(shareFileBrowserWindow *)parent);
	hashfile = NULL;
}

shareFileBrowser::~shareFileBrowser()
{
	pauseXmlGen();
	waitThread();
}

void shareFileBrowser::enableEnv(bool b)
{
	cancelButton->setDisabled(b);
	validButton->setDisabled(b);
}

void shareFileBrowser::waitThread()
{
	thread->wait();
}

bool shareFileBrowser::isHashingPaused()
{
	return pauseHashing;
}

void shareFileBrowser::genXmlFile(QListView * listview)
{
	QDomDocument flistdb( "Gloster-Shared-Files" );
	QDomDocument flist("Gloster-File-List");
	QDomElement base,baselist,infos,hashes;
	QFile filedb,filelist;
	GHashFile h;
	unsigned int i;
	QString th;

	nbfiles = 0;
	totalsize = 0;

	infos = flistdb.createElement("infos");
	infos.setAttribute("nodeid",QString((graphe->nodes())[0].nodeid));
	flistdb.appendChild(infos);
	base = flistdb.createElement("dir");
	base.setAttribute("name","/");
	infos.appendChild(base);
	hashes = flist.createElement("hashes");
	flist.appendChild(hashes);
	baselist = flist.createElement("dir");
	baselist.setAttribute("name","/");
	hashes.appendChild(baselist);
	vListItem * child = (vListItem *) listview->firstChild()->firstChild();
	while(child)
	{
		if(child->type() == item_dir)
			base.appendChild(addXmlDir(flistdb, flist, baselist, child));
		else
			base.appendChild(addfile(flistdb, flist, baselist, child));
		child = (vListItem *) child->nextSibling();
	}

	FileListMutex.lock();
	filedb.setName(gconf->confDir() +"filedb.xml");
	filelist.setName(gconf->confDir() +"filelist.xml");
	filedb.open(IO_Truncate | IO_ReadWrite | IO_Translate);
	filelist.open(IO_Truncate | IO_ReadWrite | IO_Translate);
	QTextStream dbstream( &filedb );
	QTextStream liststream( &filelist);

	dbstream << flistdb.toString();
	shareList->setFileListSize(filedb.size());
	filedb.close();

	h.readFromFile(gconf->confDir() +"filedb.xml");
	th = h.getFileHash();
	hashes.setAttribute("hash",th);
	shareList->setFileListHash(th);
	
	for(i=0;i<h.getChunkNum();++i)
		hashes.setAttribute("p"+QString::number(i),h.getChunkHash(i));

	liststream << flist.toString();
	shareList->setFileListSize(filelist.size());
	filelist.close();
	FileListMutex.unlock();
	hashEvent * event;
	event = new hashEvent(QString::null);
	QApplication::postEvent(parent(), event);
	statsEvent * statsevent;
	statsevent = new statsEvent(nbfiles,totalsize);
	QApplication::postEvent(parent(), statsevent);
}


QDomElement shareFileBrowser::addXmlDir(QDomDocument & flistdb, QDomDocument & flist, QDomElement & baselist, vListItem * item)
{
	QDomElement dbpath,listpath;
	if(item->type() == item_dir)
	{
		dbpath = flistdb.createElement("dir");
		dbpath.setAttribute("name",item->name());

		listpath = flist.createElement("dir");
		listpath.setAttribute("name",item->name());
		baselist.appendChild(listpath);

		vListItem * child = (vListItem *) item->firstChild();
		while(child)
		{
			if(child->type() == item_dir)
				dbpath.appendChild(addXmlDir(flistdb,flist,listpath, child));
			else
				dbpath.appendChild(addfile(flistdb,flist,listpath,child));
			child = (vListItem *) child->nextSibling();
		}
		return dbpath;
	}
	else
		return addfile(flistdb,flist,baselist,item);
}


/* Description :
	 Ajoute un fichier dans les fichiers xml
*/
QDomElement shareFileBrowser::addfile(QDomDocument & flistdb, QDomDocument& flist, QDomElement & path, vListItem * child)
{
	QDomElement dbfile,listfile;
	unsigned int type;
	QString temp = "";
	QFileInfo f(child->absFilePath());
	listfile = flist.createElement("file");
	listfile.setAttribute("name", f.fileName());
	listfile.setAttribute("path",child->absFilePath());

	if(!pauseHashing && (child->hashList().isEmpty() || child->hashList().first().isEmpty() || child->size()!=f.size() || child->lastModified()!=f.lastModified().toTime_t()) )
	{
		hashEvent * event;
		event = new hashEvent("("+QString::number((int)(((float)nbfiles/(float)vbrowser->nbFiles())*100))+"%) "+tr("Hachage du fichier")+" "+f.fileName()+" "+tr("en cours")+"...");
		QApplication::postEvent(parent(), event);

		type = gconf->defineFileType(f.extension(FALSE));

		listfile.setAttribute("type",type);
		child->setFileType(type);
		listfile.setAttribute("size",f.size());
		child->setSize(f.size());
		listfile.setAttribute("lastmodified",QString::number(f.lastModified().toTime_t()));
		child->setLastModified(f.lastModified().toTime_t());
		hashfile = new GHashFile;
		hashfile->readFromFile(child->absFilePath());
		if(!pauseHashing)
		{
			QStringList hashlist;
			temp = hashfile->getFileHash();
			listfile.setAttribute("hash",temp);
			hashlist << temp;
			for(unsigned int i=0; i<hashfile->getChunkNum();++i)
			{
				listfile.setAttribute("p"+QString::number(i),hashfile->getChunkHash(i));
				hashlist << hashfile->getChunkHash(i);
			}
			child->setHashList(hashlist);
		}
		delete(hashfile);
		hashfile = NULL;
	}
	else
	{
		listfile.setAttribute("type",(type=child->fileType()));
		listfile.setAttribute("hash",(temp=child->hashList().first()));
		listfile.setAttribute("size",QString::number(child->size()));
		listfile.setAttribute("lastmodified",child->lastModified());
		unsigned int i=0;
		QStringList hashlist = child->hashList();
		QStringList::Iterator it = hashlist.begin();
 		for(++it; it != hashlist.end(); ++it,++i)
	        	listfile.setAttribute("p"+QString::number(i),*it);
	}

	path.appendChild(listfile);

	dbfile = flistdb.createElement("file");
	dbfile.setAttribute("name", f.fileName());
	dbfile.setAttribute("hash",temp);
	dbfile.setAttribute("size",f.size());
	dbfile.setAttribute("type",type);

	if(!pauseHashing)
	{
		shareList->filelist[type][temp] = f.absFilePath();
		incNbFiles(f.size());
	}

	return dbfile;
}

/* slot */
void shareFileBrowser::launchXmlGen()
{
	if(pauseHashing)
		pauseHashing = FALSE;
	thread->start(QThread::NormalPriority);
	QEvent * event;
	event = new QEvent((QEvent::Type)(QEvent::User+1));
	QApplication::postEvent(parent(), event);
}

void shareFileBrowser::pauseXmlGen()
{
	if(hashfile)
		hashfile->stopHashing();
	pauseHashing = TRUE;
}

/*****************************************************************************
 *
 * Class SFBThread
 *
 *****************************************************************************/
/* =========== Thread supplementaire du SFBThread ============ */

void SFBThread::run()
{
	mutex->lock();
	shareList->clearQMaps();
	parent->genXmlFile(parent->vbrowser->leftListView());
	mutex->unlock();
	shareList->QMapsInfos();
	if(!parent->pauseHashing)
	{
		QEvent * event;
		event = new QEvent(QEvent::User);
		QApplication::postEvent(win, event);
	}
}


/*****************************************************************************
 *
 * Class shareFileBrowserWindow
 *
 *****************************************************************************/
shareFileBrowserWindow::shareFileBrowserWindow(QWidget * parent, const char * name, WFlags f)
	: QMainWindow(parent,name,f)
{
	sfb = new shareFileBrowser(this,"shareFileBrowser",f);
	setCentralWidget(sfb);
	sfb->setBaseSize(400,430);
	setCaption(tr("Fichiers partags"));
	setIcon(QPixmap::fromMimeSource("fileshare.png"));
}

shareFileBrowserWindow::~shareFileBrowserWindow()
{
}

void shareFileBrowserWindow::validButton()
{
	sfb->launchXmlGen();
}

void shareFileBrowserWindow::pauseXmlGen()
{
	sfb->pauseXmlGen();
}

void shareFileBrowserWindow::closeEvent(QCloseEvent * e)
{
	if(sfb->isHashingPaused())
	{
		hide();
		delete(sfb);
		sfb = new shareFileBrowser(this,"shareFileBrowser",getWFlags());
		setCentralWidget(sfb);
		sfb->show();
		sfb->setBaseSize(400,430);
		validButton();
		e->ignore();
	}
	else
		QWidget::closeEvent(e);
}

bool shareFileBrowserWindow::event(QEvent * e)
{
	if(e->type() == QEvent::User+3)
		emit statsInfos(((statsEvent *)e)->nbfile,((statsEvent *)e)->totalsize);
	else if(e->type() == QEvent::User+2)
		emit updateStatusBar(((hashEvent *)e)->msg);
	else if(e->type() == QEvent::User+1)
		hide();
	else if(e->type() == QEvent::User)
	{
		shareList->initFileListInfos();
		deleteLater();
	}
	else
		return QMainWindow::event(e);
	return TRUE;
}

void shareFileBrowserWindow::showEvent(QShowEvent * e)
{
	sfb->pauseXmlGen();
	sfb->waitThread();
	QMainWindow::showEvent(e);
}
