#ifndef SHAREFILEBROWSER_H
#define SHAREFILEBROWSER_H

#include <qwidget.h>
#include <qmainwindow.h>
#include <qobject.h>
#include <qfile.h>
#include <qxml.h>
#include <qpushbutton.h>
#include <qdict.h>
#include <qlistview.h>
#include <qdom.h>
#include <qthread.h>
#include "../../files/qsharelist.h"
#include "../../graphes.h"
#include "../../gconfig.h"
#include "../../files/ghashfile.h"

class QGrid;
class vDirectoryView;
class StructureParser;
class DirectoryView;
class QShareFileView;
class QListViewItem;
class QLabel;
class vFileBrowser;
class SFBThread;
class vListItem;
class shareFileBrowserWindow;
class dirView;

extern QMutex * mutex;
extern QSharelist * shareList;
extern c_graphes * graphe;
extern GConfig * gconf;

typedef QDict<unsigned int> fcat;
enum FSB_enum {SFB_add=0,SFB_save};

class hashEvent : public QEvent
{
	public :
	hashEvent(QString m)
	:QEvent((QEvent::Type)(QEvent::User+2)),msg(m) {}

	QString msg;
};

class statsEvent : public QEvent
{
	public :
	statsEvent(Q_UINT32 nf,double ts )
	:QEvent((QEvent::Type)(QEvent::User+3)),nbfile(nf),totalsize(ts) {}

	Q_UINT32 nbfile;
	double totalsize;
};

class shareFileBrowser: public QWidget
{
	Q_OBJECT

	public :
		shareFileBrowser(QWidget * parent = 0, const char * name = 0, WFlags f = 0);
		~shareFileBrowser();
		void genXmlFile(QListView *);
		QPushButton * validButton;

	public slots :
		void launchXmlGen();
		void pauseXmlGen();
		void enableEnv(bool);
		void waitThread();
		bool isHashingPaused();
		void incNbFiles(double size) { nbfiles += 1; totalsize += size/(1048576); } //Taille en Mo
		void decNbFiles(double size) { nbfiles -= 1; totalsize -= size/(1048576); } //Taille en Mo

	private :
	/* functions */
		QDomElement addfile(QDomDocument & flistdb, QDomDocument& flist, QDomElement & path, vListItem * child);
		QDomElement addXmlDir(QDomDocument & flistdb, QDomDocument& flist, QDomElement & baselist, vListItem * item);
	/* Variables */
		vFileBrowser * vbrowser;
		dirView * fileviewer;
		DirectoryView * mw;
		QPushButton * cancelButton;
		QLabel * label;
		SFBThread * thread;
		GHashFile * hashfile;

		Q_UINT32 nbfiles;
		double totalsize;

		/* resuming and pausing capabilities for hashing */
		bool pauseHashing;

	friend class SFBThread;
};


class SFBThread : public QThread
{
	public:
		SFBThread(shareFileBrowser * p,shareFileBrowserWindow * w):QThread(),parent(p),win(w) {}
		virtual void run();

	private :
		shareFileBrowser * parent;
		shareFileBrowserWindow * win;
};


class shareFileBrowserWindow : public QMainWindow
{
	Q_OBJECT

	public :
		shareFileBrowserWindow(QWidget * parent = 0, const char * name = 0, WFlags f = WType_TopLevel);
		~shareFileBrowserWindow();
		void validButton();
		void pauseXmlGen();
		void closeEvent(QCloseEvent * e);
		bool event(QEvent * e);
		void showEvent(QShowEvent * e);

	signals:
		void updateStatusBar(const QString &);
		void statsInfos(Q_UINT32,double);

		private :
			shareFileBrowser * sfb;
};

#endif
