#include "dirview.h"
#include "vdirview.h"
#include "qsharefileview.h"

#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qpixmap.h>
#include <qevent.h>
#include <qpoint.h>
#include <qmessagebox.h>
#include <qdragobject.h>
#include <qmime.h>
#include <qstrlist.h>
#include <qstringlist.h>
#include <qapplication.h>
#include <qheader.h>
#include <qstring.h>


/*****************************************************************************
 *
 * Class vDirectoryDrag
 *
 *****************************************************************************/
vDirectoryDrag::vDirectoryDrag(QWidget * parent,const char * name )
    : QStoredDrag("Gloster/vDirectory", parent, name )
{
	QByteArray data(sizeof(int));
	*((int *) (&(data[0])))= 0;

	addSelectedItem(data,NULL);

	setEncodedData( data );
}

bool vDirectoryDrag::canDecode( QMimeSource * e )
{
    return e->provides( "Gloster/vDirectory" );
}

bool vDirectoryDrag::decode(QDropEvent* e, QLVIList& l)
{
	char * result;
	QByteArray temp= e->data( "Gloster/vDirectory" );
	if ( temp.size()>=4 )
	{
		e->accept();
		result = temp.data();
		for(int i=1;i<=(*((int *) (&(result[0]))));++i)
			l << *(QListViewItem **) (result+(i*sizeof(int)));
		return TRUE;
  }
  return FALSE;
}


void vDirectoryDrag::addSelectedItem(QByteArray& data, QListViewItem * item)
{
	QListViewItem * child;
	if(!item)
		child = ((QListView *)parent())->firstChild();
	else
		child = item->firstChild();
	while(child)
	{
		if(child->isSelected())
		{
			data.resize(data.size()+sizeof(QListViewItem *));
			*((char **) (&(data[0])))+= 1;
			*((QListViewItem **) (&(data[data.size()-sizeof(QListViewItem *)])))= child;
		}
		if(child->isOpen())
			addSelectedItem(data,child);
		child = child->nextSibling();
	}
}

/*****************************************************************************
 *
 * Class vDirDrag
 *
 *****************************************************************************/

vDirDrag::vDirDrag(QWidget * parent,const char * name )
    : QStoredDrag("Gloster/vDir", parent, name )
{
	QByteArray data(sizeof(int));
	*((int *) (&(data[0])))= 0;

	addSelectedItem(data,NULL);

	setEncodedData( data );
}

bool vDirDrag::canDecode( QMimeSource * e )
{
    return e->provides( "Gloster/vDir" );
}

bool vDirDrag::decode( QDropEvent* e, QLVIList& l)
{
	char * result;
	QByteArray temp= e->data( "Gloster/vDir" );
	if ( temp.size()>=4 )
	{
		e->accept();
		result = temp.data();
		for(int i=1;i<=(*((int *) (&(result[0]))));++i)
			l << *(QListViewItem **) (result+(i*sizeof(int)));
		return TRUE;
  }
  return FALSE;
}

void vDirDrag::addSelectedItem(QByteArray& data, QListViewItem * item)
{
	QListViewItem * child;
	if(!item)
		child = ((QListView *)parent())->firstChild();
	else
		child = item->firstChild();
	while(child)
	{
		if(child->isSelected())
		{
			data.resize(data.size()+sizeof(QListViewItem *));
			*((char **) (&(data[0]))) += 1;
			*((QListViewItem **) (&(data[data.size()-sizeof(QListViewItem *)])))= ((vDirView *)parent())->findSelectedItem(child->text(0));
		}
		if(child->isOpen())
			addSelectedItem(data,child);
		child = child->nextSibling();
	}
}

/*****************************************************************************
 *
 * Class vListItem
 *
 *****************************************************************************/

void vListItem::setOpen( bool o )
{
    if ( o )
        setPixmap(folderOpen);
    else
        setPixmap(folderClosed);

    QListViewItem::setOpen( o );
}


void vListItem::setup()
{
    setExpandable( TRUE );
    QListViewItem::setup();
}


/*****************************************************************************
 *
 * Class vDirectoryView
 *
 *****************************************************************************/

vDirectoryView::vDirectoryView( QWidget *parent, const char *name,bool activeDrop,SelectionMode m)
    : QListView( parent, name ),
	nodeid(QString::null),dropItem( 0 ), 
	mousePressed( FALSE ),activedrop(activeDrop), 
	selectionmode(m), nbfiles(0),
	totalsize(0)
{
	autoopen_timer = new QTimer( this );
	thread = new VDVThread(this);

	addColumn(tr("Nom"));
	setTreeStepSize( 10 );

	if(activeDrop)
	{
		setAcceptDrops( TRUE );
		viewport()->setAcceptDrops( TRUE );
	}
	setDefaultRenameAction(Accept);

	createCMenu();

	connect( autoopen_timer, SIGNAL( timeout() ),this, SLOT( openFolder() ) );
	connect(this,SIGNAL(itemRenamed(QListViewItem *,int,const QString&)),this,SLOT(isNameValid(QListViewItem *,int,const QString&)));
	connect(this,SIGNAL(selectionChanged(QListViewItem *)),this,SLOT(validateSelectionChanged(QListViewItem *)));

	setSelectionMode(QListView::Single);
	setAllColumnsShowFocus( TRUE );
}



void vDirectoryView::openFolder()
{
   autoopen_timer->stop();
   if ( dropItem && !dropItem->isOpen() )
	{
   		dropItem->setOpen( TRUE );
    	dropItem->repaint();
   }
}


bool vDirectoryView::canDecode(QDragMoveEvent* e)
{
	return(vDirectoryDrag::canDecode(e) || QUriDrag::canDecode(e) || vDirDrag::canDecode(e));
}


void vDirectoryView::contentsDragEnterEvent( QDragEnterEvent *e )
{
	mousePressed = 2;
	if (!canDecode(e))
	{
		e->ignore();
		return;
	}
}


void vDirectoryView::contentsDragMoveEvent( QDragMoveEvent *e )
{
	if (!canDecode(e))
	{
		e->ignore();
		return;
    }
    QPoint vp = contentsToViewport( ( (QDragMoveEvent*)e )->pos() );
    QListViewItem *i = itemAt( vp );
	if(i)
	{
		setSelected(dropItem,FALSE);
		setSelected( i, TRUE );
		dropItem = i;
	}

	mousePressed = 2;
	autoopen_timer->stop();
	autoopen_timer->start( autoopenTime );
	e->accept();
}

void vDirectoryView::contentsDragLeaveEvent( QDragLeaveEvent * )
{
    autoopen_timer->stop();
	setSelected(dropItem,FALSE);
    dropItem = 0;
	mousePressed = 0;
}

void vDirectoryView::acceptQUriDrag(QListViewItem* item, QDropEvent* e)
{
	QListViewItem * newdir;
	emit disableVSpace();
	QStrList lst;
	QUriDrag::decode( e, lst );
	e->acceptAction();
	for(unsigned int i = 0; i<lst.count();++i)
	{
		QString path = QDir::convertSeparators(QUriDrag::uriToLocalFile(lst.at(i)));
		QString filename = path.section(QDir::separator (),-1);
		QFileInfo f(path);
		if(f.isDir())
		{
			if(!(newdir = nameAlreadyExist(item,filename)))
				newdir = (QListViewItem *) new vListItem((vListItem *)item,filename,QString::null,folderClosed,item_dir);
				newdir->setRenameEnabled(0,TRUE);
			thread->setInfos(newdir,path,dir);
		}
		else
			thread->setInfos(item,path,file);
	}
	thread->start();
}

void vDirectoryView::acceptvDirectoryDrag(QListViewItem* item, QDropEvent* e)
{

	emit disableVSpace();
	QListViewItem *newchild;
	QLVIList dirList;
	if(vDirectoryDrag::canDecode(e))
		vDirectoryDrag::decode(e,dirList);
	else
		vDirDrag::decode(e,dirList);

	QLVIList::iterator it;
  	for(it = dirList.begin(); it != dirList.end(); ++it)
	{
		newchild = *it;
		if(newchild != item && !isParent(item,newchild))
		{
			item->insertItem(newchild);
			e->acceptAction();
			emit transmitSelectionChanged(newCurrent);
		}
	}
	emit enableVSpace();
}

QListViewItem * vDirectoryView::testDropEventDest(QDropEvent* e)
{
	QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
	if(!item)
		item = firstChild();
	return item;
}

void vDirectoryView::contentsDropEvent( QDropEvent *e )
{
  autoopen_timer->stop();
	QListViewItem *item = testDropEventDest(e);
	if(QUriDrag::canDecode(e))
		acceptQUriDrag(item,e);
	else
		acceptvDirectoryDrag(item,e);

	mousePressed = FALSE;
}


void vDirectoryView::contentsMousePressEvent( QMouseEvent* e )
{
    QListView::contentsMousePressEvent(e);
    QPoint p( contentsToViewport( e->pos() ) );
    QListViewItem *i = itemAt( p );
    if ( i )
			if(e->button() == LeftButton)
			{
				emit validateSelectionChanged(i);
				mousePressed = TRUE;
				setFocus();
			}
}


void vDirectoryView::contentsMouseMoveEvent( QMouseEvent* e )
{
    if ( mousePressed && (e->pos().manhattanLength() > QApplication::startDragDistance()) )
		{
        mousePressed = FALSE;
        QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
        if (item && ((ListItem *)item)->type() != item_special)
				{
					vDirectoryDrag *sd = new vDirectoryDrag(this);
    			sd->setPixmap(QPixmap::fromMimeSource("folder_new.png"),QPoint(8,8));
    			sd->dragMove();
				}
   	}
}


void vDirectoryView::contentsMouseReleaseEvent( QMouseEvent * )
{
    mousePressed = FALSE;
}

/* SLOT */
void vDirectoryView::validateSelectionChanged(QListViewItem * item)
{
	if(mousePressed < 2 && (((ListItem *)item)->type() == item_special || ((ListItem *)item)->type() == item_dir))
	{
		newCurrent = item;
		setSelectionMode(QListView::Single);
		emit transmitSelectionChanged(item);
		setSelectionMode(selectionmode);
	}
}

/* SLOT */
void vDirectoryView::doCancelDragDrop()
{
	mousePressed = 0;
}

void vDirectoryView::addDir(QListViewItem * item, const QString & dirpath)
{
	QListViewItem * element;

	QDir dir(dirpath,QString::null,QDir::Name | QDir::IgnoreCase,QDir::Files | QDir::NoSymLinks | QDir::Readable | QDir::IgnoreCase);
	for(uint i = 0;i<dir.count();i++)
	{
		if(dir[i] == "." || dir[i] == "..")
			continue;
		addFile(item,dirpath+QDir::separator()+dir[i]);
	}

	dir.setFilter(QDir::Dirs | QDir::NoSymLinks | QDir::Readable | QDir::IgnoreCase);
	for(uint i = 0;i<dir.count();i++)
	{
		if(dir[i] != "." && dir[i] != "..")
		{
			if(!(element = nameAlreadyExist(item,dir[i])))
				element = (QListViewItem *) new vListItem((vListItem *)item,dir[i],QString::null,folderClosed,item_dir);
			addDir(element, dirpath + QDir::separator() + dir[i]);
		}
	}
}

void vDirectoryView::addFile(QListViewItem * item, const QString & filepath)
{
	vListItem * newfile;
	QFileInfo f(filepath);

	if(f.isReadable() && f.size()>0)
	{
		newfile = new vListItem(item,f.fileName(),QString::number(f.size()),fileNormal,
								item_file,f.absFilePath(),QStringList(),f.lastModified().toTime_t());
		newfile->setVisible(FALSE);
		incNbFiles(f.size());
	}
}

bool vDirectoryView::event ( QEvent * e )
{
	if(e->type() == (QEvent::User))
	{
		emit enableVSpace();
		if(newCurrent)
			validateSelectionChanged(newCurrent);
	}
	else
		return QListView::event(e);
	return TRUE;
}

bool vDirectoryView::isParent(QListViewItem* item,QListViewItem* newchild)
{
	QListViewItem * myChild;

	myChild = item->parent();
	while(myChild && newchild!= myChild)
			myChild = myChild->parent();
	return(myChild==newchild);
}


/* debut des fonctions specifiques au menu contextuel du vDirectoryView */

void vDirectoryView::createCMenu()
{
	cMenu = new QPopupMenu(this);
	cMenu->insertItem( tr("Nouveau Dossier"),  this, SLOT(newItem()), CTRL+Key_N, 0 );
	cMenu->setItemVisible(0,TRUE);
	cMenu->insertItem( tr("Renommer"),  this, SLOT(renameItem()), CTRL+Key_R, 1  );
	cMenu->insertItem( tr("Supprimer"),  this, SLOT(deleteItem()), Key_Delete, 2 );
	connect(this,SIGNAL(contextMenuRequested(QListViewItem *,const QPoint &,int)),this,SLOT(Menucontext(QListViewItem*,const QPoint&,int)));
}

void vDirectoryView::Menucontext(QListViewItem* item,const QPoint& p,int)
{
	if(activedrop)
	{
		setSelectionMode(QListView::Single);
		if(item)
		{
			if(((ListItem *)item)->type()==item_special)
			{
				if(item->text(0)=="..")
					return;
				cMenu->setItemVisible(1,FALSE);
				cMenu->setItemVisible(2,FALSE);
			}
			else
			{
				cMenu->setItemVisible(1,((ListItem *)item)->type()!=item_file);
				cMenu->setItemVisible(2,TRUE);
			}
		}
		else
		{
			cMenu->setItemVisible(1,FALSE);
			cMenu->setItemVisible(2,FALSE);
		}
		cMenu->exec(p);
		setSelectionMode(selectionmode);
	}
}

void vDirectoryView::newItem()
{
	QListViewItem * temp;
	QListViewItem *newElt;
	QString name;
	int i=0;
	setSelectionMode(QListView::Single);
	update();
	temp =selectedItem();
	if(!temp)
		temp = firstChild();
	if(hasFocus())
	{
		name = tr("Nouveau Dossier");
		if(nameAlreadyExist(temp,name))
		{
			do
			{
				i++;
				name = tr("Nouveau Dossier")+"("+QString::number(i)+")";
			}
			while(nameAlreadyExist(temp,name));
		}
		newElt = (QListViewItem *) new vListItem( (vListItem *) temp, name,QString::null,folderClosed,item_dir);
		newElt->setRenameEnabled (0,TRUE);
		newElt->startRename(0);
	}
	setSelectionMode(selectionmode);
}

void vDirectoryView::deleteItem()
{
	QListViewItem * temp;
	QListViewItem * parent;
	setSelectionMode(QListView::Single);
	if((temp =selectedItem()) && hasFocus() && ((ListItem *)temp)->type() != item_special)
	{
		if(((ListItem *)temp)->size() != item_file)
			decNbFiles(((ListItem *)temp)->size());
		parent = temp->parent();
		delete(temp);
		setSelected(parent,TRUE);
	}
	setSelectionMode(selectionmode);
	update();
}

void vDirectoryView::renameItem()
{
	QListViewItem * temp;
	setSelectionMode(QListView::Single);
	if((temp =selectedItem()) && hasFocus())
	{
		oldName = temp->text(0);
		temp->startRename(0);
	}
	setSelectionMode(selectionmode);
}

bool vDirectoryView::nameExist(QListViewItem * parent,const QString& name)
{
	QListViewItem * child;
	int i=0;
	child	= parent->firstChild();
	while(child && i<2)
	{
		if(child->text(0)==name)
			i++;
		child = child->nextSibling();
	}
	return(i>1);
}

QListViewItem * vDirectoryView::nameAlreadyExist(QListViewItem * item, const QString& name)
{
	QListViewItem * child;
	child	= item->firstChild();
	while(child && child->text(0)!=name)
		child = child->nextSibling();

	return(child);
}


void vDirectoryView::isNameValid(QListViewItem * item,int col,const QString& name)
{
	QRegExp rx("^\\s+$");
	name.simplifyWhiteSpace();
	if(name == QString::null || name == ".." || name == "/" || name == "." || rx.search(name) == 0 || nameExist(item->parent(),name))
	{
		item->setText(col,oldName);
		item->startRename(col);
	}
	else
		item->setText(col,name);
	sort();
}

/* fin des fonctions specifiques au menu contextuel du vDirectoryView*/



/*****************************************************************************
 *
 * Class vDirView
 *
 *****************************************************************************/
void vDirView::contentsMouseDoubleClickEvent ( QMouseEvent * e )
{
	QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
	if(item && (((ListItem *)item)->type() == item_dir || ((ListItem *)item)->type() == item_special))
		forceUpdatevDirView(item->text(0));
	else if(item && ((ListItem *)item)->type() == item_file && !activedrop)
		launchDownload();
}


void vDirView::contentsMouseMoveEvent( QMouseEvent* e )
{
    if ( mousePressed && (e->pos().manhattanLength() > QApplication::startDragDistance()) )
		{
        mousePressed = FALSE;
        QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
        if (item && ((ListItem *)item)->type() != item_special)
				{
					vDirDrag *sd = new vDirDrag(this);
					sd->setPixmap(QPixmap::fromMimeSource("folder_new.png"),QPoint(8,8));
					sd->dragMove();
				}
   	}
}

void vDirView::acceptvDirectoryDrag(QListViewItem* item, QDropEvent* e)
{
	emit disableVSpace();
	QListViewItem *newchild;
	QLVIList dirList;
	vDirDrag::decode(e,dirList);

	QLVIList::iterator it;
  	for(it = dirList.begin(); it != dirList.end(); ++it)
	{
		newchild = *it;
		if(newchild != item && ((ListItem *)item)->type()!=item_file)
		{
			if(item->listView() == this)
				item = findSelectedItem(item->text(0));
			if(item!=newchild)
			{
				item->insertItem(newchild);
				e->acceptAction();
			}
		}
	}
	emit forceUpdatevDirView(currentdir);
	emit enableVSpace();
}

QListViewItem * vDirView::testDropEventDest(QDropEvent* e)
{
	QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
	if(item && ((ListItem *)item)->type() == item_dir)
		item = findSelectedItem(item->text(0));
	else if(item && ((ListItem *)item)->type() == item_special)
		item = currentdir->parent();
	if(!item || ((ListItem *)item)->type() == item_file)
		item = currentdir;
	return item;
}

bool vDirView::canDecode(QDragMoveEvent* e)
{
	return(vDirDrag::canDecode(e) || QUriDrag::canDecode(e));
}

bool vDirView::event ( QEvent * e )
{
	if(e->type() == (QEvent::User))
	{
		emit enableVSpace();
		newCurrent = currentdir;
		emit forceUpdatevDirView(newCurrent);
	}
	else
		return QListView::event(e);
	return TRUE;
}


QListViewItem * vDirView::findSelectedItem(QString name)
{
	QListViewItem * myChild;
	myChild = currentdir->firstChild();
	while(myChild && myChild->text(0)!= name)
			myChild = myChild->nextSibling();
	if(!myChild)
	{
		if(!currentdir->parent())
			return currentdir;
		else
			return currentdir->parent();
	}
	return(myChild);
}

void vDirView::forceUpdatevDirView(QString name)
{
	QListViewItem * myChild;
	myChild = findSelectedItem(name);
  if(myChild)
		currentdir->listView()->setSelected(myChild,TRUE);
}


bool vDirView::nameExist(QListView * listview, const QString& name)
{
	QListViewItem * child;
	int i=0;
	child	= listview->firstChild();
	while(child && i<2)
	{
		if(child->text(0)==name)
			i++;
		child = child->nextSibling();
	}
	return(i>1);
}

bool vDirView::nameAlreadyExist(QListView * listview, const QString& name)
{
	QListViewItem * child;
	child	= listview->firstChild();
	while(child && child->text(0)!=name)
		child = child->nextSibling();

	return(child!=NULL);
}


void vDirView::isNameValid(QListViewItem * item,int col,const QString& name)
{
	QRegExp rx("^\\s+$");
	if(name == "" || name == ".." || name == "/" || name == "." || rx.search(name) == 0 || nameExist(item->listView(),name))
	{
		item->setText(0,oldName);
		item->startRename(col);
	}
	else
	{
		QListViewItem * elt = findSelectedItem(oldName);
		elt->setText(0,name);
	}
	sort();
}

void vDirView::renameItem()
{
	QListViewItem * temp;
	if((temp =selectedItem()) && hasFocus())
	{
		oldName = temp->text(0);
		temp->startRename(0);
	}
}

void vDirView::newItem()
{
	QListViewItem *newElt;
	QString name;
	int i=0;
	if(hasFocus())
	{
		name = tr("Nouveau Dossier");
		if(nameAlreadyExist(this,name))
		{
			do
			{
				i++;
				name =tr("Nouveau Dossier")+"("+QString::number(i)+")";
			}
			while(nameAlreadyExist(this,name));
		}
		(void) new vListItem( (vListItem *) currentdir, name,QString::null,folderClosed,item_dir );
		newElt = new ListItem((QListView*)this,name,QString::null,folderClosed,item_dir);
		newElt->setRenameEnabled (0,TRUE);
		oldName = name;
		newElt->startRename(0);
	}
}

void vDirView::deleteItem()
{
	QListViewItem * tmp;
	QListViewItem * parent;
	if(hasFocus())
	{
		selectedItemList result = selectedItems();
		selectedItemList::iterator it;
		for ( it = result.begin(); it != result.end(); ++it )
		{
			if(((ListItem *)(*it))->type() != item_special)
			{
				tmp = findSelectedItem((*it)->text(0));
				if(((ListItem *)(*it))->type() != item_file)
					emit decNbFiles(((ListItem *)(*it))->size());
				parent = tmp->parent();
				delete(tmp);
				delete(*it);
			}
		}
	}
}

void vDirView::contentsMousePressEvent( QMouseEvent* e )
{
    QListView::contentsMousePressEvent(e);
    QPoint p( contentsToViewport( e->pos() ) );
    QListViewItem *i = itemAt( p );
    if ( i )
			if(e->button() == LeftButton)
			{
				mousePressed = TRUE;
				setFocus();
			}
}


void vDirView::Menucontext(QListViewItem* item,const QPoint& p,int i)
{
	selectedItemList result = selectedItems();
	if(activedrop)
	{
		cMenu->setItemVisible(3,FALSE);
		if(result.count()>1)
		{
				cMenu->setItemVisible(0,FALSE);
				cMenu->setItemVisible(1,FALSE);
				cMenu->setItemVisible(2,TRUE);
				cMenu->exec(p);
				return;
		}
		vDirectoryView::Menucontext(item,p,i);
	}
	else
	{
		if(item && ((ListItem *)item)->type()!=item_special && ((ListItem *)item)->type()!=item_dir)
		{
			cMenu->setItemVisible(0,FALSE);
			cMenu->setItemVisible(1,FALSE);
			cMenu->setItemVisible(2,FALSE);
			cMenu->setItemVisible(3,TRUE);
			cMenu->exec(p);
		}
	}
}

void vDirView::launchDownload()
{
	vListItem * temp;

	if(hasFocus())
	{
		selectedItemList result = selectedItems();
		selectedItemList::iterator it;
		for ( it = result.begin(); it != result.end(); ++it )
		{
			temp = (vListItem *)findSelectedItem((*it)->text(0));
			net->downloader->download(temp->text(0),temp->size(),temp->hashList().first());
		}
	}
}

void vDirView::createCMenu()
{
	cMenu->insertItem(QPixmap::fromMimeSource("download.png"), tr("Tlcharger"),  this, SLOT(launchDownload()), CTRL+Key_D, 3 );
}

selectedItemList vDirView::selectedItems()
{
	QListViewItem * child;
	selectedItemList result;

	child	= firstChild();
	while(child)
	{
		if(child->isSelected())
			result << child;
		child = child->nextSibling();
	}
	return result;
}

/*****************************************************************************
 *
 * Class VDVThread
 *
 *****************************************************************************/

void VDVThread::run()
{
	thread_queue temp;
	QEvent * event;
	while(1)
	{
		lmutex.lock();
		if(	queue.isEmpty())
		{
			lmutex.unlock();
			break;
		}
		temp=queue.front();
		lmutex.unlock();
		temp.item->listView()->setUpdatesEnabled(FALSE);
		if(temp.mode == file)
			parent->addFile(temp.item,temp.filename);
		else
			parent->addDir(temp.item,temp.filename);
		lmutex.lock();
		queue.pop_front();
		lmutex.unlock();
		temp.item->listView()->setUpdatesEnabled(TRUE);
	}

	parent->newCurrent = temp.item;
	event = new QEvent(QEvent::User);
	QApplication::postEvent(parent, event);
}

void VDVThread::setInfos(QListViewItem * i, const QString& name, VDV_mode m)
{
	thread_queue elt;

	elt.item = i;
	elt.filename = QDir::cleanDirPath(name);
	elt.mode = m;
	lmutex.lock();
	queue << elt;
	lmutex.unlock();
}
