#ifndef VDIRVIEW_H
#define VDIRVIEW_H

#include <qdom.h>
#include <qlistview.h>
#include <qstring.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qtimer.h>
#include <qevent.h>
#include <qdragobject.h>
#include <qpopupmenu.h>
#include <qthread.h>
#include <qregexp.h>
#include "dirview.h"
#include "../../network/gnetwork.h"

#define _INVALID_FILE_TYPE 10

class QWidget;
class QDragEnterEvent;
class QDragMoveEvent;
class QDragLeaveEvent;
class QDropEvent;
class QStoreDrag;
class vDirectoryView;

extern const int autoopenTime;
extern GNetwork * net;

typedef QValueList<QListViewItem *> selectedItemList;
enum VDV_mode {file=0, dir};
struct thread_queue
{
	QListViewItem * item;
	QString filename;
	VDV_mode mode;
};

typedef QValueList<QListViewItem*> QLVIList;

class VDVThread : public QThread
{
	public:
		VDVThread(vDirectoryView * p):QThread(),parent(p) {}
		virtual void run();
		void setInfos(QListViewItem * i, const QString& name, VDV_mode m);

		VDV_mode mode;
		vDirectoryView * parent;
		QValueList<thread_queue> queue;
		QMutex lmutex;
};


class vDirectoryDrag : public QStoredDrag
{
	public :
		vDirectoryDrag(QWidget * parent = 0, const char * name = 0);
		~vDirectoryDrag() {};

		static bool canDecode(QMimeSource * e);
		static bool decode(QDropEvent * e, QLVIList& l);
		void addSelectedItem(QByteArray& data, QListViewItem * item);
};

class vDirDrag : public QStoredDrag
{
	public :
		vDirDrag(QWidget * parent = 0, const char * name = 0);
		~vDirDrag() {};

		static bool canDecode(QMimeSource * e);
		static bool decode(QDropEvent * e, QLVIList& l);
		void addSelectedItem(QByteArray& data, QListViewItem * item);
};

class vListItem : public ListItem
{
	public:
		vListItem( QListView * parent, const QString& filename,const QString& filesize, QPixmap * pix, item_type t,
    	const QString& abspath = QString::null, const QStringList& hashlist = QStringList(), unsigned int lastmodified = 0, unsigned int filetype = _INVALID_FILE_TYPE)
				: ListItem(parent,filename,filesize,pix,t),AbsPath(abspath),HashList(hashlist),LastModified(lastmodified),FileType(filetype) {}
		vListItem( QListViewItem * parent, const QString& filename,const QString& filesize, QPixmap * pix, item_type t,
    	const QString& abspath = QString::null, const QStringList& hashlist = QStringList(), unsigned int lastmodified = 0, unsigned int filetype = _INVALID_FILE_TYPE)
				: ListItem(parent,filename,filesize,pix,t),AbsPath(abspath),HashList(hashlist),LastModified(lastmodified),FileType(filetype) {}

    	void setOpen( bool );
    	void setup();

		inline QString absFilePath() { return AbsPath; }
		inline QStringList hashList() { return HashList; }
		inline unsigned int lastModified() { return LastModified; }
		inline unsigned int fileType() { return FileType; }

		inline void setAbsFilePath(const QString& s) { AbsPath = s; }
		inline void setHashList(const QStringList& l) { HashList = l; }
		inline void setLastModified(unsigned int i) { LastModified = i; }
		inline void setFileType(unsigned int i) { FileType = i; }

	private :
		QString AbsPath;
		QStringList HashList; //Le premier element correspond au hash du fichier complet et les autres aux hashs des parts
		unsigned int LastModified;
		unsigned int FileType;
};


class vDirectoryView : public QListView
{
    Q_OBJECT

public:
    	vDirectoryView( QWidget *parent = 0, const char *name = 0, bool activeDrop = TRUE,SelectionMode m = QListView::Single);
		QString nodeid;
	inline Q_UINT32 nbFiles() { return nbfiles; }
	inline double totalSize() { return totalsize; }	

public slots :
	void incNbFiles(double size) { nbfiles += 1; totalsize += size/(1024*1024); }
	void decNbFiles(double size) { nbfiles -= 1; totalsize -= size/(1024*1024); }

signals:
	void transmitSelectionChanged(QListViewItem *);
	void disableVSpace();
	void enableVSpace();

private :
	bool nameExist(QListViewItem * parent, const QString& name);
	QListViewItem * nameAlreadyExist(QListViewItem *, const QString&);

protected slots:
  	void openFolder();
	void doCancelDragDrop();
	virtual void validateSelectionChanged(QListViewItem *);
	bool isParent(QListViewItem* item,QListViewItem* newchild);
	virtual void isNameValid(QListViewItem *,int,const QString&);
	virtual void newItem();
	virtual void deleteItem();
	virtual void renameItem();
	virtual void Menucontext(QListViewItem*,const QPoint& p,int);

protected:
	/* functions */
	bool event ( QEvent * e );
	virtual void contentsDragEnterEvent( QDragEnterEvent *e );
	virtual void contentsDragMoveEvent( QDragMoveEvent *e );
	virtual void contentsDragLeaveEvent( QDragLeaveEvent *e );
	virtual void contentsDropEvent( QDropEvent *e );
	virtual void contentsMouseMoveEvent( QMouseEvent *e );
	virtual void contentsMousePressEvent( QMouseEvent *e );
	virtual void contentsMouseReleaseEvent( QMouseEvent *e );
	virtual void addFile(QListViewItem * item, const QString & filepath);
	virtual void addDir(QListViewItem * item, const QString & dirpath);
	virtual void createCMenu();
	virtual void acceptQUriDrag(QListViewItem* item, QDropEvent* e);
	virtual void acceptvDirectoryDrag(QListViewItem* item, QDropEvent* e);
	virtual QListViewItem * testDropEventDest(QDropEvent* e);
	virtual bool canDecode(QDragMoveEvent* e);

/* Variables */
	QListViewItem *dropItem;
	QListViewItem *newCurrent;
	QTimer* autoopen_timer;
	QPoint presspos;
	int mousePressed;
	QPopupMenu * cMenu;
	VDVThread * thread;
	QString oldName;
	bool activedrop;
	SelectionMode selectionmode;
	Q_UINT32 nbfiles;
	double totalsize;

	friend class VDVThread;
	friend class StructureParser;
	friend class vFileBrowser;
};

class vDirView:public vDirectoryView
{
		Q_OBJECT

	public :
		vDirView( QWidget *parent = 0, const char *name = 0, bool activeDrop = TRUE )
			: vDirectoryView(parent,name,activeDrop,QListView::Extended ),currentdir(0),activedrop(activeDrop)
				{ addColumn(tr("Taille"));setSelectionMode(QListView::Extended);createCMenu();}

		QListViewItem * findSelectedItem(QString name);
		QListViewItem* currentdir;

	signals :
		void forceUpdatevDirView(QListViewItem*);
		void startDL();
		void decNbFiles(double);

	protected slots :
		virtual void validateSelectionChanged(QListViewItem *) {}
		void isNameValid(QListViewItem * item,int col,const QString& name);
		virtual void renameItem();
		virtual void newItem();
		virtual void deleteItem();
	  	void launchDownload();

	protected :
		bool event ( QEvent * e );
		virtual void acceptvDirectoryDrag(QListViewItem* item, QDropEvent* e);
		virtual void contentsMouseDoubleClickEvent ( QMouseEvent * e );
		virtual void contentsMouseMoveEvent( QMouseEvent* e );
		virtual QListViewItem * testDropEventDest(QDropEvent* e);
		virtual bool canDecode(QDragMoveEvent* e);
		void contentsMousePressEvent( QMouseEvent* e );
		void Menucontext(QListViewItem*,const QPoint& p,int);
		virtual void createCMenu();
		selectedItemList selectedItems();

	private :
		void forceUpdatevDirView(QString name);
		bool nameExist(QListView * listview, const QString& name);
		bool nameAlreadyExist(QListView * listview, const QString& name);
		bool activedrop;
};

#endif
