#include <qgl.h>
#include <qvaluelist.h>
#include <math.h>
#include "qgraphe.h"
#define PI 3.141592653921
#define MINDIST 0

GLuint rot = 0;
GLuint rot2 = 0;

/* Description :
    Constructeur de l'objet QGraphe. Demarre
*/
QGraphe::QGraphe(class c_graphes * graphe, QWidget *parent, const char *name ) : QGLWidget( parent, name )
{
	unsigned short int i;
	qDebug("construction QGraphe");
  	dlist = 0;
	pgraphe = graphe;
	node = (nodecoords *) malloc(graphe->nodes().size()*sizeof(nodecoords));
	for(i=0;i<graphe->nodes().size();i++)
		node[i].x = node[i].y = node[i].z = node[i].isset = 0;
	node[0].isset = 1;
    startTimer(TIMER);
	connect(graphe, SIGNAL(graphe_modified()), this, SLOT(graphe_modified()));
}


QGraphe::~QGraphe()
{
	qDebug("destruction QGraphe");
	free(node);
}

/* Description :
    Fonction d'initialisation d'opengl. Appelle a la creation du contexte et a chaque redimenssionement de la fenetre
    C'est dans cette fonction qu'on peut creer des listes d'objets.
*/
void QGraphe::initializeGL()
{
//    static GLfloat pos[4] = {5.0, 5.0, 10.0, 1.0 };

    qglClearColor( black );
    genlist();
	calculcoord();
    glPolygonMode(GL_FRONT_AND_BACK,GL_FILL)    ;
//    glLightfv( GL_LIGHT0, GL_POSITION, pos );
//    glEnable( GL_CULL_FACE );
//    glEnable( GL_LIGHTING );
//    glEnable( GL_LIGHT0 );
//    glEnable( GL_DEPTH_TEST );

//    glEnable( GL_NORMALIZE );
}


/* Description :
    Fonction appelle dans le cas du redimenssionement de la fenetre.
    Permet de conserver les proportions
*/
void QGraphe::resizeGL( int width, int height )
{
    glViewport( 0, 0, width, height );
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective( 45, (float) width / (float) height, 0.01, 800.0 );
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}


/* Description :
    Fonction d'affichage de la scene opengl
*/
void QGraphe::paintGL()
{
	unsigned short int i;

    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glLoadIdentity();

	glPushMatrix();
		glTranslatef(0,0,-70);
		glRotatef(rot2,0,1,0);
		for(i=0;i<pgraphe->nodes().size();i++)
			if(node[i].isset)
				shownode(i);
		glCallList(dlist+2);
	glPopMatrix();
	rot+=3;
	if(rot%2 ==0) rot2++;
}


/* Description :
    instructions executees a chaque passage du timer.
    updateGL() : rafraichit le contexte opengl en appellant la fonction initializeGL si elle n'a pas ete appelle et
    appelle ensuite la fonction paintGL()
*/
void QGraphe::timerEvent(QTimerEvent*)
{
    updateGL();
}


/* Description :
    Fabrique la liste d'affichage qui contient le graphe
*/
void QGraphe::genlist()
{
    GLfloat rayon=1.5;
    GLfloat x,y,z;
    GLint i,j;
    GLint n=8;
    static GLfloat ared[4] = {0.8f, 0.1f, 0.0f, 1.0f };
    if(!dlist) dlist = glGenLists(3);
    glNewList(dlist, GL_COMPILE);
    glMaterialfv( GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ared );

        qglColor( white );
        glBegin(GL_TRIANGLE_STRIP);
                glVertex3f(0.5f,-0.5f,0.5f);
                glVertex3f(0.5f, 0.5f,0.5f);
                glVertex3f(-0.5f,-0.5f,0.5f);
                glVertex3f(-0.5f,0.5f, 0.5f);
        glEnd();
        glBegin(GL_TRIANGLE_STRIP);
                glVertex3f(0.5f,-0.5f,-0.5f);
                glVertex3f(0.5f,0.5f,-0.5f);
                glVertex3f(0.5f, -0.5f,0.5f);
                glVertex3f(0.5f,0.5f,0.5f);
        glEnd();
        glBegin(GL_TRIANGLE_STRIP);
                glVertex3f(-0.5f,-0.5f,-0.5f);
                glVertex3f(-0.5f,0.5f,-0.5f);
                glVertex3f(-0.5f, -0.5f,0.5f);
                glVertex3f(-0.5f,0.5f,0.5f);
        glEnd();
        glBegin(GL_TRIANGLE_STRIP);
                glVertex3f(0.5f,-0.5f,-0.5f);
                glVertex3f(0.5f, 0.5f,-0.5f);
                glVertex3f(-0.5f,-0.5f,-0.5f);
                glVertex3f(-0.5f,0.5f, -0.5f);
        glEnd();
  glEndList();

  /* Creation de la sphere */
  glNewList(dlist+1, GL_COMPILE);
    glPushMatrix();
    glRotatef(90,1.0f,0.0f,0.0f);
    for(j=0;j<=n;j++)
    {
          glBegin(GL_LINE_STRIP);
          for(i=0;i<=n;i++)
          {
              x=sin(j*PI*2/(n-1))*cos(-PI/2+i*PI/(n-1))*rayon;
              y=cos(j*PI*2/(n-1))*cos(-PI/2+i*PI/(n-1))*rayon;
              z=sin(-PI/2+i*PI/(n-1))*rayon;
              glVertex3f(x,y,z);
          }
              glEnd();
    }
    for(i=0;i<=n;i++)
    {
          glBegin(GL_LINE_STRIP);
          for(j=0;j<=n;j++)
          {
              x=sin(j*PI*2/(n-1))*cos(-PI/2+i*PI/(n-1))*rayon;
              y=cos(j*PI*2/(n-1))*cos(-PI/2+i*PI/(n-1))*rayon;
              z=sin(-PI/2+i*PI/(n-1))*rayon;
              glVertex3f(x,y,z);
          }
              glEnd();
    }
    glPopMatrix();

  /* Fin de la creation de la liste */
    glEndList();
}


/* Description :
    Fonction qui intercepte le signal de modification du graph et qui met a jour la liste d'affichage en consequence
*/
void QGraphe::graphe_modified()
{
	unsigned short int i;

	node = (nodecoords *) realloc(node,pgraphe->nodes().size()*sizeof(nodecoords));
	for(i=0;i<pgraphe->nodes().size();i++)
		node[i].x = node[i].y = node[i].z = node[i].isset = 0;
	node[0].isset = 1;
	genlist();
	calculcoord();
}


/* Description :
	Fonction charge d'afficher une node j
*/
void QGraphe::shownode(unsigned short int j)
{
	glPushMatrix();
		glTranslatef(node[j].x,node[j].y,node[j].z);
//		glTranslatef(node[j].x,0.0f,-30.0+node[j].z);
		glRotatef(rot%360,0.0f,1.0f,0.0f);
//		glRotatef(abs(rot%360),1.0f,0.0f,0.0f);
		glCallList(dlist);
	glPopMatrix();

	glPushMatrix();
		glTranslatef(node[j].x,node[j].y,node[j].z);
//		glTranslatef(node[j].x,node[j].y,-30.0+node[j].z);
		glRotatef(rot%360,0.0f,1.0f,0.0f);
		glCallList(dlist+1);
	glPopMatrix();
}


/* Description :
	Calcule les coordonnes des nodes dans l'espace
	Au passage genere la  liste d'affichage des traits reliant les nodes
*/
void QGraphe::calculcoord()
{
	unsigned short int i,j,k;
	GLfloat teta;
	p_arete A;
	QValueList<short unsigned int>	F;

	srand(5);
	F.prepend(0);
	glNewList(dlist+2,GL_COMPILE);
	qglColor(blue);
	while(!F.isEmpty())
	{
		i = F.last();
		F.pop_back();
		A = pgraphe->sommets_list[i].arete;
		if(pgraphe->sommets_list[i].nbaretes>0)
		{
			if(pgraphe->sommets_list[i].nbaretes == 1)
				teta = 120;
			else
				teta = 360/pgraphe->sommets_list[i].nbaretes;
			k=0;
			do
			{
				j = A->sommet;
				if(!node[j].isset)
				{
					node[j].x = node[i].x-11*cos(k*teta);
					node[j].y = node[i].y+5*cos((float)10*rand());
					node[j].z = node[i].y-11*sin(k*teta);
					node[j].isset = 1;
					F.prepend(j);
				}
				glBegin(GL_LINES);
					glVertex3f(node[i].x,node[i].y,node[i].z);
					glVertex3f(node[j].x,node[j].y,node[j].z);
				glEnd();
				k++;
				A = A->suivant;
			}
			while(A);
		}
	}
	glEndList();
}

void QGraphe::mousePressEvent(QMouseEvent *)
{

}
