#include <qapplication.h>
#include <qpixmap.h>
#include <qmutex.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qtranslator.h>
#include <qtextcodec.h>
#include <qdict.h>
#include <qtimer.h>
#include <qsplashscreen.h>
#include <qdir.h>
#include "main.h"
#include "gui/mainwin.h"
#include "gui/firstconf.h"
#include "network/gnetwork.h"
#include "gui/cntlst.h"
#include "gtypes.h"
#include "webserver/gwebserver.h"
#include "crypt/crypt_rsa.h"
#include "gui/trayicon.h"
#include "glanguage.h"
#include "gconfig.h"


GConfig * gconf;
GNetwork *net=NULL;
QSharelist * shareList=NULL;
GWebServer * httpd=NULL;
QMutex * mutex;
QMutex FileListMutex;
TrayIcon *tray=NULL;


int main( int argc, char ** argv )
{
	QApplication a( argc, argv );
	gconf = new GConfig();

	mutex = new QMutex(TRUE);

	GLanguage::setAppLanguage();

	if (gconf->splashScreen())
	{
		QSplashScreen *splash = new QSplashScreen( QPixmap::fromMimeSource("splash.jpg") );
		splash->show();
		QTimer *timer = new QTimer( splash );
		timer->start(1500, true);
		QObject::connect(timer,SIGNAL(timeout()),splash,SLOT(close()));
	}

	if  (!gconf->isConfigured())
	{
		firstconf * win= new firstconf(0,"firstconf",Qt::WType_Dialog);
		win->show();
	}
	else
	{
		QMessageBox* w=NULL;
		if (!check_key_pair(gconf->myPubKey(),gconf->myPrivKey()))
		{
			QString pub,priv;
			qWarning(a.tr("Attention : La cl priv et la cl public ne correspondent pas. Regnration en cours..."));
			generate_key_pair(pub,priv);
			gconf->setMyPrivKey(priv);
			gconf->setMyPubKey(pub);
			w= new QMessageBox(0,"w");
			w->setText(a.tr("Attention : La cl prive et la cl public ne correspondent pas. Elles ont t regnr automatiquement."));
			w->setIcon(QMessageBox::Warning);
		}
		launchMainWin();
		if(w)
			w->show();
	}

	a.connect( &a, SIGNAL( lastWindowClosed() ), &a, SLOT( quit() ) );

	int res = a.exec();

	if(gconf->isConfigured())
		gconf->save();
	delete httpd;
	delete tray;

	return res;
}

void launchMainWin()
{
	MainWin *mw = new MainWin(0,"MainWindow");
	QPopupMenu * menu;

	qApp->setMainWidget( mw );
	menu = new QPopupMenu;
	menu->insertItem( QObject::tr("Cacher"),mw, SLOT(hideo()) );
	menu->insertSeparator();
	menu->insertItem( QObject::tr("Montrer"),mw, SLOT(showo()));
	menu->insertSeparator();
	menu->insertItem(QObject::tr("Quitter"),qApp, SLOT(quit()));
	tray = new TrayIcon(QPixmap::fromMimeSource("Gloster.png"), "Gloster",menu,0,0);
	QObject::connect(tray,SIGNAL(clicked( const QPoint&,int )),mw,SLOT(hideo()));
	tray->show();

	QObject::connect(qApp,SIGNAL(aboutToQuit()),mw,SLOT(aboutToQuit()));

	net->downloader->loadTempFiles();

	if (gconf->webActivated() && gconf->webListenPort()!=0)
		httpd = new GWebServer(gconf->webPassword(), gconf->webListenPort());
}
