/***************************************************************************
 *            gfilesocket.cpp
 *
 *  Thu May  5 12:20:53 2005
 *  Copyright  2005  CrYpToNyM
 *  Email
 ****************************************************************************/

#include <stdlib.h>
#include <string.h> //pour le memcpy
#include <qdict.h>
#include "gfiledlsocket.h"
#include "gnetwork.h"
#include "../graphes.h"
#include "../crypt/crypt_des.h"
#include "../crypt/crypt_rsa.h"

extern c_graphes *graphe;
extern QString MyPrivKey;
extern GNetwork *net;
extern GConfig * gconf;


GFileDlSocket::GFileDlSocket(Q_UINT16 uid,const QString &nid,const QString &h, Q_UINT64 fsize,QObject * parent, Q_UINT64 rsize, const char * name)
  : QObject ( parent, name )
{

  DES_ks temp;
	nodeid=nid;

	if (!uid)
	{
		uid=graphe->NodeidToId(nodeid);
		if (uid==USHRT_MAX)
		{
			ok=false;
			socket=NULL;
			return;
		}
	}
	ok=true;
	ip=graphe->nodes()[uid].host;
	port=graphe->nodes()[uid].port;

        dist_uid = uid;
	remaining=0;
	written=0;
	filesize=fsize;
	realsize=rsize;
	hash=h;
	nbchunk=filesize/CHUNK_SIZE + (filesize % CHUNK_SIZE?1:0);
	dat.resize(PART_SIZE);

        generate_key_from_string(cle_reseau, &temp);
	socket = new GCryptSocket(temp,GDLSOCK,this);

	connect(socket,SIGNAL(connected()),this,SLOT(onConnect()));
	connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
	connect(socket,SIGNAL(connectionClosed()),this,SLOT(onClose()));
	connect(socket,SIGNAL(error ( int )),this,SLOT(onError(int)));
	//connect(socket,SIGNAL(bytesWritten ( int )),this,SLOT(onWritten(int)));
        connect(this,SIGNAL(NeedKey(Q_UINT16)),net,SLOT(sendKeyRequest(Q_UINT16)));
	socket->connectToHost(ip,port);
}

GFileDlSocket::GFileDlSocket(const QString &h, Q_UINT64 fsize,QObject * parent, Q_UINT64 rsize, const char * name)
  : QObject ( parent, name )
{
	ok=true;
	nodeid="ID_du_socket_entrant_____";
	remaining=0;
	written=0;
	filesize=fsize;
	realsize=rsize;
	hash=h;
	nbchunk=filesize/CHUNK_SIZE + (filesize % CHUNK_SIZE?1:0);
	dat.resize(PART_SIZE);

}

GFileDlSocket::~GFileDlSocket()
{
	if (socket)
	{
		socket->disconnect();
		socket->deleteLater();
	}
}

/*
void GFileDlSocket::onWritten( int nbytes)
{
	if (nbytes>=0)
		written+=nbytes;
}
*/
void GFileDlSocket::setSocket(GCryptSocket *s)
{
	socket=s;
	connect(socket,SIGNAL(connectionClosed()),this,SLOT(onClose()));
	connect(socket,SIGNAL(error ( int )),this,SLOT(onError(int)));
	//connect(socket,SIGNAL(bytesWritten ( int )),this,SLOT(onWritten(int)));
	connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
	sendGet(hash,filesize);
}

void GFileDlSocket::sendGet(const QString &hash, unsigned long long fsize)
{
	char data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+5];

	data[0]=fdl_getfile;
	memcpy(data+1,graphe->nodes()[0].nodeid,NODE_ID_SIZE);
	hex2bin_hash(hash.latin1(),data+1+NODE_ID_SIZE);

#if BYTE_ORDER != LITTLE_ENDIAN
	A VERIFIER
	data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH]=*(((char *)&fsize)+sizeof(fsize)-1);
	data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+1]=*(((char *)&fsize)+sizeof(fsize)-2);
	data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+2]=*(((char *)&fsize)+sizeof(fsize)-3);
	data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+3]=*(((char *)&fsize)+sizeof(fsize)-4);
	data[1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+4]=*(((char *)&fsize)+sizeof(fsize)-5);
#else
	memcpy(data+1+NODE_ID_SIZE+MD5_DIGEST_LENGTH,(char *)&fsize,5);
#endif

	if (realsize)
	{
		filesize=realsize;
		nbchunk=filesize/CHUNK_SIZE + (filesize % CHUNK_SIZE?1:0);
	}

	if (socket->writeCryptedBlock(data,1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+5)!=(1+NODE_ID_SIZE+MD5_DIGEST_LENGTH+5))
	{
		socket->close();
		emit sockClose();
	} else
		socket->flush();
}
/*
void GFileDlSocket::getList()
{
	char data[1+NODE_ID_SIZE];

	data[0]=fdl_getlist;
	memcpy(data+1,graphe->nodes()[0].nodeid,NODE_ID_SIZE);
	if (socket->writeCryptedBlock(data,1+NODE_ID_SIZE)!=(1+NODE_ID_SIZE))
	{
		socket->close();
		emit sockClose();
		qDebug("getList erreur");
	} else
	{
		socket->flush();
		qDebug("getList fini envoye");
	}
}
*/
void GFileDlSocket::getPart(Q_UINT16 chunk, unsigned char part)
{
	char data[1+2+1];
	part_pos p;

	p.chunk=chunk;
	p.part=part;

#if BYTE_ORDER != LITTLE_ENDIAN
	A FAIRE
	chunk=htons(chunk) //Ordre des bits pour passage sur le reseau
#endif
	data[0]=fdl_getpart;
	memcpy(data+1,&chunk,2);
	data[3]=part;
	if (socket->writeCryptedBlock(data,4)==4)
	{
		part_queue.push_back(p);
	}else
	{
		socket->close();
		emit sockClose();
	}
}


void GFileDlSocket::handShakeFilter()
{
	if (!socket->decryptedBytesAvailable())
		return;
	switch (socket->decryptedGetch())
	{
		case ful_acceptdl:
			disconnect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
			connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(transferFilter()));
			transferFilter();
			break;
		case ful_queue:
			if (socket->decryptedBytesAvailable()>=2)
			{
				char c[2];
				c[0]=socket->decryptedGetch();
				c[1]=socket->decryptedGetch();
				emit inQueue(ntohs(*(Q_UINT16 *)(c)));
			}
			else
				socket->decryptedUngetch(ful_queue);
			break;
		default:
			socket->close();
			emit sockClose();
	}
}
void GFileDlSocket::transferFilter()
{
	unsigned long int len,i;
	char *data,t[MD5_DIGEST_LENGTH*2+1];
	QStringList l;

	if (!socket->decryptedBytesAvailable())
	return;

	if (!remaining)
	{
		switch (socket->decryptedGetch())
		{
			case ful_havefile:
				len=NODE_ID_SIZE+nbchunk*MD5_DIGEST_LENGTH+ (nbchunk+7)/8;
				if (socket->decryptedBytesAvailable()>=len)
				{
					data = (char *)malloc(len);
					socket->readCryptedBlock(data,len);
					data[NODE_ID_SIZE-1]=0;
					nodeid=data;
					chunks.resize(nbchunk);
					for (i=0;i<nbchunk;++i)
					{
						chunks[(int)i]=data[(int)(NODE_ID_SIZE+nbchunk*MD5_DIGEST_LENGTH+i/8)] & (1<<(i%8));
						bin2hex_hash((unsigned char *)data+NODE_ID_SIZE+MD5_DIGEST_LENGTH*i,t);
						l.append(t);
					}
					emit recvFileInf(l,chunks);
					free(data);
				}else
					socket->decryptedUngetch(ful_havefile);
				break;
			case ful_haventfile:
				emit haventFile();
				break;
			case ful_ul:
				if (socket->decryptedBytesAvailable()>=3)
				{
					size=socket->decryptedGetch(); 			//poids faible
					size+=socket->decryptedGetch()*256;		//moyen
					size+=socket->decryptedGetch()*65536;	//fort
					remaining=size;
					dat.resize(remaining);
					transferFilter();			//On relance pour recuperer ce qu'il faut ecrire dans le fichier
				}else
					socket->decryptedUngetch(ful_ul);
				break;
			case ful_new:
				if (socket->decryptedBytesAvailable()>=2)
				{
					i=socket->decryptedGetch()*256;		//poids fort
					i+=socket->decryptedGetch();				//faible
					chunks.setBit(i);
					emit newChunkSource(i);	//On le dit a l'exterieur
				}else
					socket->decryptedUngetch(ful_new);
				break;
			default:
				socket->close();
				emit sockClose();
		}
	}else
	{
		len = socket->readCryptedBlock(&(dat[(int)(size-remaining)]),remaining);
		if (len>0)
		{
			written += len;
			remaining -= len;
			if ( (remaining==0) && (!part_queue.empty()) )
			{
				emit newPart(part_queue.front().chunk,part_queue.front().part,dat);
				part_queue.pop_front();
			}
		}else
		{
			socket->close();
			emit sockClose();
		}
	}
}

void GFileDlSocket::onError(int e)
{
	if ((e==QSocket::ErrConnectionRefused)||(e==QSocket::ErrHostNotFound))
		emit unableToConnect();
	else
		emit sockClose();
}
void GFileDlSocket::onClose()
{
	emit sockClose();
}
void GFileDlSocket::onConnect()
{
	DES_cblock *des_key;
	QString pubk;

	pubk=gconf->pubKey(nodeid);
	if (pubk!=QString::null)
	{
		qDebug("Generation de cle de session");
		des_key=generate_random_key();
		qDebug("Cle generee : %s - envoi du paquet",buffer_bin_to_hex(*des_key,8).latin1());
                QString cryptedsessionkey = rsa_encrypt_printable((unsigned char*)des_key,8,pubk);
                qDebug("Cle de session cryptee : %s", cryptedsessionkey.latin1());
		socket->writeCryptedLine("GDLSOCK "+cryptedsessionkey);
		qDebug("Paquet envoye, liberation de la cle");
                socket->setCryptKey(des_key);
		free(des_key);
		qDebug("Liberation terminee");

		sendGet(hash,filesize);
	}
	else
	{
          emit(NeedKey(dist_uid));
		qDebug("Cle publique inconnu, pas de connexion directe possible");
		socket->close();
	}
}

void GFileDlSocket::close()
{
	socket->close();
}

const QString & GFileDlSocket::nodeId()
{
	return nodeid;
}
