/***************************************************************************
 *            gfileulsocket.cpp
 *
 *  Thu May  5 23:17:20 2005
 *  Copyright  2005  CrYpToNyM
 *  crypt0nym@sf.net
 ****************************************************************************/

#include <string.h>
#include <qdict.h>

#include "gfileulsocket.h"
#include "gnetwork.h"
#include "../graphes.h"
#include "../crypt/crypt_des.h"
#include "../crypt/crypt_rsa.h"


#ifndef min
#define min(a,b) (((a)<(b))?(a):(b))
#endif



#define UPLOAD_TIMER_TIME 200


extern c_graphes *graphe;
extern GConfig * gconf;
extern QString MyPrivKey;
extern GNetwork *net;

GFileUlSocket::GFileUlSocket(const char *nodeid, const QString &ip, Q_UINT16 port, QObject * parent, const char * name)
  : QObject( parent, name )
{
  DES_ks temp;

	qDebug("GFileUlSocket::GFileUlSocket (outgoing)");
  generate_key_from_string(cle_reseau, &temp);

	file_index=-1;
	remaining=0;
	timer=NULL;
	byte_per_tick=0;
	memcpy(distantNodeID,nodeid,NODE_ID_SIZE);
	socket = new GCryptSocket(temp,GULSOCK,this);
	written=0;
	setMaxSpeed(0);
        dist_uid = graphe->NodeidToId(nodeid);

	connect(socket,SIGNAL(connected()),this,SLOT(sendHi()));
	connect(socket,SIGNAL(connectionClosed()),this,SLOT(onClose()));
	connect(socket,SIGNAL(bytesWritten(int)),this,SLOT(onWritten(int)));
        connect(this,SIGNAL(NeedKey(Q_UINT16)),net,SLOT(sendKeyRequest(Q_UINT16)));

	socket->connectToHost(ip,port);
}

GFileUlSocket::GFileUlSocket(const char *nodeid, GCryptSocket* s,QObject * parent, const char * name)
  : QObject( parent, name )
{
	file_index=-1;
	remaining=0;
	timer=NULL;
	byte_per_tick=0;
	memcpy(distantNodeID,nodeid,NODE_ID_SIZE);
	socket=s;
	written=0;
	setMaxSpeed(0);
        dist_uid = graphe->NodeidToId(nodeid);

	connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
	connect(socket,SIGNAL(connectionClosed()),this,SLOT(onClose()));
	connect(socket,SIGNAL(bytesWritten(int)),this,SLOT(onWritten(int)));
        connect(this,SIGNAL(NeedKey(Q_UINT16)),net,SLOT(sendKeyRequest(Q_UINT16)));
}

GFileUlSocket::~GFileUlSocket()
{
	qDebug("GFileUlSocket::~GFileUlSocket()");
	delete timer;
	socket->disconnect();
	socket->deleteLater();
}

void GFileUlSocket::start()
{
	handShakeFilter();
}

void GFileUlSocket::onClose()
{
	qDebug("Fermeture du GFileUlSocket de %s", distantNodeID);
	emit sockClose();
}

void GFileUlSocket::onWritten( int nbytes)
{
	written+=nbytes;
}

void GFileUlSocket::sendHi()
{
	DES_cblock *des_key;

	QString pubk;

	pubk=gconf->pubKey(distantNodeID);
	if (pubk!=QString::null)
	{

		qDebug("GFileUlSocket::sendHi()");
		connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
		qDebug("Generation de cle de session");
		des_key=generate_random_key();
		qDebug("Cle generee : %s - envoi du paquet",buffer_bin_to_hex(*des_key,8).latin1());
                QString cryptedsessionkey = rsa_encrypt_printable((unsigned char*)des_key,8,pubk);
                qDebug("Cle de session cryptee : %s", cryptedsessionkey.latin1());

		socket->writeCryptedLine("GULSOCK "+cryptedsessionkey);
		qDebug("Paquet envoye, liberation de la cle");
		socket->setCryptKey(des_key);
		free(des_key);
		qDebug("Liberation terminee");



		socket->writeCryptedBlock(graphe->nodes()[0].nodeid,NODE_ID_SIZE);

	}
	else
	{
          emit(NeedKey(dist_uid));
		qDebug("Cle publique inconnu, pas de connexion directe possible");
		socket->close();
	}

		/*	if (socket->putCryptedCh(ful_hi)!=ful_hi)
	{
		socket->close();
		emit sockClose();
	}*/
}

unsigned long int GFileUlSocket::getMaxSpeed()
{
	/*return byte_per_tick/512; //byte_per_tick*2 /1024*/
	return byte_per_tick*(1000/UPLOAD_TIMER_TIME);
}

void GFileUlSocket::setMaxSpeed(unsigned long int s)
{
	disconnect(socket,SIGNAL(bytesWritten(int)),this,SLOT(onWriteNoLimit(int)));
	if (s)
	{
		/*byte_per_tick = s*512; // s/2 *1024*/
		byte_per_tick = s/(1000/UPLOAD_TIMER_TIME);
		if (timer==NULL)
		{
			remaining=0;
			timer= new QTimer(this);
			connect(timer,SIGNAL(timeout ()),this,SLOT(sendData()));
			timer->start(UPLOAD_TIMER_TIME);
		}
	}else
	{
		if (timer)	//si on etait en ul limite
		{
			byte_per_tick=0;
			delete timer;
			timer=NULL;
			if (remaining)	//si il en reste a ecrire
			{
				if (!(socket->writeCryptedBlock(dat.data()+dat.size()-remaining,remaining)==(long long int)remaining))
				{
					socket->close();
					emit sockClose();
					return;
				}
			}
		}
		connect(socket,SIGNAL(bytesWritten(int)),this,SLOT(onWriteNoLimit(int)));

	}
}

void GFileUlSocket::onWriteNoLimit(int b)	//Si il n'y a plus de donnees restantes, on demande la suite
{
	qWarning("GFileUlSocket : onWriteNoLimit(%d)",b);
	if (remaining)	//si c'est des donnees d'un part
	{
		if ((int)remaining<=b)	//si le part a ete totalement ecrit
		{
			if (!part_queue.empty())	//s'il reste qqc dans la file
			{
				emit newPart(part_queue.front().chunk,part_queue.front().part);	//On defile
				part_queue.pop_front();
			}
		}else	//si le part est pas totalement ecrit
			remaining-=b;	//on met a jour pour savoir combien il reste a ecrire
	}
}

void GFileUlSocket::sendFileInf(const char *chunkhashes,const QBitArray &chunks)
{
	long long int i;
	i=chunks.size()/8 + (chunks.size() % 8?1:0);

	connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(transferFilter()));

	if ((socket->putCryptedCh(ful_havefile)!=ful_havefile)||
		(socket->writeCryptedBlock(graphe->nodes()[0].nodeid,NODE_ID_SIZE)!=NODE_ID_SIZE) ||
		(socket->writeCryptedBlock(chunkhashes,MD5_DIGEST_LENGTH*chunks.size())!=(long long int) MD5_DIGEST_LENGTH*chunks.size())||
		(socket->writeCryptedBlock(chunks.data(),i)!=i))
	{
		socket->close();
		emit sockClose();
	} else
		transferFilter();
}

void GFileUlSocket::inQueue(Q_UINT16 pos)
{
	pos=htons(pos);
	if ((socket->putCryptedCh(ful_queue)!=ful_queue)||
		(socket->writeCryptedBlock((char *)&pos,2)!=2))
	{
		socket->close();
		emit sockClose();
	}
}

void GFileUlSocket::sendPart(const QByteArray &data)
{
		qDebug("sendPpart avec socket #"+QString::number(index));
		remaining=data.size();
		socket->putCryptedCh(ful_ul);
#if BYTE_ORDER != LITTLE_ENDIAN
		A FAIRE
#else
		socket->putCryptedCh(*((char *) &remaining));
		socket->putCryptedCh(*(((char *) &remaining)+1));
		socket->putCryptedCh(*(((char *) &remaining)+2));
#endif

		if (timer)
		{
			dat=data;
		}else
		{
			if (socket->writeCryptedBlock(data,data.size())!=(long long int)data.size())
			{
				socket->close();
				emit sockClose();
			}
		}
}

void GFileUlSocket::sendData()
{
	long long int i;

	if (socket->bytesToWrite()>5*byte_per_tick)	//Si on a trop de donnees en attente
		return;


	if(remaining)
	{
		i=min(byte_per_tick,remaining);
		if (socket->writeCryptedBlock(dat.data()+dat.size()-remaining,i)!=i)
		{
			socket->close();
			emit sockClose();
			return;
		} else
			remaining-=i;
	}else	if (!part_queue.empty())	//Il n'y a plus de donnees a envoyer mais il y a des demandes en attente
	{
		emit newPart(part_queue.front().chunk,part_queue.front().part);	//On depile
		part_queue.pop_front();
	}
}

void GFileUlSocket::newChunk(Q_UINT16 chunk)
{
	chunk=htons(chunk);
	if (remaining)
	{
		remaining+=3;
		dat.resize(dat.size()+3);
		dat[dat.size()-3]=ful_new;
		dat[dat.size()-2]=((char *)&chunk)[0];
		dat[dat.size()-1]=((char *)&chunk)[1];
	} else
	{
		if ((socket->putCryptedCh(ful_new)!=ful_new)||
		(socket->writeCryptedBlock((char*)&chunk,2)!=2))
		{
			socket->close();
			emit sockClose();
		}
	}
}

void GFileUlSocket::handShakeFilter()
{
	if (!socket->decryptedBytesAvailable())
		return;
	switch (socket->decryptedGetch())
	{
		case fdl_getfile:
			if (socket->decryptedBytesAvailable()>=(NODE_ID_SIZE+MD5_DIGEST_LENGTH+5))
			{
				socket->readCryptedBlock(distantNodeID,NODE_ID_SIZE);
				distantNodeID[NODE_ID_SIZE-1]=0; //Pour la securite :)
				socket->readCryptedBlock(hash,MD5_DIGEST_LENGTH);
#if BYTE_ORDER != LITTLE_ENDIAN
PAS ENCORE IMPLEMENTE
#else
				filesize.c[0]=socket->decryptedGetch();	//poids faible
				filesize.c[1]=socket->decryptedGetch(); 	//moyen
				filesize.c[2]=socket->decryptedGetch();	//moyen
				filesize.c[3]=socket->decryptedGetch();	//moyen
				filesize.c[4]=socket->decryptedGetch();	//fort
				filesize.c[5]=0;
				filesize.c[6]=0;
				filesize.c[7]=0;
#endif
				disconnect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
				socket->putCryptedCh(ful_acceptdl);
				emit askingFile(hash,filesize.val);
			}else
				socket->decryptedUngetch(fdl_getfile);
			break;

/*		case fdl_getlist:
			if (socket->decryptedBytesAvailable()>=(NODE_ID_SIZE+MD5_DIGEST_LENGTH+5))
			{
				qDebug("fdl_getfile");
				socket->readCryptedBlock(distantNodeID,NODE_ID_SIZE);
				distantNodeID[NODE_ID_SIZE-1]=0; //Pour la securite :)
				qDebug("nodeid=%s",distantNodeID);
				disconnect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handShakeFilter()));
				socket->putCryptedCh(ful_acceptdl);
				emit askingList(distantNodeID);
			}else
				socket->decryptedUngetch(fdl_getlist);
			break;
*/
		default:
			socket->close();
			emit sockClose();
	}
}

void GFileUlSocket::transferFilter()
{
	Q_ULONG b;
	if (!socket->decryptedBytesAvailable())
	return;
		switch (socket->decryptedGetch())
		{
			case fdl_getpart:
				if ((b=socket->decryptedBytesAvailable())>=3)
				{
					part_pos p;

					p.chunk=socket->decryptedGetch();		//poids faible
					p.chunk+=socket->decryptedGetch()*256;		//fort
					p.part=socket->decryptedGetch();
					if (remaining||timer)		//si on est en transfert ou si le timer tourne
						part_queue.push_back(p);					//on empile la demande
					else
						emit newPart(p.chunk,p.part);
					if (b>3)	//s'il reste des donnees dans le socket
						transferFilter();
				}else
					socket->decryptedUngetch(fdl_getpart);
				break;
			default:
				socket->close();
				emit sockClose();
		}
}

const QString &GFileUlSocket::nodeName()
{
	if (name.isEmpty())
	{
		Q_UINT16 id=graphe->NodeidToId(distantNodeID);
		if(id<USHRT_MAX)
			name=graphe->nodes()[id].name;
		else
			name=graphe->nodes()[id].nodeid;
	}
	return name;
}

QString GFileUlSocket::nodeId()
{
	return distantNodeID;
}

void GFileUlSocket::close()
{
	socket->close();
}
