/***************************************************************************
 *            gnetwork.cpp
 *
 *  Sun Jan  2 17:15:35 2005
 *  Copyright  2005  Goudal Francois
 *  francois@goudal.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <qobject.h>
#include <qstring.h>
#include <qdeepcopy.h>

#include "gnetwork.h"
#include "../graphes.h"
#include "rfc_gloster.h"
#include "gfilesocketcommon.h"
#include "gfileulsocket.h"
#include "gfiledlsocket.h"
#include "../crypt/gcryptsocket.h"
#include "../crypt/crypt_des.h"

c_graphes *graphe; //Tres moche mais en attendant...
QString cle_reseau = "";

//GNetwork implementation

//Constructor
GNetwork::GNetwork(const QString &nick, const QString &nodeid, const QString &ip, const t_connexion con_type, Q_UINT16 con_speed, bool route, Q_UINT16 nbslot,const QString &incoming, const QString &tempdir, const QString &netkey, QObject * parent , const char * name)
  : QObject( parent, name )
{
  char con_speed_graph;

  cle_reseau = QDeepCopy<QString>(netkey);

  qDebug("[gnetwork.cpp] - Debug : Construction du GNetwork");

  if (route)
    {
      if (con_speed)
        con_speed_graph = con_speed>1000?2:(con_speed>500?3:(con_speed>100?4:(con_speed>50?5:(con_speed>25?6:(con_speed>10?7:(con_speed>5?8:9))))));
      else
        con_speed_graph=1;
    }
  else
    con_speed_graph=0;


  graph = new c_graphes(nick,nodeid,ip,0,con_type,con_speed_graph,this);
  graphe=graph;
  connect(graph,SIGNAL(addnode(Q_UINT16)),this,SLOT(sendstar(Q_UINT16)));
  connect(graph,SIGNAL(addnode(Q_UINT16)),this,SLOT(onNewNode(Q_UINT16)));
  connect(graph,SIGNAL(suppnode(Q_UINT16)),this,SLOT(chatCleanListOnRemoval(Q_UINT16)));
  connect(graph,SIGNAL(suppnode(Q_UINT16)),this,SLOT(onDelNode(Q_UINT16)));
  nbnodes=0;
  nodes=NULL;
  listensocket=NULL;

  chanlist.nb_chans=0;
  chanlist.chans=NULL;

  //init transfert de fichier
  downloader = new GDownloader(tempdir,incoming,this);
  connect(downloader,SIGNAL(askSources(Q_UINT64,const QString&)),this,SLOT(sendSearchByHash(Q_UINT64,const QString &)));
  connect(downloader,SIGNAL(askConnect(const QString&)),this,SLOT(askConnect(const QString &)));
  connect(downloader,SIGNAL(finishedList(const QString &)),this,SLOT(fileListfinished(const QString &)));
  uploader = new GUploader(con_speed,nbslot,downloader->gettempfileVector(),this);

  shareList = new QSharelist(this);
  connect(shareList,SIGNAL(resultToReceive()),this,SLOT(sendResult()));
}

//Destructor
GNetwork::~GNetwork()
{
  int i;

  for (i=0;i<chanlist.nb_chans;i++)
    {
      delete chanlist.chans[i].chan;
      free(chanlist.chans[i].nodes);
    }
  free(chanlist.chans);

  qDebug("[gnetwork.cpp] - Debug : Destruction du GNetwork");

  free(nodes);
  delete graph;
}


Q_UINT16 GNetwork::getnbnodes()           //nb of nodes in the network
{
  return nbnetnodes;
};

bool GNetwork::listen(Q_UINT16 port)
{
  if (listensocket)
    delete listensocket;

  listensocket = new GListenSocket(port,this);

  if (listensocket->ok())
    {
      //connect(listensocket,SIGNAL(NewConn(GSocket *)),this,SLOT(incomingconnexion(GSocket *)));
      connect(listensocket,SIGNAL(NewConn(int)),this,SLOT(incomingSocket(int)));
			graph->nodes(0).port=port;
			qDebug("Port d'ecoute ds le graphe : %d - et en vrai : %d",graph->nodes()[0].port, port);
			return true;
    }
  else
    {
        delete listensocket;
        listensocket=NULL;
				graph->nodes(0).port=0;
        return false;
    }
}

void GNetwork::incomingSocket(int socket)
{
  DES_ks temp;

  generate_key_from_string(cle_reseau, &temp);

  GCryptSocket* s = new GCryptSocket(temp, this );
  connect(s,SIGNAL(readyDecryptedRead()),this,SLOT(incomingSocketFilter()));
  connect(s,SIGNAL(connectionClosed()),this,SLOT(deleteQSocket()));
  s->setSocket( socket );
}

void GNetwork::deleteQSocket()
{
	qDebug("deleteQSocket : destruction d'un socket entrant non identifie");
	GCryptSocket* socket = (GCryptSocket*)sender();
	delete socket;
}

void GNetwork::incomingSocketFilter()
{
	GSocket *s;
	char nodeid[NODE_ID_SIZE];

	GCryptSocket *socket = (GCryptSocket*)sender();
	//	QString socket_id = socket->readCryptedLine();
	if (socket->type==GULSOCK)
	  {
	    qDebug("Socket entrant : Socket d'upload");
			socket->disconnect(this);
			//Il faut passer le socket a l'objet qui gere les sockets d'ul
			uploader->incomingUl("NODE_ID_DU_SOCKET_ENTRANT",socket);
	  }
	else if (socket->type==GDLSOCK)
	  {
	    qDebug("Socket entrant : Socket de download");
			socket->disconnect(this);
			//Il faut passer le socket a l'objet qui gere les sockets de dl
			socket->readCryptedBlock(nodeid,NODE_ID_SIZE);
			nodeid[NODE_ID_SIZE-1]=0;
			downloader->incomingSocket(nodeid,socket);
	  }
	else if (socket->type==GSOCKET)
	  {
	    qDebug("Socket entrant : Socket GSocket");
			socket->disconnect(this);
			s = new GSocket( socket,graph,this );
			incomingconnexion(s);
			s->connectIncomingFilter();
	  }
}


void GNetwork::connecttohost(const QString &IPAddr, Q_UINT16 Port)
{
	GSocket *s = new GSocket( this, graph );
	connect(s, SIGNAL(connected()),this,SLOT(connexionsucceed()));
	connect(s, SIGNAL(connectionClosed()), this, SLOT(discardClient()) );
	connect(s, SIGNAL(ConnectStar()),this,SLOT(SocketConnected()));
	connect(s, SIGNAL(ConnectStar()),this,SLOT(broadcaststar()));
	connect(s, SIGNAL(newNode(Q_UINT16)),this,SLOT(onNewNode(Q_UINT16)));

	s->connectToHost(IPAddr,Port);

}

void GNetwork::SocketConnected()
{
	GSocket *s = (GSocket *) sender();
	emit connected(s->peerName(),s->peerPort());
}


void GNetwork::discardClient()
{


	GSocket* socket = (GSocket*)sender();
	if (socket->GetID())
	{
		qDebug("[gnetwork.cpp] - Debug : Suppression du node %d",socket->GetID());
		graph->show_graph();
  	graph->supp_connection(0,socket->GetID());
		broadcaststar();
		graph->show_graph();
	} else
		qDebug("[gnetwork.cpp] - Debug : Suppression du socket d'un client absent du graphe");

	emit disconnected(socket->peerName(),socket->peerPort());
	delete socket;

}
/*
void GNetwork::linknode(Q_UINT16 node)
{
  node=0; //Inutile, juste pour virer le warning a la compilation

creation d'un nouveau socket et connexion
    Le probleme c'est de trouver la bonne methode pour les id
    ensuite broadcast du nouveau lien

}

void GNetwork::unlinknode(Q_UINT16 node)
{
    nodes[node]->close();
    delete nodes[node];

    Broadcaster la coupure du lien
    ou alors le faire après l'emission du signal... a voir!

    emit(unlinked(0,node));
}
*/
void GNetwork::sendChatUserEvent(const QString &channame, bool Connection)
{
  unsigned short int i,j;
  QString packet = channame;

  if (Connection)
  {
    packet+=" 1";
    for (i=0;i<chanlist.nb_chans;i++)
			if (channame==*chanlist.chans[i].chan)
			{
		  	for(j=0;j<chanlist.chans[i].nb_nodes;j++)
					emit recvChatUserEvent(graph->sommets_list[chanlist.chans[i].nodes[j]].name,channame,true);
	  		break;
			}
  }
  else
  	packet+=" 0";

	onChatUserEvent(0,channame,Connection);
  broadcast(RFC_CUEV, packet);

}

void GNetwork::sendChatMsg(const QString &channame, const QString &msg)
{
  int i;

  QString packet = channame;

  packet+=" ";
  packet+=msg;

  for (i=0;i<chanlist.nb_chans;i++)
    if (channame==*chanlist.chans[i].chan)
		{
    	multicast(RFC_CMSG,chanlist.chans[i].nodes ,chanlist.chans[i].nb_nodes ,packet);
      break;
		}


  qDebug("[gnetwork.cpp] - Debug : Envoi d'un message de Chat dans le Chan %s",channame.latin1());

}

void GNetwork::sendChatPv(Q_UINT16 nodeid, const QString &msg)
{
  unicast(RFC_CMPV, nodeid, msg, Delay);
}

void GNetwork::sendPublicKey(Q_UINT16 uid)
{
  if (graphe->uidExists(uid))
    unicast(RFC_GKEY, uid, gconf->myPubKey(), Delay);
}

void GNetwork::sendKeyRequest(Q_UINT16 uid)
{
  if (graphe->uidExists(uid))
    unicast(RFC_WKEY, uid);
}

void GNetwork::getFileList(Q_UINT16 uid)
{
	qDebug("GNetwork::getFileList(%i)",uid);
	unicast(RFC_FLST, uid);
}

void GNetwork::recvFileList(Q_UINT16 uid)
{
	qDebug("GNetwork::recvFileList(%i)",uid);
	unicast(RFC_RFLS,uid,QString::number(shareList->fileListSize())+' '+shareList->fileListHash());
}

void GNetwork::recvFileListHash(Q_UINT16 uid, Q_UINT64 size, const QString &hash)
{
	qDebug("GNetwork::recvFileListHash");
	if (size)
		downloader->getFileList(uid,graphe->sommets_list[uid].nodeid,size,hash);
	else
		emit recvFileList("");
}

void GNetwork::fileListfinished(const QString &file)
{
	qDebug("GNetwork::fileListfinished "+file);
	emit recvFileList(file);
}

void GNetwork::onChatUserEvent(Q_UINT16 user, const QString &channame, bool Connection)
{
  unsigned short int i,j;

  qDebug("[gnetwork.cpp] - Reception d'un evenement de chat : Utilisateur : %d, Chan : %s,Connexion : %d", user, channame.latin1(), Connection);
  for (i=0;i<chanlist.nb_chans;i++)
    if (channame==*chanlist.chans[i].chan)
      break;
  if (i<chanlist.nb_chans)
    {
      //Le chan existe deja
      for (j=0;j<chanlist.chans[i].nb_nodes;j++)
	  		if (chanlist.chans[i].nodes[j]==user)
	    		break;
      if ((j<chanlist.chans[i].nb_nodes)&&(!Connection))
	{
	  // Si il existe et veut se deconnecter
	  if (chanlist.chans[i].nb_nodes==1)
	    {
	      emit(recvChatChanEvent(channame, false));
	      //Dernier user dans le chat se deconnecte, on vire le chan
	      delete chanlist.chans[i].chan;
	      free(chanlist.chans[i].nodes);
	      if (i!=chanlist.nb_chans-1)
	        memcpy(chanlist.chans+i,chanlist.chans+chanlist.nb_chans-1,sizeof(s_chat_user_list));
	      chanlist.nb_chans--;
	      chanlist.chans=(s_chat_user_list *)realloc(chanlist.chans,chanlist.nb_chans*sizeof(s_chat_user_list));
	      if (chanlist.nb_chans==0)
		chanlist.chans=NULL;
	    }
	  else
	    {
	      //Il reste des gens ds le chan, on le vire et on defragmente
	      if (j!=chanlist.chans[i].nb_nodes-1)
	        memcpy(chanlist.chans[i].nodes+j,chanlist.chans[i].nodes+chanlist.chans[i].nb_nodes-1,sizeof(Q_UINT16));
	      chanlist.chans[i].nb_nodes--;
	      chanlist.chans[i].nodes=(Q_UINT16 *)realloc(chanlist.chans[i].nodes,chanlist.chans[i].nb_nodes*sizeof(Q_UINT16));
	      emit(recvChatUserEvent(graph->sommets_list[user].name, channame, false));
	    }
	}
      else if ((j==chanlist.chans[i].nb_nodes) && Connection)
	{
	  //Si il n'existe pas encore et veut se connecter dans un chan qui existe
	  chanlist.chans[i].nb_nodes++;
	  chanlist.chans[i].nodes=(Q_UINT16 *)realloc(chanlist.chans[i].nodes,chanlist.chans[i].nb_nodes*sizeof(Q_UINT16));
	  chanlist.chans[i].nodes[chanlist.chans[i].nb_nodes-1]=user;
	  emit(recvChatUserEvent(graph->sommets_list[user].name, channame, true));
	}
    }
  else if (Connection)
    {

      //Creation d'un chan et d'un user dans ce chan.
      chanlist.nb_chans++;
      chanlist.chans=(s_chat_user_list *)realloc(chanlist.chans,chanlist.nb_chans*sizeof(s_chat_user_list));
      chanlist.chans[chanlist.nb_chans-1].chan = new QString(channame);
      chanlist.chans[chanlist.nb_chans-1].nodes=NULL;
      chanlist.chans[chanlist.nb_chans-1].nb_nodes=0;

      chanlist.chans[chanlist.nb_chans-1].nb_nodes++;
      chanlist.chans[chanlist.nb_chans-1].nodes=(Q_UINT16 *)realloc(chanlist.chans[chanlist.nb_chans-1].nodes,chanlist.chans[chanlist.nb_chans-1].nb_nodes*sizeof(Q_UINT16));
      chanlist.chans[chanlist.nb_chans-1].nodes[chanlist.chans[chanlist.nb_chans-1].nb_nodes-1]=user;
      emit(recvChatChanEvent(channame, true));
      emit(recvChatUserEvent(graph->sommets_list[user].name, channame, true));
    }

}


void GNetwork::chatCleanListOnRemoval(Q_UINT16 node)
{
  int i,j;

    qDebug("[gnetwork.cpp] - Suppression du node %d de la liste du chat pour cause de depart du reseau",node);
  for (i=0;i<chanlist.nb_chans;i++)
      for (j=0;j<chanlist.chans[i].nb_nodes;j++)
	  if (chanlist.chans[i].nodes[j]==node)
	    {
	      onChatUserEvent(node, *(chanlist.chans[i].chan), false);
	      break;
	    }

}

void GNetwork::onChatMsg(const QString &sender, const QString &channame, const QString &msg)
{
  emit(recvChatMsg( sender, channame, msg));
}

void GNetwork::onChatPv(Q_UINT16 node, const QString &sender, const QString &msg)
{
	if(!graphe->isBlocked(node))
		emit(recvChatPv(node, sender, msg));
}


void GNetwork::onKeyRequest(Q_UINT16 uid)
{
  emit(recvKeyRequest(uid));
}

void GNetwork::onGiveKey(QString nodeid, QString key)
{
  emit(recvGiveKey(nodeid, key));
}


void GNetwork::multicast(const QString &packettype,Q_UINT16 *destids,Q_UINT16 nbdest,const QString &data)
{
  s_broadcast_dc_list * destlist;
  int i,j;
  char *temp;
  QString packet;


  destlist=graph->Get_MulticastLists(destids, nbdest);

  for (j=0;j<destlist->nb_dc;j++)
    {
      if (destlist->dc[j].nbnodes!=0)
	{
	  temp = (char *)malloc(NODE_ID_SIZE+1);
	  memcpy(temp, graph->sommets_list[0].nodeid,NODE_ID_SIZE);
	  temp[NODE_ID_SIZE]=0;
	  packet = packettype+" ";
		packet+=temp;
	  free(temp);
	  packet+=" ";
	  for (i=0;i<destlist->dc[j].nbnodes-1;i++)
	    {
	      temp = (char *)malloc(NODE_ID_SIZE+1);
	      memcpy(temp, graph->sommets_list[destlist->dc[j].list[i]].nodeid,NODE_ID_SIZE);
	      temp[NODE_ID_SIZE]=0;
	      packet+=temp;
	      free(temp);
	      packet+=",";
	    }
	  temp = (char *)malloc(NODE_ID_SIZE+1);
	  memcpy(temp, graph->sommets_list[destlist->dc[j].list[i]].nodeid,NODE_ID_SIZE);
	  temp[NODE_ID_SIZE]=0;
	  packet+=temp;
	  free(temp);
	  packet+=" ";

	  packet+=data;

	  graph->sommets_list[graph->GoToNode(destlist->dc[j].i, Delay)].socket->writeCryptedLine(packet);

	}
    }
}

void GNetwork::routage(const QString &packettype,const QString &senderid, Q_UINT16 *destids,Q_UINT16 nbdest,const QString &data)
{
  s_broadcast_dc_list * destlist;
  int i,j;
  char *temp;
	QString packet;


  destlist=graph->Get_MulticastLists(destids, nbdest);

  for (j=0;j<destlist->nb_dc;j++)
    {
      if (destlist->dc[j].nbnodes!=0)
	{
	  packet = packettype+" ";
		packet+=senderid;
	  packet+=" ";
	  for (i=0;i<destlist->dc[j].nbnodes-1;i++)
	    {
	      temp = (char *)malloc(NODE_ID_SIZE+1);
	      memcpy(temp, graph->sommets_list[destlist->dc[j].list[i]].nodeid,NODE_ID_SIZE);
	      temp[NODE_ID_SIZE]=0;
	      packet+=temp;
	      free(temp);
	      packet+=",";
	    }
	  temp = (char *)malloc(NODE_ID_SIZE+1);
	  memcpy(temp, graph->sommets_list[destlist->dc[j].list[i]].nodeid,NODE_ID_SIZE);
	  temp[NODE_ID_SIZE]=0;
	  packet+=temp;
	  free(temp);
	  packet+=" ";

	  packet+=data;

	  graph->sommets_list[graph->GoToNode(destlist->dc[j].i, Delay)].socket->writeCryptedLine(packet);

	}
    }
}

void GNetwork::unicast(const QString &packettype,Q_UINT16 destid,const QString &data, e_ParcoursMode mode)
{
  QString packet = packettype + " ";

  packet+=graph->sommets_list[0].nodeid;
  packet+=" ";
  packet+=graph->sommets_list[destid].nodeid;
  packet+=" ";
  packet+=data;
  graph->sommets_list[graph->GoToNode(destid, mode)].socket->writeCryptedLine(packet);
}


void GNetwork::broadcast(const QString &packettype,const QString &data)
{
  s_broadcast_dc_list * destlist;
  int i,j;
  QString packet;


  destlist=graph->Get_BroadcastLists();

  for (j=0;j<destlist->nb_dc;j++)
  {
  	if (destlist->dc[j].nbnodes!=0)
		{
	  	packet = packettype+" ";
			packet+=graph->sommets_list[0].nodeid;
	  	packet+=" ";
	  	for (i=0;i<destlist->dc[j].nbnodes-1;i++)
	    {
	      packet+=graph->sommets_list[destlist->dc[j].list[i]].nodeid;
	      packet+=",";
	    }
		  packet+=graph->sommets_list[destlist->dc[j].list[i]].nodeid;
		  packet+=" ";

	  	packet+=data;

	  	graph->sommets_list[graph->GoToNode(destlist->dc[j].i, Delay)].socket->writeCryptedLine(packet);

		}
  }
}


void GNetwork::sendstar(Q_UINT16 node)
{
  QString star;
  int i,j;

  qDebug("[gnetwork.cpp] - Debug : Envoi d'etoile au node %d\n",node);
  star=graph->Get_Star();

  star+=";";
  for (i=0;i<chanlist.nb_chans;i++)
  {
    for (j=0;j<chanlist.chans[i].nb_nodes;j++)
		{
	  	if (chanlist.chans[i].nodes[j]==0)
	    {
				star+=*(chanlist.chans[i].chan);
	  		star+=";";
	  		break;
			}
		}
  }
  qDebug("L'etoile est : %s",star.latin1());
  unicast(RFC_STAR, node, star);
}

void GNetwork::broadcaststar()
{
  QString star;
  int i,j;

  qDebug("[gnetwork.cpp] - Debug : Envoi d'etoile en broadcast");
  star=graph->Get_Star();

  star+=";";
  for (i=0;i<chanlist.nb_chans;i++)
  {
    for (j=0;j<chanlist.chans[i].nb_nodes;j++)
		{
	  	if (chanlist.chans[i].nodes[j]==0)
			{
	    	star+=*(chanlist.chans[i].chan);
	      star+=";";
				break;
			}
    }
	}
  qDebug("L'etoile est : %s",star.latin1());
  broadcast(RFC_STAR, star);
}

void GNetwork::netupdate(QStringList &star)
{
  graph->show_graph();
	qDebug("[gnetwork.cpp] Debug - Etoile receptionnee, mise a jour du graphe");
	graph->Put_Star(star);
	graph->show_graph();
  //Penser a emettre un message pour la GUI pour le rafraichissement de la liste
}

void GNetwork::chanupdate(Q_UINT16 node, QStringList &chanlist)
{
  for ( QStringList::Iterator it = chanlist.begin(); it != (chanlist.end()); ++it )
    {
      onChatUserEvent(node, *it, true);
      qDebug("Etoile Chan : %s",(*it).latin1() );
    }

}

void GNetwork::incomingconnexion(GSocket * Sock) /* Connexion entrante */
{

	connect(Sock, SIGNAL(connectionClosed()), this, SLOT(discardClient()) );
	connect(Sock, SIGNAL(newNode(Q_UINT16)),this,SLOT(onNewNode(Q_UINT16)));
	connect(Sock,SIGNAL(ConnectStar()),this,SLOT(SocketConnected()));
	connect(Sock,SIGNAL(ConnectStar()),this,SLOT(broadcaststar()));
	connect(Sock,SIGNAL(recvStar(QStringList &)),this,SLOT(netupdate(QStringList &)));
	connect(Sock,SIGNAL(recvChanStar(Q_UINT16, QStringList &)),this,SLOT(chanupdate(Q_UINT16, QStringList &)));

	connect(Sock, SIGNAL(routePacket(const QString &,const QString &,Q_UINT16 *,Q_UINT16,const QString &)),this,SLOT(routage(const QString &,const QString &,Q_UINT16 *,Q_UINT16,const QString &)));

	connect(Sock, SIGNAL(recvChatUserEvent(Q_UINT16, const QString &, bool)), this, SLOT(onChatUserEvent(Q_UINT16, const QString &, bool)));
	connect(Sock, SIGNAL(recvChatMsg(const QString &, const QString &, const QString &)), this, SLOT(onChatMsg(const QString &, const QString &, const QString &)));
	connect(Sock, SIGNAL(recvChatPv(Q_UINT16, const QString &, const QString &)), this, SLOT(onChatPv(Q_UINT16,const QString &, const QString &)));

	connect(Sock, SIGNAL(recvSearchByName(Q_UINT16,const QString &,unsigned int)),this,SLOT(startSearchFile(Q_UINT16,const QString &,unsigned int)));
	connect(Sock, SIGNAL(recvSearchByNameAns(Q_UINT16,const QString &,const QStringList &)),this,SLOT(recvSearchByNameAns(Q_UINT16,const QString &,const QStringList &)));
	connect(Sock, SIGNAL(recvSearchByHash(unsigned short int,const QString&,Q_UINT64)), this, SLOT(recvSearchByHash(unsigned short int,const QString&,Q_UINT64)));
	connect(Sock, SIGNAL(recvSearchByHashAns(unsigned short int,const QString&,Q_UINT64,const QString&)), this, SLOT(recvSearchByHashAns(unsigned short int,const QString&,Q_UINT64,const QString&)));

	connect(Sock, SIGNAL(recvKeyRequest(Q_UINT16)), this, SLOT(onKeyRequest(Q_UINT16)));
        connect(Sock, SIGNAL(recvGiveKey(QString, QString)), this, SLOT(onGiveKey(QString, QString)));


	connect(Sock, SIGNAL(recvFileList(Q_UINT16)), this, SLOT(recvFileList(Q_UINT16)));
	connect(Sock, SIGNAL(recvFileListHash(Q_UINT16,Q_UINT64,const QString &)), this, SLOT(recvFileListHash(Q_UINT16,Q_UINT64,const QString &)));

	connect(Sock, SIGNAL(recvAskConnect(Q_UINT16)), this, SLOT(recvAskConnect(Q_UINT16)));

	nbnodes++;
	nodes=(GSocket **)realloc(nodes,nbnodes*sizeof(GSocket *));
	nodes[nbnodes-1]=Sock;
}

void GNetwork::connexionsucceed() /* Connexion sortante */
{
	GSocket* Sock = (GSocket*)sender();

	connect(Sock,SIGNAL(recvStar(QStringList &)),this,SLOT(netupdate(QStringList &)));
	connect(Sock,SIGNAL(recvChanStar(Q_UINT16, QStringList &)),this,SLOT(chanupdate(Q_UINT16, QStringList &)));

	connect(Sock, SIGNAL(routePacket(const QString &,const QString &,Q_UINT16 *,Q_UINT16,const QString &)),this,SLOT(routage(const QString &,const QString &, Q_UINT16 *,Q_UINT16,const QString &)));

	connect(Sock, SIGNAL(recvChatUserEvent(Q_UINT16, const QString &, bool)), this, SLOT(onChatUserEvent(Q_UINT16, const QString &, bool)));
	connect(Sock, SIGNAL(recvChatMsg(const QString &, const QString &, const QString &)), this, SLOT(onChatMsg(const QString &, const QString &, const QString &)));
	connect(Sock, SIGNAL(recvChatPv(Q_UINT16, const QString &, const QString &)), this, SLOT(onChatPv(Q_UINT16, const QString &, const QString &)));

	connect(Sock, SIGNAL(recvSearchByName(Q_UINT16,const QString &,unsigned int)),this,SLOT(startSearchFile(Q_UINT16,const QString &,unsigned int)));
	connect(Sock, SIGNAL(recvSearchByNameAns(Q_UINT16,const QString &,const QStringList &)),this,SLOT(recvSearchByNameAns(Q_UINT16,const QString &,const QStringList &)));
	connect(Sock, SIGNAL(recvSearchByHash(unsigned short int,const QString&,Q_UINT64)), this, SLOT(recvSearchByHash(unsigned short int,const QString&,Q_UINT64)));
	connect(Sock, SIGNAL(recvSearchByHashAns(unsigned short int,const QString&,Q_UINT64,const QString&)), this, SLOT(recvSearchByHashAns(unsigned short int,const QString&,Q_UINT64,const QString&)));

	connect(Sock, SIGNAL(recvKeyRequest(Q_UINT16)), this, SLOT(onKeyRequest(Q_UINT16)));
        connect(Sock, SIGNAL(recvGiveKey(QString, QString)), this, SLOT(onGiveKey(QString, QString)));

	connect(Sock, SIGNAL(recvFileList(Q_UINT16)), this, SLOT(recvFileList(Q_UINT16)));
	connect(Sock, SIGNAL(recvFileListHash(Q_UINT16,Q_UINT64,const QString &)), this, SLOT(recvFileListHash(Q_UINT16,Q_UINT64,const QString &)));

	connect(Sock, SIGNAL(recvAskConnect(Q_UINT16)), this, SLOT(recvAskConnect(Q_UINT16)));

	nbnodes++;
	nodes=(GSocket **)realloc(nodes,nbnodes*sizeof(GSocket *));
	nodes[nbnodes-1]=Sock;
}


void GNetwork::onNewNode(Q_UINT16 uid)
{
	qDebug("Mettre le nouveau node %i dans la liste de contacts",uid);
	emit newNode(uid,graph->nodes(uid).name,graph->nodes(uid).ConnexionType);
	emit newNode(uid);
	emit newcnt(graph->nodes(uid).port,graph->nodes(uid).host);
}
void GNetwork::onDelNode(Q_UINT16 uid)
{
	((c_graphes *) sender())->show_graph();
	qDebug("Suppression du node %i",uid);
	emit delNode(uid);
}


void GNetwork::startSearchFile(Q_UINT16 uid, const QString& name, unsigned int type)
{
	searchSenders << uid;
	shareList->searchFile(name,type);
}


void GNetwork::sendResult()
{
	t_searchResult result;
	QString wordSearched;
	QString msg;

	wordSearched = ((QSharelist *)sender())->getResult(result);
	if(!result.empty())
	{
		msg = wordSearched + "/";

		t_searchResult::iterator it;
		for ( it = result.begin(); it != result.end(); ++it )
			msg += (*it).name + " " + QString::number((*it).size) + " " + (*it).hash + " " + QString::number((*it).type) + ";";
		unicast(RFC_RSRH,searchSenders.front(),msg);
		searchSenders.pop_front();
	}
}

void GNetwork::recvSearchByNameAns(Q_UINT16 from, const QString &srchstr, const QStringList &res)
{
	QStringList result=res;
	QString s = QDeepCopy<QString>(srchstr);

	for (QStringList::Iterator it = result.begin(); it != result.end(); ++it )
	{
		qDebug("resultat recu : " + s + " " +*it);
		qDebug((*it).section(' ',0,-4));
		emit recvSearchResult(s,(*it).section(' ',0,-4),(*it).section(' ',-3,-3),(*it).section(' ',-2,-2), (*it).section(' ',-1).toLong(),graph->nodes()[from].name);
	}
	emit searchProgress();
}

void GNetwork::sendSearchFile(const QString& name, unsigned int type)
{
	broadcast(RFC_SRCH,QString::number(type)+' '+name);
}

void GNetwork::recvSearchByHash(unsigned short int from, const QString &hash, Q_UINT64 size)
{
	qDebug("GNetwork::recvSearchByHash(%i,%s,%llu)",from,hash.latin1(),size);
	QString s;
	if ((s=shareList->haveFile(hash,size))!=QString::null)
		unicast(RFC_HRSR,from,hash + " " + QString::number(size) + " " + s.section('/',-1));
	else
		qDebug("fichier non trouve");
}
void GNetwork::recvSearchByHashAns(unsigned short int from, const QString &hash, Q_UINT64 size, const QString &filename)
{
	qDebug("GNetwork::recvSearchByHashAns(%i,%s,%llu,%s)",from,hash.latin1(),size,filename.latin1());
	downloader->addSource(from,graph->nodes()[from].nodeid,filename,size,hash);
}
void GNetwork::sendSearchByHash(Q_UINT64 filesize, const QString &hash)
{
	qDebug("GNetwork::sendSearchByHash(%llu,%s)",filesize,hash.latin1());
	broadcast(RFC_HSRH,hash+" "+QString::number(filesize));
}

void GNetwork::askConnect(const QString &nodeid)
{
	qDebug("GNetwork::askConnect "+nodeid);
	Q_UINT16 id=graphe->NodeidToId(nodeid);
	if (id<USHRT_MAX)
		unicast(RFC_ACON, id);
}

void GNetwork::recvAskConnect(Q_UINT16 uid)
{
	qDebug("GNetwork::recvAaskConnect "+QString::number(uid));
	uploader->outgoingUl( graphe->sommets_list[uid].nodeid, graphe->sommets_list[uid].host,graphe->sommets_list[uid].port);
}

void GNetwork::download(const QString &filename,Q_UINT64 filesize,const QString &hash)
{
	downloader->download(filename,filesize,hash);
}

void GNetwork::cancel(Q_UINT64 filesize,const QString &hash)
{
	downloader->cancel(filesize,hash);
}
