/***************************************************************************
 *            gnetwork.h
 *
 *  Sun Jan  2 17:14:21 2005
 *  Copyright  2005  Goudal Francois
 *  francois@goudal.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GNETWORK_H
#define _GNETWORK_H

#include <stdlib.h>
#include <qstring.h>
#include <qobject.h>
#include "../gtypes.h"
#include "../graphes.h"
#include "gsocket.h"
#include "glistensocket.h"
#include "../files/guploader.h"
#include "../files/gdownloader.h"
#include "../files/qsharelist.h"
#include "../gconfig.h"

extern GConfig * gconf;
extern QSharelist * shareList;
extern QString cle_reseau;
extern QDict<QString> Keys;

class GNetwork : public QObject
{
  Q_OBJECT

    public:
	GNetwork(const QString &nick, const QString &nodeid, const QString &ip, const t_connexion con_type, Q_UINT16 con_speed, bool route, Q_UINT16 nbslot, const QString &incoming, const QString &tempdir, const QString &netkey, QObject * parent = 0,const char * name = 0); //constructor
	~GNetwork();                        //destructor

	Q_UINT16 getnbnodes();           //nb of nodes in the network
  // Pour le graphe en 3d dans l'interface
	c_graphes *GetGraph() {return graph;}
	bool listen(Q_UINT16 port);               //Listen on selected port

	//Transfert de fichier
	GDownloader *downloader;
	GUploader *uploader;

	s_chan_list chanlist;

	public slots:
		void connecttohost(const QString &IPAddr, Q_UINT16 Port);
		/*  void linknode(Q_UINT16 node);             //create a direct connection
		  void unlinknode(Q_UINT16 node);           //close a direct connection*/

		void sendstar(Q_UINT16 node);
		void broadcaststar();

	//send our star to node
		void netupdate(QStringList &); //when a star is received
		void chanupdate(Q_UINT16 node,QStringList &); //when a star is received

		void sendChatUserEvent(const QString &channame, bool Connection);
		void sendChatMsg(const QString &channame, const QString &msg);
		void sendChatPv(Q_UINT16 nodeid, const QString &msg);

        	void sendPublicKey(Q_UINT16 uid);
        	void sendKeyRequest(Q_UINT16 uid);



		void sendSearchFile(const QString& name, unsigned int type = 8);
		void getFileList(Q_UINT16 uid);

	//Passer ca en pv?
		void onChatUserEvent(Q_UINT16 user, const QString &channame, bool Connection);
		void onChatMsg(const QString &sender, const QString &channame, const QString &msg);
		void onChatPv(Q_UINT16 node, const QString &sender, const QString &msg);
        	void onKeyRequest(Q_UINT16 uid);
        	void onGiveKey(QString nodeid, QString key);

		void chatCleanListOnRemoval(Q_UINT16 node);

	//Pour la fenetre de transfert
		void download(const QString &filename,Q_UINT64 filesize,const QString &hash);
		void cancel(Q_UINT64 filesize,const QString &hash);


	signals:
		  /*void linked(Q_UINT16 node1,Q_UINT16 node2);
		  void unlinked(Q_UINT16 node1, Q_UINT16 node2);*/
		void connected(const QString &IPAddr, Q_UINT16 Port);
		void disconnected(const QString &IPAddr, Q_UINT16 Port);
		void newNode(Q_UINT16 uid, const QString &nick, t_connexion ConnexionType);    //le prototype est a revoir... doit-on y 		mettre les liens? ca depend a quel moment le signal est emit : avant l'ajout ou apres?
		void newNode(Q_UINT16 uid);
		void newcnt(Q_UINT16 Port, const QString &IPAddr);
		void delNode(Q_UINT16 node);
		//void recvKey(UserURL user);
	
		void recvSearchResult(const QString &srch,const QString &file,const QString &size,const QString &hash, unsigned int filetype, const QString &nick);
		void searchProgress();

		void recvChatChanEvent(const QString &channame, bool creation);
		void recvChatUserEvent(const QString &name, const QString &channame, bool connection);
		void recvChatMsg( const QString &sender, const QString &channame, const QString &msg);
		void recvChatPv(Q_UINT16 node, const QString &sender, const QString &msg);
		
		void recvFileList(const QString &file);
	
		void recvKeyRequest(Q_UINT16 uid);
		void recvGiveKey(QString nodeid, QString key);


		//void recvSendFile(Q_UINT16 node,FileURL url);
		//void recvSendPrivateFile(Q_UINT16 node,FileURL url);

	protected slots:
		void incomingconnexion(GSocket * Sock);		//an incoming connexion arrived
		void connexionsucceed();		//a connexion succeeded

		void broadcast( const QString &packettype, const QString &data);  //broadcast the given data
		void multicast( const QString &packettype, Q_UINT16 *destids, Q_UINT16 nbdest, const QString &data);
		void unicast( const QString &packettype, Q_UINT16 destid, const QString &data=QString::null, e_ParcoursMode mode=Delay);
		void routage(const QString &packettype,const QString &senderid, Q_UINT16 *destids,Q_UINT16 nbdest,const QString &data);

	private slots:
		void SocketConnected();
		void startSearchFile(Q_UINT16 uid, const QString& name, unsigned int type = 8);
		void recvSearchByNameAns(Q_UINT16 from, const QString &srchstr, const QStringList &filelist);
		void sendSearchByHash(Q_UINT64 size, const QString &hash);
		void recvSearchByHash(unsigned short int from, const QString &hash, Q_UINT64 size);
		void recvSearchByHashAns(unsigned short int from, const QString &hash, Q_UINT64 size, const QString &filename);
		void recvFileList(Q_UINT16 uid);
		void recvFileListHash(Q_UINT16 uid, Q_UINT64 size, const QString &hash);
		void fileListfinished(const QString &file);
	
		void askConnect(const QString &nodeid);
		void recvAskConnect(Q_UINT16 uid);


		void discardClient();
		void deleteQSocket();
		void incomingSocket(int socket);
		void incomingSocketFilter();
		void onNewNode(Q_UINT16 uid);
		void onDelNode(Q_UINT16 uid);
		void sendResult();
	
	private:

	  	GListenSocket *listensocket;         //listen socket attached to this client

	  	GSocket **nodes;                    //list of directly connected nodes
	  	//    Q_UINT16 nodeslistsize;             //list size
	  	Q_UINT16 nbnodes;                   //nb of directly connected nodes

	  	Q_UINT16 nbnetnodes;                //nb of nodes in the network

	  	c_graphes * graph;

		//Variables pour la recherche de fichier
		QValueList<Q_UINT16> searchSenders;
};



#endif /* _GNETWORK_H */
