/***************************************************************************
 *	      goscket.cpp
 *
 *  Sun Feb 27 23:30:54 2005
 *  Copyright  2005  User
 *  Email
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "gsocket.h"
#include "rfc_gloster.h"
#include "../crypt/gcryptsocket.h"
#include "../crypt/crypt_des.h"
#include "gnetwork.h"


GSocket::GSocket(GCryptSocket *s,c_graphes *g,QObject * parent, const char * name)
  : QObject( parent, name )
{
  qDebug("GSocket::GSocket : Transformation QSocket en GSocket");
  socket=s;
  id=0;
  graph=g;
  peerP=s->peerPort();
  peerN=s->peerAddress().toString();
  connect(socket, SIGNAL(delayedCloseFinished()), this, SLOT(discardClient()) );
  connect(socket, SIGNAL(connectionClosed()), this, SLOT(discardClient()) );
  connect(socket, SIGNAL(error(int)), this, SLOT(discardClientErr(int)));
}

GSocket::GSocket(QObject * parent, c_graphes *g, const char * name)
  : QObject( parent, name )
{
  DES_ks temp;

  qDebug("[gsocket.cpp] - Debug : Construction du GSocket");

  generate_key_from_string(cle_reseau, &temp);

  socket=new GCryptSocket(temp, this);
  id=0;
  graph=g;
  connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handshakeIncomingPacketFilter()));
  connect(socket,SIGNAL(connected()),this,SLOT(onConnect()));
  connect(socket, SIGNAL(delayedCloseFinished()), this, SLOT(discardClient()) );
  connect(socket, SIGNAL(connectionClosed()), this, SLOT(discardClient()) );
  connect(socket, SIGNAL(error(int)), this, SLOT(discardClientErr(int)));

}

GSocket::~GSocket()
{
  socket->disconnect();
  socket->deleteLater ();
}


QString GSocket::peerName() const
{
  return peerN;
}

Q_UINT16 GSocket::peerPort()
{
  return peerP;
}

void GSocket::connectIncomingFilter()
{
  connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handshakeIncomingPacketFilter()));
  handshakeIncomingPacketFilter();
}

void GSocket::connectToHost(const QString &ip, Q_UINT16 port)
{
  peerN=ip;
  peerP=port;
  socket->connectToHost(ip,port);
}

void GSocket::close()
{
  socket->close();
}

void GSocket::onConnect()
{
  emit connected();
  writeCryptedLine("GSOCKET");
  disconnect(socket,SIGNAL(readyDecryptedRead()),0,0);
  connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(handshakeOutcomingPacketFilter()));
  handshakeHello();
}

void GSocket::discardClient()
{
  emit connectionClosed();
}
void GSocket::discardClientErr(int)
{
  emit connectionClosed();
}

void GSocket::writeLine(const QString &line)
{
  QString buffer=line.simplifyWhiteSpace();
  if (rePacketRouting.exactMatch(buffer) || reHandShakeACPT.exactMatch(buffer) || reHandShakeHELO.exactMatch(buffer) || reUNKN.exactMatch(buffer) || reWRNG.exactMatch(buffer))
    {
      buffer += '\n';
      socket->writeBlock(buffer.latin1(),buffer.length());
    }
  else
    qDebug("Tentative d'envoi de packet non conforme");
}

void GSocket::writeCryptedLine(const QString &line)
{
  socket->writeCryptedLine(line);
}

QString GSocket::readCryptedLine()
{
  return socket->readCryptedLine();
}

void GSocket::handshakeIncomingPacketFilter()
{
  QString line;


  while (socket->canReadCryptedLine())
    {
      line = socket->readCryptedLine().simplifyWhiteSpace();

      if (reHandShakeHELO.exactMatch(line))
	{
	  QString name=line.section(" ",1,1);
	  qDebug("[gsocket.cpp] - Debug : Reception d'un HELO de %s", name.latin1());

	  QString nodeid=line.section(" ",2,2);
	  bool okt,oks,okc;
	  char con_type,con_speed;
	  unsigned char starcount;
	  Q_UINT16 listenport;
	  QString hostname;

	  con_type=line.section(" ",3,3).toUShort(&okt);
	  con_speed=line.section(" ",4,4).toUShort(&oks);
	  okt = okt && oks;
	  hostname=line.section(" ",5,5);
	  listenport = line.section(" ",6,6).toUShort(&oks);
	  starcount=line.section(" ",7,7).toUShort(&okc);
	  if (okt && oks && okc && (name.length()<16) && (nodeid.length()==NODE_ID_SIZE-1) && (con_type>=1) && (con_type<=5))
	    {
	      if (USHRT_MAX==(id=graph->node_exist(nodeid)))
		{
		  id=graph->Ajouter_Sommet(0,name,nodeid,hostname,listenport,(t_connexion) con_type,con_speed,true,starcount,(GSocket *) this);
		  emit newNode(id);
		}
	      else
		{
		  if (!graph->Ajouter_Arete(0,id))
		    {	//Tentative de double connexion
		      qWarning("Tentative de double connexion");
		      id=0;
		      discardClient();
		      return;
		    }
		  graph->sommets_list[id].socket=(GSocket *) this;
		}
	      disconnect(socket,SIGNAL(readyDecryptedRead()),0,0);
	      connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(packetFilter()));
	      handshakeAccepted();
	      emit(ConnectStar());
	      packetFilter();
	      break;
	    }
	  else
	    writeCryptedLine(RFC_WRNG+" "+line.latin1());
	} else
	  if (reUNKN.exactMatch(line)||reWRNG.exactMatch(line))
	    {
	      qDebug( "[gsocket.cpp] - Debug : Reception de notification de paquet emis invalide : %s", line.latin1());
	    }
	  else
	    {
	      qDebug( "[gsocket.cpp] - Debug : Reception d'un paquet inconnu : %s", line.latin1());
	      writeCryptedLine(RFC_UNKN+" "+line.latin1());
	    }
    }
}

void GSocket::handshakeOutcomingPacketFilter()
{
  while (socket->canReadCryptedLine())
    {
      QString line = socket->readCryptedLine().simplifyWhiteSpace();

      if (reHandShakeACPT.exactMatch(line))
	{
	  QString name=line.section(" ",1,1);
	  QString nodeid=line.section(" ",2,2);
	  bool okt,oks,okc;
	  char con_type,con_speed;
	  QString ip;
	  Q_UINT16 port;
	  unsigned char starcount;

	  con_type=line.section(" ",3,3).toUShort(&okt);
	  con_speed=line.section(" ",4,4).toUShort(&oks);
	  okt = okt && oks;
	  ip=line.section(" ",5,5);
	  port=line.section(" ",6,6).toUShort(&oks);
	  starcount=line.section(" ",7,7).toUShort(&okc);
	  if (okt && oks && okc && (name.length()<16) && (nodeid.length()==NODE_ID_SIZE-1) && (con_type>=1) && (con_type<=5))
	    {
	      if (USHRT_MAX==(id=graph->node_exist(nodeid)))
		{
		  id=graph->Ajouter_Sommet(0,name,nodeid,ip,port,(t_connexion) con_type,con_speed,true,starcount,(GSocket *) this);
		  emit newNode(id);
		}
	      else
		{
		  if (!graph->Ajouter_Arete(0,id))
		    {	//Tentative de double connexion
		      qWarning("Tentative de double connexion");
		      id=0;
		      discardClient();
		      return;
		    }
		  graph->sommets_list[id].socket=(GSocket *) this;
		}
	      disconnect(socket,SIGNAL(readyDecryptedRead()),0,0);
	      connect(socket,SIGNAL(readyDecryptedRead()),this,SLOT(packetFilter()));
	      emit(ConnectStar());
	      packetFilter();
	      break;
	    }
	  else
	    writeCryptedLine(RFC_WRNG+" "+line.latin1());
	} else
	  if (reUNKN.exactMatch(line)||reWRNG.exactMatch(line))
	    {
	      qDebug( "[gsocket.cpp] - Debug : Reception de notification de paquet emis invalide : %s", line.latin1());
	    }
	  else
	    {
	      qDebug( "[gsocket.cpp] - Debug : Reception d'un paquet inconnu : %s", line.latin1());
	      writeCryptedLine(RFC_UNKN+" "+line.latin1());
	    }
    }
}

bool isNode(const QString &node)
{
  QHostAddress ip;
  return true;
  return (node.section(' ',0,0).length()<16) /*Longueur du nom*/
    && (node.section(' ',1,1)==NODE_ID_SIZE-1) /*Longueur du nodeid*/
    && (ip.setAddress(node.section(' ',2,2))) /*IP valide (pas de test sur le port)*/
    && ((node.section(' ',4,4).toUShort()>0) && (node.section(' ',4,4).toUShort()<6)); /*CType valide pas de test sur speed*/
}

bool isNodeList(QStringList nodelist)
{
  for ( QStringList::Iterator it = nodelist.begin(); it != (nodelist.end()); ++it )
    {
      if ( !(isNode(*it)) )
	return false;
    }
  return true;
}


void GSocket::packetFilter()
{
  QStringList list;
  QStringList list2;
  QStringList dest;
  QString line;
  QString ptype;
  QString sender;
  QString chanlist;
  QString nodeslist;

  Q_UINT16 *route_dest;
  Q_UINT16 nb_route_dest;

  while ( !((line = socket->readCryptedLine().simplifyWhiteSpace()).isEmpty()) )
    {
      if (!rePacketRouting.exactMatch(line))
	{
	  if (reUNKN.exactMatch(line)||reWRNG.exactMatch(line))
	    qDebug( "[gsocket.cpp] - Debug : Reception de notification de paquet emis invalide : %s", line.latin1());
	  else
	    writeCryptedLine(RFC_UNKN+" "+line);

	  continue;
	}

      nb_route_dest=0;

      ptype = line.left(4);
      sender = line.mid(5,NODE_ID_SIZE-1);
      dest = QStringList::split(',',line.section(' ',2,2));

      route_dest = (Q_UINT16 *)malloc(dest.count()*sizeof(Q_UINT16));

      for ( QStringList::Iterator it = dest.begin(); it != (dest.end()); ++it )
	{
	  if (*it == graph->sommets_list[0].nodeid)
	    {
	      if (ptype==RFC_STAR)
		{
		  qDebug("[gsocket.cpp] - Debug : Reception d'une etoile : %s",line.latin1());
		  QString data = line.section(' ',3).simplifyWhiteSpace();
		  nodeslist = data.section(';',0,0);
		  chanlist = data.section(';',1);
		  list=QStringList::split(',',nodeslist);
		  list2=QStringList::split(';',chanlist);

		  if ( (list[0].section(' ',1,1)==sender) && isNodeList(list) )
		    {
		      emit(recvStar(list));
		      emit(recvChanStar(graph->NodeidToId(sender),list2));
		    }
		  else
		    {
		      qDebug( "[gsocket.cpp] - Debug : Etoile invalide");
		      writeCryptedLine(RFC_WRNG+' '+line.latin1());
		    }
		}
	      else if (ptype==RFC_QUIT)
                {
                  qDebug( "[gsocket.cpp] - Debug : Reception d'une notification de depart %s", line.latin1());
                  close();
                }
              else if (ptype==RFC_CUEV)
                {
                  QString name = line.section(' ',3,3);
                  QString connection = line.section(' ',4,4);

                  if ((!name.isEmpty())&&(!connection.isEmpty())&&((connection=="0")||(connection=="1")))
                    if (connection=="0")
                      emit(recvChatUserEvent(graph->NodeidToId(sender), name, false ));
                    else
                      emit(recvChatUserEvent(graph->NodeidToId(sender), name, true));
                  else
                    writeCryptedLine(RFC_WRNG+" "+line.latin1());

                }
              else if (ptype==RFC_CMSG)
                {
                  QString channame = line.section(' ',3,3);
                  QString msg = line.section(' ',4);

                  if ((!channame.isEmpty())&&(!msg.isEmpty()))
                    emit(recvChatMsg(graph->sommets_list[graph->NodeidToId(sender)].name, channame, msg));
                  else
                    writeCryptedLine(RFC_WRNG+" "+line.latin1());

                }
              else if (ptype==RFC_SRCH)
                {
                  QString data = line.section(' ',3);
                  emit recvSearchByName(graph->NodeidToId(sender),data.mid(2),QString(data[0]).toULong());
                }
              else if (ptype==RFC_RSRH)
                {
                  QString reslst,s=line.section(' ',3)+';';
                  reslst=s;
                  s=s.section('/',0,0);
                  reslst = reslst.section('/',1,1);
                  emit recvSearchByNameAns(graph->NodeidToId(sender),s,QStringList::split(';',reslst));
                }
              else if (ptype==RFC_HSRH)
                {
                  emit recvSearchByHash(graph->NodeidToId(sender),line.section(' ',3,3),line.section(' ',4,4).toULongLong());
                }
              else if (ptype==RFC_HRSR)
                {
                  emit recvSearchByHashAns(graph->NodeidToId(sender),line.section(' ',3,3),line.section(' ',4,4).toULongLong(),line.section(' ',5));
                }
              else if (ptype==RFC_CMPV)
                {
                  QString msg = line.section(' ',3);

                  if (!msg.isEmpty())
                    {
                      emit (recvChatPv(graph->NodeidToId(sender),graph->sommets_list[graph->NodeidToId(sender)].name,msg));
                    }
                  else
                    writeCryptedLine(RFC_WRNG+" "+line.latin1());
                }
              else if (ptype==RFC_FLST)
                {
                  emit recvFileList(graph->NodeidToId(sender));
                }
              else if (ptype==RFC_RFLS)
                {
                  QString data = line.section(' ',3).simplifyWhiteSpace();
                  qDebug(data);
                  emit recvFileListHash(graph->NodeidToId(sender),data.section(' ',0,0).toULongLong(),data.section(' ',-1));
                }
              else if (ptype==RFC_ACON)
                {
                  emit recvAskConnect(graph->NodeidToId(sender));
                }
              else if (ptype==RFC_WKEY)
                {
                  emit recvKeyRequest(graph->NodeidToId(sender));
                }
              else if (ptype==RFC_GKEY)
                {
                  QString key = line.section(' ',3);
                  emit recvGiveKey(sender,key);
                }
              else if ((ptype==RFC_WRNG)||(ptype==RFC_UNKN))
                {
                  qDebug( "[gsocket.cpp] - Debug : Reception d'une notification de paquet invalide : %s", line.latin1());
                }
              else /* Packet non identifie */
                {
                  qDebug( "[gsocket.cpp] - Debug : Reception d'un paquet inconnu : %s", line.latin1());
                  writeCryptedLine(RFC_UNKN+" "+line.latin1());
                }
	    }
	  else
	    {
	      //Routage
	      if (graph->node_exist(*it)<USHRT_MAX)
		{
		  route_dest[nb_route_dest]=graph->NodeidToId(*it);
		  nb_route_dest++;

		}
	    }
	}
      if (nb_route_dest>0)
	emit(routePacket(ptype, sender, route_dest, nb_route_dest,line.section(' ',3).simplifyWhiteSpace() ));
      free(route_dest);
    }



}



void GSocket::handshakeHello()
{
  QString hand = RFC_HELO;

  QString con_speed;
  con_speed.setNum(graph->sommets_list[0].ConnexionSpeed);

  hand+= " ";
  hand+= graph->sommets_list[0].name;
  hand+= " ";
  hand+= graph->sommets_list[0].nodeid;
  hand+= " ";
  hand+= (char) ((int) graph->sommets_list[0].ConnexionType +(int)'0');
  hand+= " ";
  hand+= con_speed;
  hand+= " ";
  hand+= graph->sommets_list[0].host;
  hand+= " ";
  hand+= QString::number(graph->sommets_list[0].port);
  hand+= " ";
  hand+= QString::number(graph->sommets_list[0].starcount);

  writeCryptedLine(hand);
}

void GSocket::handshakeAccepted()
{
  QString hand = RFC_ACPT;

  QString con_speed;
  con_speed.setNum(graph->sommets_list[0].ConnexionSpeed);

  hand+= " ";
  hand+= graph->sommets_list[0].name;
  hand+= " ";
  hand+= graph->sommets_list[0].nodeid;
  hand+= " ";
  hand+= (char) (((int) graph->sommets_list[0].ConnexionType) + ((int) '0'));
  hand+= " ";
  hand+= con_speed;
  hand+= " ";
  hand+= graph->sommets_list[0].host;
  hand+= " ";
  hand+= QString::number(graph->sommets_list[0].port);
  hand+= " ";
  hand+= QString::number(graph->sommets_list[0].starcount);

  writeCryptedLine(hand);
}
