#include <qserversocket.h>
#include <qvaluevector.h>
#include <qsocket.h>
#include <qfile.h>
#include <qdir.h>
#include <qstringlist.h>
#include <qregexp.h>
#include <qbitarray.h>
#include "../network/gnetwork.h"
#include "../graphes.h"
#include "../files/gfileconst.h"
#include "gwebserver.h"
#include "base64.h"

#define LOGIN "admin"

extern GNetwork *net;
extern c_graphes *graphe;
extern QString applicationdirpath;

GWebServer::GWebServer(const QString password, unsigned short port, QObject* parent)
  : QServerSocket(port,1,parent)
{
  if ( !ok() )
    {
      qDebug("GWebServer : Erreur : Impossible d'ecouter sur le port %d", port);
    }
  else
    {
      qDebug("GWebServer : Serveur Web en ecoute sur le port %d", port);
    }
  connect(net,SIGNAL(connected(const QString &, Q_UINT16)),this,SLOT(Connexion(const QString &, Q_UINT16)));
  connect(net,SIGNAL(disconnected(const QString &, Q_UINT16)),this,SLOT(Deconnexion(const QString &, Q_UINT16)));
  connect(net,SIGNAL(recvSearchResult(const QString &,const QString &,const QString &,const QString &, unsigned int, const QString &)),this,SLOT(SearchResult(const QString &,const QString &,const QString &,const QString &, unsigned int, const QString &)));

  connect(net->downloader,SIGNAL(tempLoaded(const QString &,Q_UINT64,const QString &,QBitArray &)),this,SLOT(NewDL(const QString &,Q_UINT64,const QString &,QBitArray &)));

  connect(net->downloader,SIGNAL(newPart(Q_UINT64,const QString &,Q_UINT32,unsigned char)),this,SLOT(UpdateDL(Q_UINT64,const QString &,Q_UINT32,unsigned char)));

  connect(net->downloader,SIGNAL(finished(Q_UINT64,const QString &)),this, SLOT(DLFinished(Q_UINT64,const QString &)));

  connect(net->uploader,SIGNAL(askingFile(Q_UINT64, const QString &, const QString &, const QString &)),this,SLOT(NewUL(Q_UINT64, const QString &, const QString &, const QString &)));
  connect(net->uploader,SIGNAL(Transferring(Q_UINT64, const QString &, const QString &, unsigned long int)),this,SLOT(UpdateUL(Q_UINT64, const QString &, const QString &, unsigned long int)));
  connect(net->uploader,SIGNAL(connectionLost(const QString &, Q_UINT64, const QString &)),this,SLOT(ULFinished(const QString &, Q_UINT64 , const QString &)));
  passwd = password;
}

void GWebServer::SearchResult(const QString &srch,const QString &file,const QString &size,const QString &hash, unsigned int, const QString &)
{
  unsigned int i;
  bool added=false;
  s_result temp;

  if (srch==lastsearch)
    {
      for (i=0;i<results.count();i++)
        {
          if (hash==results[i].hash)
            {
              results[i].sources++;
              added=true;
            }
        }
      if (!added)
        {
          temp.file = file;
          temp.size = size;
          temp.sources = 1;
          temp.hash = QDeepCopy<QString>(hash);
          temp.launched = false;
          results.append(temp);
        }
    }
}

void GWebServer::newConnection( int socket )
{
  QSocket* s = new QSocket( this );
  connect( s, SIGNAL(readyRead()), this, SLOT(readClient()) );
  connect( s, SIGNAL(delayedCloseFinished()), this, SLOT(discardClient()) );
  s->setSocket( socket );
  emit newConnect();
}

void GWebServer::readClient()
{
  QSocket* socket = (QSocket*)sender();

  while	 ( socket->canReadLine() )
    {
      request.append(socket->readLine());
      if (request.last()=="\r\n")
        {
          parseRequest(socket);
          break;
        }
    }
}

void GWebServer::Connexion(const QString &IPAddr, Q_UINT16 Port)
{
  connectednodes.append(IPAddr+':'+QString::number( Port));
}
void GWebServer::Deconnexion(const QString &IPAddr, Q_UINT16 Port)
{
  connectednodes.remove(IPAddr+':'+QString::number( Port));
}

void GWebServer::NewDL(const QString &filename,Q_UINT64 filesize,const QString &hash, QBitArray &part)
{
  unsigned int i=0;
  Q_UINT64 res=0;
  s_dl temp;

  if (part.size()>2)
    {
      while(i<=part.size()-2)
        {
          res=res+part[int(i)];
          i++;
        }
    }
  res=res*PART_SIZE;
  if (part[int(part.size()-1)])
    {
      if (filesize % PART_SIZE)
	res=res+(filesize % PART_SIZE);
      else
	res=res+PART_SIZE;

    }

  temp.filename = filename;
  temp.avancement = res*100/filesize;
  temp.taille = res;
  temp.tailletotale = filesize;
  temp.hash = hash;
  downloads.append(temp);

}

void GWebServer::UpdateDL(Q_UINT64 filesize,const QString &hash,Q_UINT32 chunk,unsigned char part)
{
  unsigned int i;

  for (i=0;i<downloads.count();i++)
    {
      if (downloads[i].hash==hash && downloads[i].tailletotale==filesize)
        {

          if (chunk*CHUNK_SIZE+(part+1)*PART_SIZE<=filesize)
	    downloads[i].taille=downloads[i].taille+(PART_SIZE);
          else
            downloads[i].taille=downloads[i].taille+(filesize%PART_SIZE);

          downloads[i].avancement=downloads[i].taille*100/(filesize);

        }
    }


}

void GWebServer::DLFinished(Q_UINT64 filesize,const QString &hash)
{
  unsigned int i;
  for (i=0;i<downloads.count();i++)
    {
      if (downloads[i].hash==hash && downloads[i].tailletotale==filesize)
        {
          downloads.erase(downloads.begin()+i);
        }
    }
}

void GWebServer::NewUL(Q_UINT64 fsize, const QString &hash, const QString &fname, const QString &nodename)

{
  s_ul temp;

  temp.file = fname;
  temp.hash = hash;
  temp.user = nodename;
  temp.size = QString::number(fsize);
  temp.speed = 0;
  uploads.append(temp);
}

void GWebServer::UpdateUL(Q_UINT64 fsize, const QString &hash, const QString &nodename, unsigned long int speed)
{
  unsigned int i;

  for (i=0;i<uploads.count();i++)
    {
      if (uploads[i].size==QString::number(fsize) && uploads[i].hash==hash && uploads[i].user==nodename)
        {
          uploads[i].speed=speed;
        }
    }

}


void GWebServer::ULFinished(const QString &hash, Q_UINT64 fsize, const QString &nodename)
{
   unsigned int i;

   for (i=0;i<uploads.count();i++)
     {
       if (uploads[i].size==QString::number(fsize) && uploads[i].hash==hash && uploads[i].user==nodename)
         {
           uploads.erase(uploads.begin()+i);
         }
     }
}


void GWebServer::request404Error(QSocket *socket)
{
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );
  os << "HTTP/1.1 404 Not Found\r\n"
    "Content-Type: text/html; charset=\"utf-8\"\r\n"
    "\r\n"
    "<h1>404 Error - Page not found</h1>\n";
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestDownloadingPage(QSocket *socket)
{
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );
  os << "HTTP/1.1 404 Not Found\r\n"
    "Content-Type: text/html; charset=\"utf-8\"\r\n"
    "\r\n"
    "<HEAD><META HTTP-EQUIV=\"Refresh\"CONTENT=\"1; URL=res.htm\"></HEAD>";
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestIndexPage(QSocket *socket)
{
  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Racine recue");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Index -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Index -->")))
    {
      os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestCssStyle(QSocket *socket)
{
  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET CSS Style");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- CSS -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- CSS -->")))
    {
      os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();

}

void GWebServer::requestMenuPage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Menu");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Menu -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Menu -->")))
    {
      os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();

}

void GWebServer::requestContactListPage(QSocket *socket)
{
  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET ContactList");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Contact List -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Contact List -->")))
    {
      os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestContactListTablePage(QSocket *socket)
{
  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;
  unsigned int i;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET ContactListTable");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Contact List Table -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Contact List Table -->")))
    {
      if (line.contains("##CLIST##"))
        {
          for (i=0;i<graphe->nodes().count();i++)
            {
              if (graphe->nodes()[i].arete!=NULL)
                {
                  os << "<tr>\n";
                  os << "<td>" << graphe->nodes()[i].name <<"</td>\n";
                  os << "<td>" << graphe->nodes()[i].host <<"</td>\n";
                  os << "<td>"<< graphe->nodes()[i].port <<"</td>\n";

                  os << "</tr>\n";
                }
            }
        }
      else
        os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestConnectionPage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;
  unsigned int i;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Connexion");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Connections -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Connections -->")))
    {

      if (line.contains("##KNOWNNODES##"))
        {
          qDebug("Generation de la liste de nodes connus");
          QStringList listeip;
          QStringList listport;
          listeip=gconf->ipList();
          listport=gconf->portList();
          QStringList::Iterator it2 = listport.begin();
          for (QStringList::Iterator it = listeip.begin(); it != listeip.end(); ++it )
            {
              os << "<option>" << (*it) << ':' << (*it2) << "</option>\n";
              ++it2;
            }
        }
      else if (line.contains("##CONNNODES##"))
        {
          qDebug("Generation de la liste de nodes connectes - nbelements : %d",connectednodes.count());
          for (i=0;i<connectednodes.count();i++)
            {
              os << "<option>" << connectednodes[i] << "</option>\n";
            }
        }
      else
        os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestTransfertsPage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  unsigned int i;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Transferts");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Transferts -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Transferts -->")))
    {

      if (line.contains("##ULLIST##"))
        {
          for (i=0;i<uploads.count();i++)
            {
              os << "<tr>\n";
              os << "<td>" << uploads[i].user << "</td>\n";
              os << "<td>" << uploads[i].file << "</td>\n";
              os << "<td>" << uploads[i].speed/1024 << "</td>\n";
              os << "</tr>\n";
            }
        }
      else
      if (line.contains("##DLLIST##"))
        {
          for (i=0;i<downloads.count();i++)
            {
              os << "<tr>\n";
              os << "<td>" << downloads[i].filename << "</td>\n";
              os << "<td>" << QString::number(downloads[i].avancement) << " %</td>\n";
              os << "<td>" << QString::number(downloads[i].taille/1024) << "</td>\n";
              os << "<td>" << QString::number(downloads[i].tailletotale/1024) << "</td>\n";
              os << "</tr>\n";
            }

        }
      else
        os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestRecherchePage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Recherche");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Recherche -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  while ((!htm_template->atEnd())&&(!line.contains("</-- Recherche -->")))
    {
      os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}

void GWebServer::requestResultatsPage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;
  unsigned int i;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Resultats");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Resultats -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }
  qDebug("Affichage des resultats : %d",results.count());
  while ((!htm_template->atEnd())&&(!line.contains("</-- Resultats -->")))
    {
      if (line.contains("##RESULTS##"))
        {
          qDebug("RESULTS trouve");
          for (i=0;i<results.count();i++)
            {
              qDebug("%s",results[i].file.latin1());
              os << "<tr>\n";
              os << "<td height=\"28\" width=\"50%\"><a href=\"download.cgi?id=" << i << "\">" << results[i].file << "</a></td>\n";
              os << "<td width=\"23%\">" << results[i].size << "</td>\n";
              os << "<td width=\"4%\">" << results[i].sources << "</td>\n";
              os << "<td width=\"23%\">" << results[i].hash << "</td>\n";
              os << "</tr>\n";

            }
        }
      else
        os << line;
      htm_template->readLine(line,42000);
    }
  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}


void GWebServer::requestParametresPage(QSocket *socket)
{

  QFile *htm_template = new QFile(gconf->applicationDirPath()+"Gloster.tmpl");
  QString line;
  bool temp;

  if (!htm_template->open( IO_ReadOnly))
    {
      request404Error(socket);
      return;
    }

  qDebug("Requete GET Parametres");
  QTextStream os( socket );
  os.setEncoding( QTextStream::UnicodeUTF8 );

  temp=true;
  while ((!(htm_template->atEnd()))&&(temp))
    {
      htm_template->readLine(line,42000);
      if (line.contains("<-- Parametres -->"))
        {
          temp = false;
        }
    }
  if (!htm_template->atEnd())
    {
      htm_template->readLine(line,42000);
    }

  while ((!htm_template->atEnd())&&(!line.contains("</-- Parametres -->")))
    {
      line.replace("##PSEUDO##", gconf->nickname() );
      line.replace("##NETKEY##", gconf->netKey() );
//      if (settings.readBoolEntry("/connexion/routage"))
//        line.replace("##ROUTE##", "CHECKED" );
//      else
        line.replace("##ROUTE##", "" );
      if (gconf->autoCon())
        line.replace("##AUTOCONNECT##", "CHECKED" );
      else
        line.replace("##AUTOCONNECT##", "" );
      line.replace("##NBCONN##", QString::number(gconf->keepCon()) );
	switch (gconf->conType())
	{
	case 1:
          line.replace("##CT_LS##", "selected" );
          line.replace("##CT_XDSL##", "" );
          line.replace("##CT_ISDN##", "" );
          line.replace("##CT_RTC##", "" );
          line.replace("##CT_SAT##", "" );
          break;
	case 2:
          line.replace("##CT_LS##", "" );
          line.replace("##CT_XDSL##", "selected" );
          line.replace("##CT_ISDN##", "" );
          line.replace("##CT_RTC##", "" );
          line.replace("##CT_SAT##", "" );
          break;
	case 3:
          line.replace("##CT_LS##", "" );
          line.replace("##CT_XDSL##", "" );
          line.replace("##CT_ISDN##", "selected" );
          line.replace("##CT_RTC##", "" );
          line.replace("##CT_SAT##", "" );
          break;
	case 4:
          line.replace("##CT_LS##", "" );
          line.replace("##CT_XDSL##", "" );
          line.replace("##CT_ISDN##", "" );
          line.replace("##CT_RTC##", "selected" );
          line.replace("##CT_SAT##", "" );
          break;
	case 5:
          line.replace("##CT_LS##", "" );
          line.replace("##CT_XDSL##", "" );
          line.replace("##CT_ISDN##", "" );
          line.replace("##CT_RTC##", "" );
          line.replace("##CT_SAT##", "selected" );
          break;
	}


      line.replace("##NBSLOTS##", QString::number(gconf->slotCount()));
      line.replace("##UPLSPEED##", QString::number(gconf->conSpeed()));
      line.replace("##LISTENPORT##", QString::number(gconf->listenPort()) );
      line.replace("##PUBLICIP##", gconf->publicIp());
      line.replace("##TEMPDIR##", gconf->tempDir());
      line.replace("##INCOMINGDIR##", gconf->downloadDir());

      if (gconf->webActivated())
        line.replace("##SRVWEB##", "CHECKED" );
      else
        line.replace("##SRVWEB##", "" );

      line.replace("##WEBPORT##", QString::number(gconf->webListenPort()));
      line.replace("##WEBPASS##", gconf->webPassword());
      os << line;
      htm_template->readLine(line,42000);
    }

  delete htm_template;
  socket->close();
  request.clear();
  emit wroteToClient();
}


void GWebServer::parseRequest(QSocket *socket)
{
  unsigned int i;
  bool authenticated=false;

  QStringList tokens = QStringList::split( QRegExp("[ \r\n][ \r\n]*"), request[0] );
  if (( tokens[0] == "GET" )||(tokens[0] == "POST"))
    {
      for (i=1;i<request.count();i++)
        {
          if (request[i].left(14)=="Authorization:")
            {
              qDebug("Authentification : Mot de passe recu");
              QString auth_string = base64_decode(request[i].right(request[i].length()-request[i].find(':')).simplifyWhiteSpace().right(request[i].length()-request[i].find(' ')).simplifyWhiteSpace().section(' ',2,2));
              QString login = auth_string.section(':',0,0);
              QString password = auth_string.section(':',1,1);
              if ((login==LOGIN)&&(password==passwd))
                authenticated=true;
              break;
            }
        }
      if (authenticated)
        {
          if (tokens[1]=="/" || tokens[1]=="/index.htm")
            {
              requestIndexPage(socket);
            }
          else if (tokens[1]=="/menu.htm")
            {
              requestMenuPage(socket);
            }
          else if (tokens[1]=="/style.css")
            {
              requestCssStyle(socket);
            }
          else if (tokens[1]=="/contact_list.htm")
            {
              requestContactListPage(socket);
            }
          else if (tokens[1].left(11)=="/connec.htm")
            {
              if (tokens[1].length()>11)
                {
                  qDebug("Donnees de formulaire sur page de connexion");
                  parseConnexionForm(tokens[1].section('?',1,1));
                }
              requestConnectionPage(socket);
            }
          else if (tokens[1]=="/transferts.htm")
            {
              requestTransfertsPage(socket);
            }
          else if (tokens[1].left(11)=="/search.htm")
            {
              if (tokens[1].length()>11)
                {
                  qDebug("Donnees de formulaire sur page de recherche");
                  parseSearchForm(tokens[1].section('?',1,1));
                }
              requestRecherchePage(socket);
            }
          else if (tokens[1].left(10)=="/param.htm")
            {
              if (tokens[1].length()>10)
                {
                  qDebug("Donnees de formulaire sur page de connexion parametres");
                  parseSettingsForm(tokens[1].section('?',1,1));

                }
              requestParametresPage(socket);
            }
          else if (tokens[1]=="/list.htm")
            {
              requestContactListTablePage(socket);
            }
          else if (tokens[1]=="/res.htm")
            {
              requestResultatsPage(socket);
            }
          else if (tokens[1].left(13)=="/download.cgi")
            {
              if (tokens[1].length()>13)
                {
                  qDebug("Donnees de demande de telechargement");
                  parseDownloadRequest(tokens[1].section('?',1,1));

                }
              requestDownloadingPage(socket);
            }
          else if (tokens[1].left(8)=="/images/")
            {
              qDebug("Requete GET Image recue : %s",tokens[1].right(tokens[1].length()-8).latin1());
              QFile *fd = new QFile("images/"+tokens[1].right(tokens[1].length()-8));

              if (fd->open( IO_ReadOnly))
                {
                  QTextStream os( socket );
                  os.setEncoding( QTextStream::Latin1 );
                  os << "HTTP/1.1 200 OK\r\n"
                    "Content-length:"<< fd->size() <<"\r\n"
                    "Content-type: image/gif\r\n"
                    "\r\n";
                  while (!fd->atEnd())
                    {
                      os << (char)fd->getch();
                    }
                  socket->close();
                  request.clear();
                  emit wroteToClient();
                }
              else
                {
                  request404Error(socket);
                }
              delete fd;
            }
          else
            {
              request404Error(socket);
            }
        }
      else
        {
          QTextStream os( socket );
          os.setEncoding( QTextStream::Latin1 );
          os << "HTTP/1.1 401 Authorization Required\r\n"
            "WWW-Authenticate: Basic realm=\"Administration distante Gloster\"\r\n"
            "\r\n"
            "<h1>401 Error - Authorization Required</h1>\n";
          socket->close();
          request.clear();
          emit wroteToClient();
        }
    }
  else
    {
      socket->close();
      request.clear();
      emit wroteToClient();
    }
}

void GWebServer::discardClient()
{
  QSocket* socket = (QSocket*)sender();
  delete socket;
  emit endConnect();
}


const QString GWebServer::decode_form_string(const QString in)
{
  QString res="";
  unsigned int i=0,t;

  while (i<in.length())
    {
      if (in[i]=='%')
        {
          if (i+2<in.length())
            {
              i++;
              sscanf(in.mid(i,2).latin1(),"%02X",&t);
              res.append((char)t);
            }
          else
            {
              res.append( in.right(in.length()-i) );
            }
          i++;
        }
      else
        {
          res.append(in[i]);
        }
      i++;
    }

  return res;
}

void GWebServer::parseDownloadRequest(const QString params)
{
  qDebug("Ligne de parametres : %s",params.latin1());
  QStringList fields = QStringList::split( '&', params );
  unsigned int id=results.count();
  unsigned int i;

  for (i=0;i<fields.count();i++)
    {
      QString line = decode_form_string(fields[i]);
      qDebug("Parametre %d : %s",i,line.latin1());

      if (line.left(3)=="id=")
        id = line.right(line.length()-3).toUInt();
    }

  if (id<results.count() && !results[i].launched)
    {
      qDebug("TELECHARGEMENT DU FICHIER %d DE LA RECHERCHE",id);
      net->downloader->download(results[id].file,results[id].size.toUInt(),results[id].hash);
      results[id].launched = true;
    }

}

void GWebServer::parseConnexionForm(const QString params)
{
  qDebug("Ligne de parametres : %s",params.latin1());
  QStringList fields = QStringList::split( '&', params );
  QString ip="";
  QString port="";
  unsigned int i;

  for (i=0;i<fields.count();i++)
    {
      QString line = decode_form_string(fields[i]);
      qDebug("Parametre %d : %s",i,line.latin1());

      if (line.left(3)=="ip=")
        ip = line.right(line.length()-3);
      if (line.left(5)=="port=")
        port = line.right(line.length()-5);
    }
  net->connecttohost(ip,port.toUInt());
}

void GWebServer::parseSearchForm(const QString params)
{
  unsigned int type;
  unsigned int i;
  QString name="";

  lastsearch = "";
  qDebug("Ligne de parametres : %s",params.latin1());
  QStringList fields = QStringList::split( '&', params );
  for (i=0;i<fields.count();i++)
    {
      QString line = decode_form_string(fields[i]);
      if (line.left(3)=="ip=")
        name = line.right(line.length()-3);
      if (line.left(12)=="type=Archive")
        type = 1;
      if (line.left(10)=="type=Audio")
        type = 2;
      if (line.left(13)=="type=Image CD")
        type = 3;
      if (line.left(16)=="type=Photo/Image")
        type = 4;
      if (line.left(15)=="type=Programmes")
        type = 5;
      if (line.left(14)=="type=Documents")
        type = 6;
      if (line.left(10)=="type=Video")
        type = 7;
      if (line.left(9)=="type=Tous")
        type = 8;
    }
  results.clear();
  lastsearch = name;
  net->sendSearchFile(name, type);

}

void GWebServer::parseSettingsForm(const QString params)
{
  unsigned int i;
  bool autoconn = false;
  bool servweb = false;
  QStringList fields = QStringList::split( '&', params );
  qDebug("Ligne de parametres : %s",params.latin1());


  for (i=0;i<fields.count();i++)
    {
      QString line = decode_form_string(fields[i]);
      qDebug("Parametre %d : %s",i,line.latin1());

      if (line.left(7)=="pseudo=")
        gconf->setNickname(line.right(line.length()-7));
      if (line.left(7)=="netkey=")
        gconf->setNetKey(line.right(line.length()-7));


      if (line.left(7)=="type=LS")
        gconf->setConType(1);
      if (line.left(15)=="type=xDSL/Cable")
        gconf->setConType(2);
      if (line.left(9)=="type=ISDN")
        gconf->setConType(3);
      if (line.left(8)=="type=RTC")
        gconf->setConType(4);
      if (line.left(14)=="type=Satellite")
        gconf->setConType(5);

      if (line.left(7)=="nbconn=")
        gconf->setKeepCon(line.right(line.length()-7).toInt());
      if (line.left(8)=="nbslots=")
        gconf->setSlotCount(line.right(line.length()-8).toInt());
      if (line.left(3)=="ul=")
        gconf->setConSpeed(line.right(line.length()-3).toInt());
      if (line.left(5)=="port=")
        gconf->setListenPort(line.right(line.length()-5).toInt());
      if (line.left(3)=="ip=")
        gconf->setPublicIp(line.right(line.length()-3));
      if (line.left(5)=="temp=")
        gconf->setTempDir(line.right(line.length()-5));
      if (line.left(9)=="incoming=")
        gconf->setDownloadDir(line.right(line.length()-9));
      if (line.left(6)=="portw=")
        gconf->setWebListenPort(line.right(line.length()-6).toInt());
      if (line.left(6)=="passw=")
        gconf->setWebPassword(line.right(line.length()-6));
      if (line.left(14)=="autoconnect=on")
        autoconn = true;
      if (line.left(9)=="srvweb=on")
        servweb = true;

    }
  gconf->setAutoCon(autoconn);
  gconf->setWebActivated(servweb);
}
