/***************************************************************************
 *            gwebserver.h
 *
 *  Sun Jan  2 18:39:43 2005
 *  Copyright  2005  Francois Goudal
 *  francois@goudal.net
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef _GWEBSERVER_H
#define _GWEBSERVER_H

#include <qserversocket.h>
#include <qstringlist.h>
#include <qdeepcopy.h>
#include <qvaluevector.h>
#include <qsocket.h>
#include <qbitarray.h>

class GConfig;

extern GConfig * gconf;

typedef struct
{
  QString file;
  QString size;
  unsigned int sources;
  QString hash;
  bool launched;
} s_result;

typedef struct
{
  QString filename;
  Q_UINT64 avancement;
  Q_UINT64 taille;
  Q_UINT64 tailletotale;
  QString hash;
} s_dl;

typedef struct
{
  QString file;
  QString user;
  QString size;
  unsigned long int speed;
  QString hash;
} s_ul;

class GWebServer : public QServerSocket
{
  Q_OBJECT
    public:
  GWebServer(const QString password, unsigned short port = 8080,  QObject* parent=0 );
  void newConnection( int socket );

 signals:
  void newConnect();
  void endConnect();
  void wroteToClient();

 private slots:
  void Connexion(const QString &IPAddr, Q_UINT16 Port);
  void Deconnexion(const QString &IPAddr, Q_UINT16 Port);
  void SearchResult(const QString &,const QString &,const QString &,const QString &, unsigned int, const QString &);
  void NewDL(const QString &filename,Q_UINT64 filesize,const QString &hash,QBitArray &part);

  void UpdateDL(Q_UINT64 filesize,const QString &fname,Q_UINT32 chunk,unsigned char part);

  void DLFinished(Q_UINT64 filesize,const QString &hash);

  void NewUL(Q_UINT64 fsize, const QString &hash, const QString &fname, const QString &nodename);
  void UpdateUL(Q_UINT64 fsize, const QString &hash, const QString &nodename, unsigned long int speed);
  void ULFinished(const QString &hash, Q_UINT64 fsize, const QString &nodename);



  void readClient();
  void discardClient();

 private:
  QStringList request;
  QStringList connectednodes;
  QValueVector<s_result> results;
  QValueVector<s_ul> uploads;
  QValueVector<s_dl> downloads;
  QString lastsearch;
  void parseRequest(QSocket *);
  void request404Error(QSocket *);
  void requestIndexPage(QSocket *);
  void requestMenuPage(QSocket *);
  void requestCssStyle(QSocket *);
  void requestContactListPage(QSocket *);
  void requestConnectionPage(QSocket *);
  void requestTransfertsPage(QSocket *);
  void requestRecherchePage(QSocket *);
  void requestParametresPage(QSocket *);
  void requestContactListTablePage(QSocket *);
  void requestResultatsPage(QSocket *);
  void requestDownloadingPage(QSocket *);
  const QString decode_form_string(const QString);
  void parseDownloadRequest(const QString);
  void parseConnexionForm(const QString);
  void parseSearchForm(const QString);
  void parseSettingsForm(const QString);
  QString passwd;
};

#endif
